﻿using Syncfusion.DocIO.DLS;
using Syncfusion.Drawing;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;
using HorizontalAlignment = Syncfusion.DocIO.DLS.HorizontalAlignment;
using VerticalAlignment = Syncfusion.DocIO.DLS.VerticalAlignment;
using SkiaSharp;
using Vinno.vCloud.ReportGenerator;
using System.Collections.Generic;
using System.Linq;
using System;
using System.IO;

namespace ReportService
{
    public static class TableGenerater
    {
        private static float _widthZoomRatio;

        /// <summary>
        /// Generate Table
        /// </summary>
        public static void Generate(ReportInfo reportInfo, RTTable table, WTextBody wordTextBody, float widthZoomRatio = 1, bool isTopTable = false)
        {
            _widthZoomRatio = widthZoomRatio;
            var validDictionary = table.Cells
                .SelectMany(x => x.Value.Blocks)
                .OfType<RTTable>()
                .ToDictionary(x => x, x => x.IsVisibleIn(reportInfo));
            var cellByRowGroups = table.Cells
                .GroupBy(x => x.Key.Row, x => x.Value)
                .OrderByDescending(x => x.Key)
                .ToArray();
            var document = table.GetReportTemplateDocument();
            var isInHeader = wordTextBody.IsInHeader();
            foreach (var cellByRowGroup in cellByRowGroups)
            {
                var cellBlocks = cellByRowGroup.SelectMany(x => x.Blocks).ToArray();
                if (cellBlocks.Length > 0 && cellBlocks.All(x => x is RTTable cellTable && !validDictionary[cellTable]))
                {
                    var rowIndex = cellByRowGroup.Key;
                    if (document.HeaderHeight != null && isInHeader)
                        document.HeaderHeight -= (float)table.RowDefinitions[rowIndex].Height;

                    table.DeleteSelectedRow(rowIndex);
                }
            }
            if (isTopTable)
            {
                FindChildRTTable(table, reportInfo, "", 0);
                if (table.RowDefinitions?.Count <= 0)
                {
                    return;
                }
            }
            if (table.RowDefinitions.Count > 0 && table.ColumnDefinitions.Count > 0)
            {
                var wordTable = wordTextBody.AddTable();
                InitializeTable(table, wordTable);
                var isLastChild = table == (table.Parent as IReportTemplateBody)?.Blocks?.Last();
                if (!isLastChild)
                {
                    wordTable.GetStyle().ParagraphFormat.KeepFollow = true;
                    wordTable.GetStyle().ParagraphFormat.Keep = true;
                }

                wordTable.TableFormat.IsBreakAcrossPages = table.AllowBreakAcrossPages;
                wordTable.TableFormat.Borders.BorderType = BorderStyle.None;
                SetTableAlignment(table.HorizontalAlignment, wordTable);
                foreach (var cell in table.Cells)
                {
                    MergeCell(wordTable, cell.Key, table);
                    var wordCell = wordTable.Rows[cell.Key.Row].Cells[cell.Key.Column];
                    SetTableCell(reportInfo, wordCell, cell.Value, validDictionary);
                }

                //table keepfollow workaround
                if (!isLastChild)
                {
                    var paragraph = wordTextBody.AddParagraph();
                    paragraph.BreakCharacterFormat.FontSize = 0;
                }
            }
        }

        /// <summary>
        /// 处理所有table数据
        /// </summary>
        /// <param name="table"></param>
        /// <param name="reportInfo"></param>
        /// <param name="cellId"></param>
        /// <returns></returns>
        private static Dictionary<string, double> FindChildRTTable(RTTable table, ReportInfo reportInfo, string cellId, double colWidth)
        {
            var dicRowDiffHeight = new Dictionary<string, double>();
            foreach (var cellItem in table.Cells)
            {
                foreach (var child in cellItem.Value.Blocks)
                {
                    if (child is RTTable rtTable)
                    {
                        double columnWidth = 0;
                        if (cellItem.Key.ColumnSpan > 1)
                        {
                            for (int i = 0; i < cellItem.Key.ColumnSpan; i++)
                            {
                                var columnIndex = cellItem.Key.Column + i;
                                columnWidth += table.ColumnDefinitions[columnIndex].Width;
                            }
                        }
                        else
                        {
                            columnWidth = table.ColumnDefinitions[cellItem.Key.Column].Width;
                        }
                        var tempDic = FindChildRTTable(rtTable, reportInfo, cellItem.Value.Id, columnWidth);
                        if (tempDic?.Count > 0)
                        {
                            dicRowDiffHeight.Add(tempDic.Keys.FirstOrDefault(), tempDic.Values.FirstOrDefault());
                        }
                    }
                }
            }
            var dic = ResetParentRowDefinitions(table, reportInfo, cellId, colWidth);
            if (dicRowDiffHeight?.Count > 0)
            {
                var dicDiff = new Dictionary<int, double>();
                foreach (var item in dicRowDiffHeight.Keys)
                {
                    var cellInfo = table.Cells.FirstOrDefault(c => c.Value.Id == item);
                    if (!dicDiff.Keys.Contains(cellInfo.Key.Row))
                    {
                        dicDiff.Add(cellInfo.Key.Row, dicRowDiffHeight[item]);
                    }
                    else
                    {
                        if (dicDiff[cellInfo.Key.Row] > dicRowDiffHeight[item])
                        {
                            dicDiff[cellInfo.Key.Row] = dicRowDiffHeight[item];
                        }
                    }

                }
                if (dicDiff?.Count > 0)
                {
                    foreach (var item in dicDiff.Keys)
                    {
                        if (dicDiff[item] > 0)
                        {
                            table.RowDefinitions[item].Height -= dicDiff[item];
                        }
                    }
                }

            }
            return dic;
        }

        /// <summary>
        /// 重置父级行高
        /// </summary>
        /// <param name="table"></param>
        /// <param name="rowDefinition"></param>
        private static Dictionary<string, double> ResetParentRowDefinitions(RTTable table, ReportInfo reportInfo, string cellId, double colWidth)
        {
            var dic = new Dictionary<string, double>();
            try
            {
                if (table.IsFoldOccupy)
                {
                    if (table.RowDefinitions.Count > 0 && table.ColumnDefinitions.Count > 0)
                    {
                        bool isAllFill = true;
                        if (colWidth > 0)
                        {
                            var tbWidth = table.ColumnDefinitions.Sum(c => c.Width);
                            if ((colWidth - tbWidth) > 5)
                            {
                                isAllFill = false;
                            }
                        }
                        bool isTopTable = true;
                        if (!string.IsNullOrEmpty(cellId))
                        {
                            isTopTable = false;
                        }
                        double diffHeight = 0;
                        double oriHeight = 0;
                        foreach (var item in table.RowDefinitions)
                        {
                            if (item.Type == WidthTypeEnum.MinValue)
                            {
                                oriHeight += (item.Height > item.MinHeight) ? item.Height : item.MinHeight;
                            }
                            else
                            {
                                oriHeight += item.Height;
                            }
                        }

                        var columnCount = table.ColumnDefinitions.Count;
                        var rowCount = table.RowDefinitions.Count;
                        //计算单元格合并信息
                        var mergeRowIndexs = CaculateCellMergeInfo(table, columnCount);
                        var groupFieldList = new List<string>();
                        var minRowSpanDic = new Dictionary<int, int>();
                        var otherMergeRowIndexs = new List<int>();
                        bool isHiddenBorder = true;
                        //查询是否存在绑定值
                        var tupleResult = QueryIsExistBindValue(table, mergeRowIndexs, reportInfo);
                        var fixedGroupFields = new List<string>();
                        if (tupleResult != null)
                        {
                            groupFieldList = tupleResult.Item1;
                            minRowSpanDic = tupleResult.Item2;
                            otherMergeRowIndexs = tupleResult.Item3;
                            fixedGroupFields = tupleResult.Item4;
                            isHiddenBorder = tupleResult.Item5;
                        }
                        var otherMergeCellDictionary = new Dictionary<CellPostion, ICell>();
                        var existCellDictionary = new Dictionary<CellPostion, ICell>();
                        ICell waitCloneICell = null;
                        var cellRowInfos = new Dictionary<string, RTRowDefinition>();
                        //过滤隐藏的单元格
                        bool isExistGroupFold = false;
                        var tupleInfo = FilterFoldOccupyCells(table, otherMergeRowIndexs, groupFieldList, rowCount, columnCount);
                        if (tupleInfo != null)
                        {
                            existCellDictionary = tupleInfo.Item1;
                            waitCloneICell = tupleInfo.Item2;
                            cellRowInfos = tupleInfo.Item3;
                            otherMergeCellDictionary = tupleInfo.Item4;
                            isExistGroupFold = tupleInfo.Item5;
                        }
                        if (existCellDictionary?.Count > 0 || otherMergeCellDictionary.Count > 0)
                        {
                            //重绘表格行信息
                            int newCellCount = 0;
                            if (isExistGroupFold)
                            {
                                var cellDictionary = ResetTableGroupRowDefinition(fixedGroupFields, groupFieldList, table, existCellDictionary, columnCount, cellRowInfos, waitCloneICell, isAllFill, isTopTable, isHiddenBorder);
                                
                                table.Cells = cellDictionary;
                            }
                            else
                            {
                                var diffCellCount = ResetTableRowDefinition(table, existCellDictionary, columnCount, cellRowInfos, otherMergeCellDictionary, ref newCellCount);
                                newCellCount += otherMergeCellDictionary.Count;
                                //重绘单元格
                                if (fixedGroupFields.Count == groupFieldList.Count)
                                {
                                    table.Cells = new Dictionary<CellPostion, ICell>();
                                    table.RowDefinitions = new List<RTRowDefinition>();
                                }
                                else
                                {
                                    var cellDictionary = ResetTabelCell(isTopTable, existCellDictionary, waitCloneICell, minRowSpanDic, table.RowDefinitions.Count, columnCount, diffCellCount, newCellCount, otherMergeCellDictionary, isAllFill, isHiddenBorder);
                                    table.Cells = cellDictionary;
                                }
                            }
                            double newHeight = 0;
                            foreach (var item in table.RowDefinitions)
                            {
                                if (item.Type == WidthTypeEnum.MinValue)
                                {
                                    newHeight += (item.Height > item.MinHeight) ? item.Height : item.MinHeight;
                                }
                                else
                                {
                                    newHeight += item.Height;
                                }
                            }
                            diffHeight = oriHeight - newHeight;

                            if (!string.IsNullOrWhiteSpace(cellId))
                            {
                                dic.Add(cellId, diffHeight);
                            }
                        }
                        else
                        {
                            table.RowDefinitions = new List<RTRowDefinition>();
                            table.Cells = new Dictionary<CellPostion, ICell>();
                            table.ColumnDefinitions = new List<RTColumnDefinition>();
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine($"Generate table isFoldOccupy error {e}");
            }
            return dic;
        }

        /// <summary>
        /// 查询是否存在绑定值
        /// </summary>
        /// <param name="table"></param>
        /// <param name="mergeRowIndexs"></param>
        /// <param name="reportInfo"></param>
        /// <returns></returns>
        private static Tuple<List<string>, Dictionary<int, int>, List<int>, List<string>, bool> QueryIsExistBindValue(RTTable table, List<int> mergeRowIndexs, ReportInfo reportInfo)
        {
            var groupFieldList = new List<string>();
            var minRowSpanDic = new Dictionary<int, int>();
            var staticTextGroupFieldList = new List<string>();
            var inputGroupFieldList = new List<string>();
            var isHiddenBorder = true;
            foreach (var cell in table.Cells)
            {
                var borders = cell.Value.Borders;
                if (borders.Bottom.BorderStyle == RTBorderStyle.None)
                {
                    isHiddenBorder = isHiddenBorder & true;
                }
                else
                {
                    isHiddenBorder = isHiddenBorder & false;
                }
                if (borders.Top.BorderStyle == RTBorderStyle.None)
                {
                    isHiddenBorder = isHiddenBorder & true;
                }
                else
                {
                    isHiddenBorder = isHiddenBorder & false;
                }
                if (borders.Left.BorderStyle == RTBorderStyle.None)
                {
                    isHiddenBorder = isHiddenBorder & true;
                }
                else
                {
                    isHiddenBorder = isHiddenBorder & false;
                }
                if (borders.Right.BorderStyle == RTBorderStyle.None)
                {
                    isHiddenBorder = isHiddenBorder & true;
                }
                else
                {
                    isHiddenBorder = isHiddenBorder & false;
                }
                if (!mergeRowIndexs.Contains(cell.Key.Row))
                {
                    var tempGroupField = ((RTCell)cell.Value).GroupField;
                    if (!string.IsNullOrWhiteSpace(tempGroupField))
                    {
                        var existContent = false;
                        foreach (var blockElement in cell.Value.Blocks)
                        {
                            if (blockElement is Paragraph paragraph)
                            {
                                foreach (var element in paragraph.Elements)
                                {
                                    existContent = ParagraphGenerater.QueryElement(element, reportInfo, staticTextGroupFieldList, inputGroupFieldList, tempGroupField);
                                    if (existContent)
                                    {
                                        continue;
                                    }
                                }
                            }
                            if (existContent)
                            {
                                groupFieldList.Add(tempGroupField.ToLower());
                                continue;
                            }
                        }
                    }
                }
                else
                {
                    if (minRowSpanDic.Keys.Contains(cell.Key.Row))
                    {
                        if (minRowSpanDic[cell.Key.Row] > cell.Key.RowSpan)
                        {
                            minRowSpanDic[cell.Key.Row] = cell.Key.RowSpan;
                        }
                    }
                    else
                    {
                        minRowSpanDic.Add(cell.Key.Row, cell.Key.RowSpan);
                    }
                }
            }

            if (staticTextGroupFieldList?.Count > 0 && inputGroupFieldList?.Count > 0)
            {
                foreach (var item in inputGroupFieldList)
                {
                    staticTextGroupFieldList.Remove(item);
                }
                if (staticTextGroupFieldList?.Count > 0)
                {
                    groupFieldList.AddRange(staticTextGroupFieldList);
                }
            }
            groupFieldList = groupFieldList.Distinct().ToList();
            return Tuple.Create(groupFieldList, minRowSpanDic, mergeRowIndexs, staticTextGroupFieldList, isHiddenBorder);
        }

        /// <summary>
        /// 计算单元格合并信息
        /// </summary>
        /// <param name="table"></param>
        /// <param name="columnCount"></param>
        /// <returns></returns>
        private static List<int> CaculateCellMergeInfo(RTTable table, int columnCount)
        {
            var otherRowIndex = new List<int>();
            var mergeCells = table.Cells.Where(c => c.Key.RowSpan > 1);
            foreach (var cell in mergeCells)
            {
                var tempGroupField = ((RTCell)cell.Value).GroupField;
                if (!string.IsNullOrEmpty(tempGroupField))
                {
                    for (var m = 0; m < cell.Key.RowSpan; m++)
                    {
                        otherRowIndex.Add((cell.Key.Row + m));
                    }
                }
            }
            otherRowIndex = otherRowIndex.Distinct().OrderBy(c => c).ToList();
            return otherRowIndex;
        }

        /// <summary>
        /// 过滤隐藏的单元格
        /// </summary>
        /// <param name="table"></param>
        /// <param name="mergeRowIndexs"></param>
        /// <param name="groupFieldList"></param>
        /// <param name="rowCount"></param>
        /// <param name="columnCount"></param>
        /// <returns></returns>
        private static Tuple<Dictionary<CellPostion, ICell>, ICell, Dictionary<string, RTRowDefinition>, Dictionary<CellPostion, ICell>, bool> FilterFoldOccupyCells
            (RTTable table, List<int> otherMergeRowIndexs, List<string> groupFieldList, int rowCount, int columnCount)
        {
            var otherMergeCellDictionary = new Dictionary<CellPostion, ICell>();
            var existCellDictionary = new Dictionary<CellPostion, ICell>();
            var dic = new Dictionary<string, RTRowDefinition>();
            bool isExistGroupFold = false;
            var dicRowIndexs = new Dictionary<string, List<int>>();
            ICell waitCloneICell = null;
            for (int i = 0; i < rowCount; i++)
            {
                for (int j = 0; j < columnCount; j++)
                {
                    var cell = table.Cells?.FirstOrDefault(c => c.Key.Row == i && c.Key.Column == j) ?? new KeyValuePair<CellPostion, ICell>();
                    if (cell.Key != null && cell.Key.RowSpan > 0)
                    {
                        dic.Add(cell.Value.Id, table.RowDefinitions[i]);
                        var tempGroupField = ((RTCell)cell.Value).GroupField;
                        if (string.IsNullOrWhiteSpace(tempGroupField))
                        {
                            if (otherMergeRowIndexs.Contains(cell.Key.Row))
                            {
                                otherMergeCellDictionary.Add(cell.Key, cell.Value);
                            }
                            else
                            {
                                if (cell.Value.Blocks?.Count > 0 && cell.Value.Blocks[0]?.Elements?.Count > 0)
                                {
                                    existCellDictionary.Add(cell.Key, cell.Value);
                                    if (dicRowIndexs.Keys.Contains(tempGroupField))
                                    {
                                        dicRowIndexs[tempGroupField].Add(cell.Key.Row);
                                    }
                                    else
                                    {
                                        dicRowIndexs.Add(tempGroupField, new List<int>() { cell.Key.Row });
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (groupFieldList.Contains(tempGroupField.ToLower()))
                            {
                                existCellDictionary.Add(cell.Key, cell.Value);
                                if (dicRowIndexs.Keys.Contains(tempGroupField))
                                {
                                    dicRowIndexs[tempGroupField].Add(cell.Key.Row);
                                }
                                else
                                {
                                    dicRowIndexs.Add(tempGroupField, new List<int>() { cell.Key.Row });
                                }
                            }
                        }
                        if (i == 0 && j == 0)
                        {
                            waitCloneICell = cell.Value.Clone() as RTCell;
                            waitCloneICell.ClearBlocks();
                        }
                    }
                }
            }
            foreach (var item in dicRowIndexs.Keys)
            {
                if (!string.IsNullOrEmpty(item)) {
                    var vals = dicRowIndexs[item];
                    if (vals.Distinct().Count() > 1)
                    {
                        isExistGroupFold = true;
                        break;
                    }
                }
            }
            return Tuple.Create(existCellDictionary, waitCloneICell, dic, otherMergeCellDictionary, isExistGroupFold);
        }

        //private static Dictionary<CellPostion, ICell> ResetTabelGroupCell(List<string> groupFieldList, bool isTopTable, Dictionary<CellPostion, ICell> existCellDictionary,
        //    ICell waitCloneICell, Dictionary<int, int> minRowSpanDic, int tableRowCount, int columnCount, int diffCellCount, int newCellCount, Dictionary<CellPostion, ICell> otherMergeCellDictionary,
        //    bool isAllFill)
        //{ }

        /// <summary>
        /// 重绘单元格
        /// </summary>
        /// <param name="existCellDictionary"></param>
        /// <param name="mergeCellDictionary"></param>
        /// <param name="waitCloneICell"></param>
        /// <param name="minRowSpanDic"></param>
        /// <param name="tableRowCount"></param>
        /// <param name="columnCount"></param>
        /// <param name="diffCellCount"></param>
        /// <returns>重新绘制的单元格</returns>
        private static Dictionary<CellPostion, ICell> ResetTabelCell(bool isTopTable, Dictionary<CellPostion, ICell> existCellDictionary,
        ICell waitCloneICell, Dictionary<int, int> minRowSpanDic, int tableRowCount, int columnCount, int diffCellCount, int newCellCount, Dictionary<CellPostion, ICell> otherMergeCellDictionary,
        bool isAllFill, bool isHiddenBorder)
        {
            int rowIndex = 0;
            int colIndex = 0;
            var cellDictionary = new Dictionary<CellPostion, ICell>();
            //重新设置表格位置
            foreach (var key in existCellDictionary.Keys)
            {
                var cellInfo = existCellDictionary[key];
                ResetCellBorderStyle(cellInfo, colIndex, rowIndex, tableRowCount, columnCount, key.RowSpan, key.ColumnSpan, isAllFill, isTopTable, false, isHiddenBorder:isHiddenBorder);
                if (key.ColumnSpan > 1)
                {
                    if ((colIndex + key.ColumnSpan) > columnCount)
                    {
                        //填充，保证表格完整
                        if (diffCellCount > 0)
                        {
                            var tempDiffColumnCount = columnCount - colIndex;
                            //如果总数量小于表格，那么clone少的单元格
                            for (var km = 0; km < tempDiffColumnCount; km++)
                            {
                                if (km > 0)
                                {
                                    colIndex++;
                                }
                                cellDictionary.Add(new CellPostion(rowIndex, colIndex, 1, 1), (waitCloneICell.Clone() as RTCell));
                            }
                            colIndex = 0;
                            rowIndex++;
                            diffCellCount = tempDiffColumnCount - tempDiffColumnCount;
                        }
                    }
                }

                key.RowSpan = 1;
                key.ColumnSpan = key.ColumnSpan;
                key.Row = rowIndex;
                key.Column = colIndex;
                cellDictionary.Add(key, cellInfo);
                if (key.ColumnSpan > 1)
                {
                    colIndex += key.ColumnSpan;
                }
                if ((colIndex + 1) >= columnCount)
                {
                    rowIndex++;
                    colIndex = 0;
                }
                else
                {
                    if (key.ColumnSpan <= 1)
                    {
                        colIndex++;
                    }
                }
            }

            if (newCellCount > cellDictionary.Count)
            {
                if (diffCellCount > 0)
                {
                    //如果总数量小于表格，那么clone少的单元格
                    for (var km = 0; km < diffCellCount; km++)
                    {
                        if (km > 0)
                        {
                            colIndex++;
                        }
                        var cellInfo = (waitCloneICell.Clone() as RTCell);
                        cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.Solid;
                        cellInfo.Background = RTColor.White;
                        ResetCellBorderStyle(cellInfo, colIndex, rowIndex, tableRowCount, columnCount, 1, 1, isAllFill, isTopTable, isHiddenBorder:isHiddenBorder);
                        cellDictionary.Add(new CellPostion(rowIndex, colIndex, 1, 1), cellInfo);
                    }
                    rowIndex++;
                }
            }
            if (otherMergeCellDictionary.Count > 0)
            {
                var lastRowIndex = 0;
                foreach (var item in otherMergeCellDictionary.Keys)
                {
                    if (lastRowIndex != 0 && lastRowIndex != item.Row)
                    {
                        rowIndex += minRowSpanDic[item.Row];
                    }
                    var cellInfo = otherMergeCellDictionary[item];
                    ResetCellBorderStyle(cellInfo, item.Column, rowIndex, tableRowCount, columnCount, item.RowSpan, item.ColumnSpan, isAllFill, isTopTable, isHiddenBorder:isHiddenBorder);
                    lastRowIndex = item.Row;
                    var newCellPostion = new CellPostion(rowIndex, item.Column, item.RowSpan, item.ColumnSpan);
                    cellDictionary.Add(newCellPostion, cellInfo);
                }
            }

            return cellDictionary;
        }

        /// <summary>
        /// 重绘表格行信息
        /// </summary>
        /// <param name="table"></param>
        /// <param name="existCellDictionary"></param>
        /// <param name="columnCount"></param>
        /// <param name="mergeRowIndexCount"></param>
        /// <returns>重绘后表格内单元格的差异值</returns>
        private static Dictionary<CellPostion, ICell> ResetTableGroupRowDefinition(List<string> fixedGroupFields, List<string> groupFieldList, RTTable table, Dictionary<CellPostion, ICell> existCellDictionary, int columnCount,
            Dictionary<string, RTRowDefinition> cellRowInfos,ICell waitCloneICell, bool isAllFill, bool isTopTable, bool isHiddenBorder)
        {
            var spanColumnCell = existCellDictionary.Where(c => c.Key.ColumnSpan >= 1 && c.Key.RowSpan == 1)?.ToList() ?? new List<KeyValuePair<CellPostion, ICell>>();

            if (fixedGroupFields.Count == groupFieldList.Count)
            { 
                return new Dictionary<CellPostion, ICell>();
            }

            var firstGroupCells = spanColumnCell.Where(c => ((RTCell)c.Value).GroupField.ToLower() == groupFieldList[0])?.ToList() ?? new List<KeyValuePair<CellPostion, ICell>>();
            var findGroupMaxCells = firstGroupCells.Where(c => c.Key.ColumnSpan == 1)?.ToList() ?? new List<KeyValuePair<CellPostion, ICell>>();
            var rowGroupCount = columnCount / findGroupMaxCells.Count;

            var fillDataRows = (groupFieldList.Count - fixedGroupFields.Count) / rowGroupCount + ((groupFieldList.Count - fixedGroupFields.Count) % rowGroupCount > 0 ? 1 : 0);
            var factFoladGroupCount = fillDataRows * rowGroupCount;

            var rows = new List<RTRowDefinition>();
            if (fixedGroupFields?.Count > 0)
            {
                for (int i = 0; i < fixedGroupFields.Count; i++)
                {
                    var firstHeader = table.RowDefinitions[i];
                    rows.Add(firstHeader);
                }
            }
            var groupRows = new List<RTRowDefinition>();
            var groupMaxRows = firstGroupCells.Select(c => c.Key.Row).Distinct().ToList();
            for (int i = 0; i < groupMaxRows.Count; i++)
            {
                if (cellRowInfos.Keys.Contains(firstGroupCells.FirstOrDefault(c => c.Key.Row == groupMaxRows[i]).Value.Id))
                {
                    var rowHeight = cellRowInfos[firstGroupCells.FirstOrDefault(c => c.Key.Row == groupMaxRows[i]).Value.Id];
                    groupRows.Add(rowHeight);
                }
            }
            for (int i = 0; i < fillDataRows; i++)
            {
                for (int j = 0; j < groupRows.Count; j++)
                {
                    rows.Add(groupRows[j]);
                }
            }

            if (rows.Count > 0)
            {
                table.RowDefinitions = rows;
            }
            int newCellCount = 0;
            var diffCellCount = 0;
            //groupMaxRows

            var diffentGroupCellCount = factFoladGroupCount - (groupFieldList.Count - fixedGroupFields.Count);
            diffCellCount  = diffentGroupCellCount * groupMaxRows.Count * findGroupMaxCells.Count;
            newCellCount = diffCellCount + (groupFieldList.Count - fixedGroupFields.Count) * firstGroupCells.Count + fixedGroupFields.Count;
            int tableRowCount = table.RowDefinitions.Count;
            var cellDictionary = new Dictionary<CellPostion, ICell>();
            int rowIndex = 0;
            int colIndex = 0;
            //重新设置表格位置
            //第一行
            for (int lx = 0; lx < fixedGroupFields.Count; lx++)
            {
                //cellDictionary.Add(existCellDictionary.Keys.FirstOrDefault(), existCellDictionary[existCellDictionary.Keys.FirstOrDefault()]);
                var fixedCellInfo = spanColumnCell.FirstOrDefault(c => ((RTCell)c.Value).GroupField.ToLower() == fixedGroupFields[lx]);
                cellDictionary.Add(fixedCellInfo.Key, fixedCellInfo.Value);
                rowIndex++;
            }
            var tempFactGroupFieldList = groupFieldList.GetRange(0, (groupFieldList.Count - fixedGroupFields.Count));
            var factGroupFieldList = tempFactGroupFieldList.OrderBy(c => c.Length).ThenBy(c => c).ToList();
            for (int m = 0; m < fillDataRows; m++)
            {
                var startIndex = m * rowGroupCount;
                var lengthRange = 0;
                if ((factGroupFieldList.Count - startIndex) > rowGroupCount)
                {
                    lengthRange = rowGroupCount;
                }
                else
                {
                    lengthRange = factGroupFieldList.Count - startIndex;
                }
                var rangeGroupField = factGroupFieldList.GetRange(startIndex, lengthRange);
                var factRowIndex = m * groupMaxRows.Count + rowIndex;
                var curColIndex = 0;
                var rangeColIndex = colIndex;
                for (int k = 0; k < rangeGroupField.Count; k++)
                {
                    curColIndex = rangeColIndex;
                    var curRowIndex = factRowIndex;
                    var tempGroupCells = spanColumnCell.Where(c => ((RTCell)c.Value).GroupField.ToLower() == rangeGroupField[k])?.ToList() ?? new List<KeyValuePair<CellPostion, ICell>>();
                    var curGroupCells = tempGroupCells.OrderBy(c => c.Key.Row).ThenBy(d => d.Key.Column).ToList();
                    var curKeyRowIndexs = curGroupCells.Select(c => c.Key.Row).Distinct().ToList();
                    for (int d = 0; d < curKeyRowIndexs.Count; d++)
                    {
                        var fCells = curGroupCells.Where(c => c.Key.Row == curKeyRowIndexs[d])?.ToList() ?? new List<KeyValuePair<CellPostion, ICell>>();
                        foreach (var item in fCells)
                        {
                            var cellInfo = item.Value;
                            ResetCellBorderStyle(cellInfo, curColIndex, curRowIndex, tableRowCount, columnCount, item.Key.RowSpan, item.Key.ColumnSpan, isAllFill, isTopTable, isHiddenBorder:isHiddenBorder);
                            item.Key.Row = curRowIndex;
                            item.Key.Column = curColIndex;
                            cellDictionary.Add(item.Key, cellInfo);
                            if (item.Key.ColumnSpan > 1)
                            {
                                curColIndex += item.Key.ColumnSpan;
                            }
                            else
                            {
                                curColIndex++;
                            }
                        }
                        if (d < (curKeyRowIndexs.Count - 1))
                        {
                            curRowIndex++;
                            curColIndex = rangeColIndex;
                        }
                        else
                        {
                            rangeColIndex = curColIndex;
                        }
                    }
                }
                if (m == (fillDataRows - 1))
                {
                    rowIndex = factRowIndex;
                }
            }
            if (diffentGroupCellCount == 0)
            {
                colIndex = columnCount;
            }
            else
            {
                colIndex = findGroupMaxCells.Count;
            }
            var initColIndex = colIndex;
            if (newCellCount > cellDictionary.Count)
            {
                if (diffCellCount > 0)
                {
                    for (var km = 0; km < diffCellCount; km++)
                    {
                        var cellInfo = (waitCloneICell.Clone() as RTCell);
                        cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.Solid;
                        cellInfo.Background = RTColor.White;
                        ResetCellBorderStyle(cellInfo, colIndex, rowIndex, tableRowCount, columnCount, 1, 1, isAllFill, isTopTable, isHiddenBorder:isHiddenBorder);
                        cellDictionary.Add(new CellPostion(rowIndex, colIndex, 1, 1), cellInfo);
                        if ((colIndex + 1) >= columnCount)
                        {
                            rowIndex++;
                            colIndex = initColIndex;
                        }
                        else
                        {
                            colIndex++;
                        }
                    }
                }
            }
            return cellDictionary;
        }

        /// <summary>
        /// 重绘表格行信息
        /// </summary>
        /// <param name="table"></param>
        /// <param name="existCellDictionary"></param>
        /// <param name="columnCount"></param>
        /// <param name="mergeRowIndexCount"></param>
        /// <returns>重绘后表格内单元格的差异值</returns>
        private static int ResetTableRowDefinition(RTTable table, Dictionary<CellPostion, ICell> existCellDictionary, int columnCount,
        Dictionary<string, RTRowDefinition> cellRowInfos, Dictionary<CellPostion, ICell> otherMergeCellDictionary, ref int newCellCount)
        {
            var spanColumnCell = existCellDictionary.Where(c => c.Key.ColumnSpan >= 1 && c.Key.RowSpan == 1)?.ToList() ?? new List<KeyValuePair<CellPostion, ICell>>();
            //查找差异
            newCellCount = 0;
            var addColumn = 0;
            var newRowCount = 0;
            var dic = new Dictionary<int, int>();
            int count = 0;
            foreach (var item in spanColumnCell)
            {
                var tempColumn = addColumn;
                addColumn += item.Key.ColumnSpan;
                if (addColumn > columnCount)
                {
                    var diffCount = columnCount - tempColumn;
                    newCellCount += (diffCount > 0 ? diffCount : 1);
                    count += diffCount;
                    newRowCount++;
                    addColumn = item.Key.ColumnSpan;
                    dic.Add(newRowCount, count);
                    count = 1;
                }
                else
                {
                    newCellCount++;
                    count++;
                }
            }
            if (addColumn <= columnCount)
            {
                newRowCount++;
                newCellCount += (columnCount - addColumn);
                count += (columnCount - addColumn);
                dic.Add(newRowCount, count);
            }
            var rows = new List<RTRowDefinition>();
            var minHeight = table.RowDefinitions[0].MinHeight;
            if (newRowCount < table.RowDefinitions.Count)
            {
                int addCellCount = 0;
                for (var i = 1; i <= newRowCount; i++)
                {
                    var cellCounts = dic[i];
                    var tempRows = new List<RTRowDefinition>();
                    for (var j = 0; j < cellCounts; j++)
                    {
                        if ((spanColumnCell.Count - 1) < addCellCount)
                        {
                            var rowDefinition = new RTRowDefinition()
                            {
                                Type = WidthTypeEnum.MinValue,
                                Height = minHeight,
                                MinHeight = minHeight
                            };
                            tempRows.Add(rowDefinition);
                            continue;
                        }
                        if (cellRowInfos.Keys.Contains(spanColumnCell[addCellCount].Value.Id))
                        {
                            tempRows.Add(cellRowInfos[spanColumnCell[addCellCount].Value.Id]);
                        }
                        addCellCount++;
                    }
                    //var rowDefinition = new RTRowDefinition()
                    //{
                    //    Type = WidthTypeEnum.MinValue,
                    //    Height = minHeight,
                    //    MinHeight = minHeight
                    //};
                    //var tempRows = new List<RTRowDefinition>();
                    //for (var j = 0; j < columnCount; j++)
                    //{
                    //    var m = i * columnCount + j;
                    //    //判断长度，防止溢出
                    //    if ((spanColumnCell.Count - 1) < m)
                    //    {
                    //        var cell = existCellDictionary.FirstOrDefault(c => c.Key.Row == i && c.Key.Column == j);
                    //        var rowDefinition = new RTRowDefinition()
                    //        {
                    //            Type = WidthTypeEnum.MinValue,
                    //            Height = cellRowInfos[cell.Value.Id].Height,
                    //            MinHeight = minHeight
                    //        }; 
                    //        tempRows.Add(rowDefinition);
                    //        continue;
                    //    }
                    //    if (cellRowInfos.Keys.Contains(spanColumnCell[m].Value.Id))
                    //    {
                    //        tempRows.Add(cellRowInfos[spanColumnCell[m].Value.Id]);
                    //    }
                    //}
                    var defaultRow = tempRows.OrderByDescending(c => c.Height).FirstOrDefault();
                    rows.Add(defaultRow);
                }

                //合并数据行处理
                var mergeRows = otherMergeCellDictionary.Select(c => c.Key.Row).Distinct().ToList();
                foreach (var item in mergeRows)
                {
                    var rowDefinition = new RTRowDefinition()
                    {
                        Type = WidthTypeEnum.MinValue,
                        Height = minHeight,
                        MinHeight = minHeight
                    };
                    var curCells = otherMergeCellDictionary.Where(c => c.Key.Row == item);
                    foreach (var cellItem in curCells)
                    {
                        if (cellRowInfos.Keys.Contains(cellItem.Value.Id))
                        {
                            if (rowDefinition.Height == 0)
                            {
                                rowDefinition.Height = cellRowInfos[cellItem.Value.Id].Height;
                            }
                            else if (cellRowInfos[cellItem.Value.Id].Height < rowDefinition.Height)
                            {
                                rowDefinition.Height = cellRowInfos[cellItem.Value.Id].Height;
                            }
                            rowDefinition.Type = cellRowInfos[cellItem.Value.Id].Type;
                            rowDefinition.MinHeight = cellRowInfos[cellItem.Value.Id].MinHeight;
                        }
                    }
                    rows.Add(rowDefinition);
                }
                table.RowDefinitions = rows;
            }
            var diffCellCount = newCellCount - spanColumnCell.Count;
            return diffCellCount;
        }

        /// <summary>
        /// 判断数字集合是否是连续的
        /// </summary>
        /// <returns></returns>
        private static bool IsContinuous(List<int> numList)
        {
            numList.Sort((x, y) => -x.CompareTo(y));//降序
            bool result = false;
            if (numList.Count <= 1)
            {
                return true;
            }
            for (int i = 1; i < numList.Count(); i++)
            {
                if (numList[i] == numList[i - 1] - 1)
                    result = true;
                else
                {
                    result = false;
                    break;
                }
            }
            return result;
        }

        /// <summary>
        /// 重设表格边框信息
        /// </summary>
        /// <param name="cellInfo"></param>
        /// <param name="colIndex"></param>
        /// <param name="rowIndex"></param>
        /// <param name="tableCount"></param>
        private static void ResetCellBorderStyle(ICell cellInfo, int colIndex, int rowIndex, int tableRowCount, int tableColumnCount, int rowSpan, int columnSpan,
            bool isAllFill, bool isTopTable, bool isShowTopBorder = true, bool isHiddenBorder = true)
        {
            if (isHiddenBorder)
            {
                cellInfo.Borders.Left.BorderStyle = RTBorderStyle.None;
                cellInfo.Borders.Right.BorderStyle = RTBorderStyle.None;
                cellInfo.Borders.Top.BorderStyle = RTBorderStyle.None;
                cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                return;
            }
            if (cellInfo.Borders.Left.BorderStyle != RTBorderStyle.None || cellInfo.Borders.Right.BorderStyle != RTBorderStyle.None ||
                                    cellInfo.Borders.Bottom.BorderStyle != RTBorderStyle.None || cellInfo.Borders.Top.BorderStyle != RTBorderStyle.None)
            {
                RTBorderStyle curBorderStyle = RTBorderStyle.None;
                if (cellInfo.Borders.Left.BorderStyle != RTBorderStyle.None)
                {
                    curBorderStyle = cellInfo.Borders.Left.BorderStyle;
                }
                else if (cellInfo.Borders.Top.BorderStyle != RTBorderStyle.None)
                {
                    curBorderStyle = cellInfo.Borders.Top.BorderStyle;
                }
                else if (cellInfo.Borders.Right.BorderStyle != RTBorderStyle.None)
                {
                    curBorderStyle = cellInfo.Borders.Right.BorderStyle;
                }
                else
                {
                    curBorderStyle = cellInfo.Borders.Bottom.BorderStyle;
                }
                if (isTopTable)
                {
                    if (curBorderStyle == RTBorderStyle.None)
                    {
                        curBorderStyle = RTBorderStyle.Solid;
                    }
                }
                if (colIndex == 0)
                {
                    if ((rowIndex + rowSpan) == tableRowCount)
                    {
                        //最后一行特殊处理
                        if (isTopTable)
                        {
                            cellInfo.Borders.Bottom.BorderStyle = curBorderStyle;
                        }
                        else
                        {
                            cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                        }
                    }
                    cellInfo.Borders.Top.BorderStyle = curBorderStyle;
                    if (isTopTable)
                    {
                        cellInfo.Borders.Left.BorderStyle = curBorderStyle;
                    }
                    else
                    {
                        cellInfo.Borders.Left.BorderStyle = RTBorderStyle.None;
                    }
                    cellInfo.Borders.Right.BorderStyle = curBorderStyle;
                    if (rowIndex == 0)
                    {
                        if (isTopTable)
                        {
                            cellInfo.Borders.Top.BorderStyle = RTBorderStyle.Solid;
                        }
                        else
                        {
                            cellInfo.Borders.Top.BorderStyle = RTBorderStyle.None;
                        }
                    }
                }
                else
                {
                    //其他列的边框处理
                    if ((rowIndex + rowSpan) == tableRowCount)
                    {
                        //最有一行特殊处理
                        if (isTopTable)
                        {
                            cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.Solid;
                        }
                        else
                        {
                            cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                        }
                    }
                    cellInfo.Borders.Top.BorderStyle = curBorderStyle;
                    cellInfo.Borders.Right.BorderStyle = curBorderStyle;
                    if ((colIndex + columnSpan) == tableColumnCount)
                    {
                        if (isAllFill)
                        {
                            if (isTopTable)
                            {
                                cellInfo.Borders.Right.BorderStyle = curBorderStyle;
                            }
                            else
                            {
                                cellInfo.Borders.Right.BorderStyle = RTBorderStyle.None;
                            }
                        }
                        else
                        {
                            cellInfo.Borders.Right.BorderStyle = curBorderStyle;
                        }
                    }
                    if (rowIndex == 0)
                    {
                        if (isTopTable)
                        {
                            cellInfo.Borders.Top.BorderStyle = RTBorderStyle.Solid;
                        }
                        else
                        {
                            cellInfo.Borders.Top.BorderStyle = RTBorderStyle.None;
                        }
                    }
                }
            }
            else
            {
                cellInfo.Borders.Top.BorderStyle = cellInfo.Borders.Bottom.BorderStyle = cellInfo.Borders.Right.BorderStyle = RTBorderStyle.Solid;
                if (rowIndex == 0)
                {
                    if (isTopTable)
                    {
                        if (isShowTopBorder) {
                            cellInfo.Borders.Top.BorderStyle = RTBorderStyle.Solid;
                        }
                        else {
                            cellInfo.Borders.Top.BorderStyle = RTBorderStyle.None;
                        }
                    }
                    else
                    {
                        cellInfo.Borders.Top.BorderStyle = RTBorderStyle.None;
                    }
                }
                if ((colIndex + columnSpan) == tableColumnCount)
                {
                    if (isTopTable)
                    {
                        cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.Solid;
                    }
                    else
                    {
                        cellInfo.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                    }
                    if (isAllFill)
                    {
                        cellInfo.Borders.Right.BorderStyle = RTBorderStyle.None;
                    }
                    else
                    {
                        cellInfo.Borders.Right.BorderStyle = RTBorderStyle.Solid;
                    }
                }
            }
            if (cellInfo.Borders.Right.BorderStyle != RTBorderStyle.None && cellInfo.Borders.Right.Thickness < 1)
            {
                cellInfo.Borders.Right.Thickness = 1;
            }
            if (cellInfo.Borders.Bottom.BorderStyle != RTBorderStyle.None && cellInfo.Borders.Bottom.Thickness < 1)
            {
                cellInfo.Borders.Bottom.Thickness = 1;
            }
            if (cellInfo.Borders.Top.BorderStyle != RTBorderStyle.None && cellInfo.Borders.Top.Thickness < 1)
            {
                cellInfo.Borders.Top.Thickness = 1;
            }
            if (cellInfo.Borders.Left.BorderStyle != RTBorderStyle.None && cellInfo.Borders.Left.Thickness < 1)
            {
                cellInfo.Borders.Left.Thickness = 1;
            }
        }

        /// <summary>
        /// Generate Table Line
        /// </summary>
        public static void GenerateTableLine(WTextBody wordTextBody, Line line)
        {
            var wordTable = wordTextBody.AddTable();
            if (line.Parent is Paragraph parentParagraph)
            {
                SetTableAlignment(parentParagraph.HorizontalAlignment, wordTable);
            }

            wordTable.ResetCells(2, 1);
            wordTable.TableFormat.Borders.BorderType = BorderStyle.None;
            wordTable.Rows[0].Cells[0].ChildEntities.Clear();
            wordTable.Rows[1].Cells[0].ChildEntities.Clear();
            wordTable.Rows[0].Cells[0].CellFormat.Borders.Bottom.LineWidth = (float)line.Thickness;
            wordTable.Rows[0].Cells[0].CellFormat.Borders.Bottom.Color = line.GetReportTemplateDocument().InvertColor
                ? Syncfusion.Drawing.Color.FromArgb(255 - line.Stroke.R, 255 - line.Stroke.G, 255 - line.Stroke.B)
                : Syncfusion.Drawing.Color.FromArgb(line.Stroke.R, line.Stroke.G, line.Stroke.B);
            wordTable.Rows[0].Cells[0].Width = (float)line.Width * _widthZoomRatio;
            wordTable.Rows[1].Cells[0].Width = (float)line.Width * _widthZoomRatio;
            wordTable.Rows[0].Height = 4;
            wordTable.Rows[0].HeightType = TableRowHeightType.Exactly;
            wordTable.Rows[1].Height = (float)(4 + line.Thickness);
            wordTable.Rows[1].HeightType = TableRowHeightType.Exactly;
            SetBorderType(wordTable.Rows[0].Cells[0].CellFormat.Borders.Bottom, line.BorderStyle);
        }

        /// <summary>
        /// Generate InputImageList
        /// </summary>
        public static void GenerateInputImageList(ReportInfo reportInfo, InputImageList inputImageList, WTextBody wordTextBody, bool showImageIndex = false)
        {
            if (inputImageList.Tag != null && reportInfo.ElementValues.ContainsKey(inputImageList.Tag))
            {
                var imageListElementValue = reportInfo.ElementValues[inputImageList.Tag] as ImageListElementValue;
                if (imageListElementValue != null)
                {
                    var columnCount = inputImageList.Column;
                    var allImagesCount = imageListElementValue.Value.Count();
                    var rowCount = allImagesCount / columnCount;
                    if (allImagesCount % columnCount != 0)
                    {
                        rowCount = rowCount + 1;
                    }

                    if (rowCount <= 0)
                    {
                        return;
                    }

                    var table = AddImageListTable(inputImageList, wordTextBody, rowCount);
                    var width = inputImageList.ImageWidth * _widthZoomRatio;
                    var height = inputImageList.ImageHeight * _widthZoomRatio;
                    InflateImageListTable(inputImageList, table, ref width, ref height);
                    AddImageListValues(table, imageListElementValue.Value.ToArray(), width, height, inputImageList.TransparentBackground, showImageIndex);
                }
            }
        }

        /// <summary>
        /// Add ImageListTable
        /// </summary>
        public static IWTable AddImageListTable(IInputImageListElement element, WTextBody container, int rowsNum)
        {
            var wordTable = container.AddTable();
            wordTable.TableFormat.Paddings.All = 0;// workaround for syncfusion bug
            SetTableAlignment(element.HorizontalAlignment, wordTable);
            wordTable.RemoveAbsPosition();
            wordTable.TableFormat.Borders.BorderType = BorderStyle.None;
            wordTable.ResetCells(rowsNum, element.Column);
            return wordTable;
        }

        /// <summary>
        /// Adjust ImageListTable 
        /// </summary>
        public static void InflateImageListTable(IInputImageListElement element, IWTable table, ref double innerWidth,
            ref double innerHeight)
        {
            var maxCellWidth = table.LastCell.Width;
            // workaround for SyncFusion bug
            var document = element.GetReportTemplateDocument();
            var bordersHeight = element.GetLogicalAncestry()
                .OfType<RTCell>()
                .Sum(x => x.Borders.Top.Thickness + x.Borders.Bottom.Thickness);
            var maxCellHeight = document.GetPageClientHeight() - 10 - bordersHeight;
            var width = innerWidth;
            var height = innerHeight;
            if (element.HasGap)
            {
                table.TableFormat.CellSpacing = InputImageList.ImageListCellSpacing;
                width += InputImageList.ImageListCellSpacing * 2 * (element.Column + 1) / element.Column;
            }

            if (element.HasImageBorder)
            {
                width += InputImage.ImageBorderIncreament;
                height += InputImage.ImageBorderIncreament;
            }

            var overflowWidth = width - maxCellWidth;
            if (overflowWidth > 0)
            {
                innerWidth -= overflowWidth;
                width -= overflowWidth;
            }

            var overflowHeight = height - maxCellHeight;
            if (overflowHeight > 0)
            {
                innerHeight -= overflowHeight;
                height -= overflowHeight;
            }

            AddImageListInnerTable(element, table, width, height, innerWidth, innerHeight);
        }

        private static void AddImageListInnerTable(IInputImageListElement element, IWTable table, double width,
            double height, double innerWidth, double innerHeight)
        {
            foreach (WTableRow row in table.Rows)
            {
                row.Height = (float)height;
                foreach (WTableCell cell in row.Cells)
                {
                    cell.CellFormat.VerticalAlignment = VerticalAlignment.Middle;
                    cell.Width = (float)width;
                    if (element.HasImageBorder)
                    {
                        cell.CellFormat.Borders.BorderType = BorderStyle.Single;
                        cell.CellFormat.Borders.LineWidth = InputImage.ImageBorderThickness;
                        cell.CellFormat.Borders.Color = Syncfusion.Drawing.Color.FromArgb(InputImage.ImageBorderColor.A,
                            InputImage.ImageBorderColor.R, InputImage.ImageBorderColor.G,
                            InputImage.ImageBorderColor.B);
                    }

                    cell.ChildEntities.Clear();
                    var innerTable = cell.AddTable();
                    innerTable.TableFormat.Paddings.All = 0;// workaround for syncfusion bug
                    innerTable.RemoveAbsPosition();
                    innerTable.TableFormat.Borders.BorderType = BorderStyle.None;
                    innerTable.TableFormat.HorizontalAlignment = RowAlignment.Center;
                    innerTable.ResetCells(1, 1);

                    innerTable.FirstRow.Height = (float)innerHeight;
                    innerTable.LastCell.Width = (float)innerWidth;
                    innerTable.LastCell.CellFormat.VerticalAlignment = VerticalAlignment.Middle;
                }
            }
        }


        /// <summary>
        /// Set imageList values
        /// </summary>
        public static void AddImageListValues(IWTable table, IImageElementValue[] values, double width, double height, bool transparentBackground, bool showImageIndex = false, ElementTag elementTag = null)
        {
            if(table.Rows==null) return;
            var columnCount = table.Rows[0].Cells.Count;
            var count = 1;
            for (var i = 0; i < table.Rows.Count; i++)
            {
                for (var j = 0; j < columnCount; j++)
                {
                    var index = i * columnCount + j;
                    if (index >= values.Length)
                    {
                        var cell = table.Rows[i].Cells[j];
                        cell.CellFormat.Borders.BorderType = BorderStyle.None;
                        cell.CellFormat.Borders.LineWidth = 0f;
                        continue;
                    }

                    byte[] bytes = null;
                    switch (values[index])
                    {
                        case FileImageValue fileImageValue:
                            bytes = fileImageValue.ImageBuffer?.GetBytes();
                            break;
                        case BufferImageValue bufferImageValue:
                            bytes = bufferImageValue.Value;
                            break;
                    }
                    if (bytes != null)
                    {
                        var cell = ((IWTable)table.Rows[i].Cells[j].ChildEntities.FirstItem)
                            .LastCell;
                        if (!transparentBackground)
                            cell.CellFormat.BackColor = Syncfusion.Drawing.Color.Black;
                        if (showImageIndex)
                        {
                            var newBytes = GetImageBytesWithIndex(bytes, count, width, height);
                            AddImageListValue(cell, newBytes, width, height, elementTag);
                            count++;
                        }
                        else
                        {
                            AddImageListValue(cell, bytes, width, height, elementTag);
                        }
                    }
                }
            }
        }

        private static byte[] GetImageBytesWithIndex(byte[] imgBytes, int index, double width, double height)
        {
            var newImageBuffer = new byte[0];
            try
            {
                using (var bitmap = SKBitmap.Decode(imgBytes))
                {
                    var bmpMagnification = 3;
                    using (var resizedBitmap = bitmap.Resize(new SKImageInfo((int)width * bmpMagnification, (int)height * bmpMagnification), SKFilterQuality.High))
                    {
                        using (var surface = SKSurface.Create(new SKImageInfo(resizedBitmap.Width, resizedBitmap.Height)))
                        {
                            var canvas = surface.Canvas;
                            canvas.Clear(SKColors.Black);

                            //draw image
                            var imgRatio = (float)bitmap.Width / (float)bitmap.Height;
                            if (width / height > imgRatio)
                            {
                                var newBmpWidth = (int)(height * imgRatio) * bmpMagnification;
                                var x = (resizedBitmap.Width - newBmpWidth) / 2;
                                var rect = new SKRectI(x, 0, newBmpWidth, resizedBitmap.Height);
                                canvas.DrawBitmap(bitmap, rect);
                            }
                            else
                            {
                                var newBmpHeight = (int)(width / imgRatio) * bmpMagnification;
                                var y = (resizedBitmap.Height - newBmpHeight) / 2;
                                var rect = new SKRectI(0, y, resizedBitmap.Width, newBmpHeight);
                                canvas.DrawBitmap(bitmap, rect);
                            }

                            //draw index
                            var sizeRate = resizedBitmap.Height / height * 0.6;
                            var fontSize = Math.Round(sizeRate * 14);
                            var font = new SKPaint
                            {
                                Typeface = SKTypeface.FromFamilyName("Arial"),
                                TextSize = (float)fontSize,
                                Color = SKColors.SkyBlue
                            };
                            canvas.DrawText(index.ToString(), 0, (float)fontSize, font);

                            using (var image = surface.Snapshot())
                            {
                                using (var ms = new MemoryStream())
                                {
                                    image.Encode(SKEncodedImageFormat.Jpeg, 100).SaveTo(ms);
                                    newImageBuffer = ms.ToArray();
                                }
                            }
                        }
                    }
                }
            }
            catch (System.Exception ex)
            {
            }
            return newImageBuffer;
        }

        private static void AddImageListValue(WTableCell cell, byte[] bytes, double width, double height, ElementTag elementTag = null)
        {
            try
            {
                float imgRatio = 0f;
                using (MemoryStream ms = new MemoryStream(bytes))
                {
                    Image image = Image.FromStream(ms);
                    if (elementTag != null && elementTag.Name == "PatientECGImage")
                    {
                        //心电图按照模板长宽展示，不做任何处理
                    }
                    else 
                    {
                        if (image.Height > 0)
                        {
                            imgRatio = (float)image.Width / (float)image.Height;
                        }
                        if (imgRatio > 0)
                        {
                            if (width / height > imgRatio)
                                width = (float)(height * imgRatio);
                            else
                                height = (float)(width / imgRatio);
                        }
                    }
                }
                
            }
            finally
            {
                var imageParagraph = cell.AddParagraph();
                if (elementTag != null && elementTag.Equals(ElementTag.DigitalSignature))
                {
                    imageParagraph.ParagraphFormat.HorizontalAlignment = HorizontalAlignment.Left;
                    imageParagraph.ParagraphFormat.LeftIndent = 3;
                }
                else
                {
                    imageParagraph.ParagraphFormat.HorizontalAlignment = HorizontalAlignment.Center;
                }
                var picture = imageParagraph.AppendPicture(bytes);
                picture.Width = (float)width;
                picture.Height = (float)height;
            }
        }

        /// <summary>
        /// Generate Atlas
        /// </summary>
        public static void GenerateAtlasElement(ReportInfo reportInfo, AtlasElement atlasElement, WTextBody wordTextBody, ReportWordDocument wordDocument)
        {
            if (atlasElement.Tag != null && reportInfo.ElementValues.ContainsKey(atlasElement.Tag))
            {
                var imageListElementValue = reportInfo.ElementValues[atlasElement.Tag] as ImageListElementValue;
                if (imageListElementValue != null && imageListElementValue.Value.Any())
                {
                    var allImagesCount = imageListElementValue.Value.Count();
                    var totalPage = 1;
                    var columnCount = atlasElement.Column;
                    var rowCount = atlasElement.Row;
                    var pageImageCount = rowCount * columnCount;
                    if (pageImageCount > 0)
                    {
                        var pages = allImagesCount / pageImageCount;
                        var addPages = allImagesCount % pageImageCount > 0 ? 1 : 0;
                        totalPage = pages + addPages;
                    }

                    var cellWidth = 0.0;
                    var cellHeight = 0.0;
                    CalcAcutalCellWidth(reportInfo, atlasElement, ref cellWidth, ref cellHeight);

                    for (var index = 0; index < totalPage; index++)
                    {
                        var container = index == 0 ? wordTextBody : wordDocument.AddSection().Body;
                        var table = AddImageListTable(atlasElement, container, rowCount);
                        DeflateImageListTable(atlasElement, table, cellWidth, cellHeight, out var width,
                            out var height);
                        var values = imageListElementValue.Value.Skip(index * pageImageCount).ToArray();
                        AddImageListValues(table, values, width, height, atlasElement.TransparentBackground);
                    }
                }
            }
        }

        private static void DeflateImageListTable(AtlasElement element, IWTable table, double width,
            double height, out double innerWidth, out double innerHeight)
        {
            innerWidth = width;
            innerHeight = height;
            if (element.HasGap)
            {
                table.TableFormat.CellSpacing = InputImageList.ImageListCellSpacing;
                innerWidth -= InputImageList.ImageListCellSpacing * 2 * (element.Column + 1) / element.Column;
                var deltaHeight = InputImageList.ImageListCellSpacing * 2 * (element.Row + 1) / element.Row;
                innerHeight -= deltaHeight;
                height -= deltaHeight;
            }
            if (element.HasImageBorder)
            {
                innerWidth -= InputImage.ImageBorderIncreament;
                innerHeight -= InputImage.ImageBorderIncreament;
                height -= InputImage.ImageBorderThickness * 2;
            }

            AddImageListInnerTable(element, table, width, height, innerWidth, innerHeight);
        }

        private static void CalcAcutalCellWidth(ReportInfo reportInfo, AtlasElement atlasElement, ref double cellWidth, ref double cellHeight)
        {
            var rowCount = atlasElement.Row;
            var columnCount = atlasElement.Column;
            //按比例缩放
            const double defaultImgRatio = 4d / 3;
            var destWidth = reportInfo.Template.GetPageClientWidth() / columnCount - 1;
            var destHeight = reportInfo.Template.GetPageClientHeight() / rowCount - 1;
            var destRatio = destWidth / destHeight;
            if (destRatio < defaultImgRatio)
            {
                cellWidth = destWidth;
                cellHeight = destWidth / defaultImgRatio;
            }
            else
            {
                cellWidth = destHeight * defaultImgRatio;
                cellHeight = destHeight;
            }
        }

        private static void SetTableAlignment(HorizontalLayout horizontalLayout, IWTable wordTable)
        {
            switch (horizontalLayout)
            {
                case HorizontalLayout.Left:
                    wordTable.TableFormat.HorizontalAlignment = RowAlignment.Left;
                    break;
                case HorizontalLayout.Center:
                    wordTable.TableFormat.HorizontalAlignment = RowAlignment.Center;
                    break;
                case HorizontalLayout.Right:
                    wordTable.TableFormat.HorizontalAlignment = RowAlignment.Right;
                    break;
            }
        }

        private static void InitializeTable(RTTable table, IWTable wordTable)
        {
            wordTable.TableFormat.Paddings.All = 0;// workaround for syncfusion bug
            wordTable.TableFormat.LeftIndent = (float)table.Margin.Left;
            var rowCount = table.RowDefinitions.Count;
            var columnCount = table.ColumnDefinitions.Count;
            wordTable.ResetCells(rowCount, columnCount);
            for (var i = 0; i < rowCount; i++)
            {
                for (var j = 0; j < columnCount; j++)
                {
                    if (double.IsNaN(table.ColumnDefinitions[j].Width))
                    {
                        continue;
                    }
                    wordTable.Rows[i].Cells[j].Width = (float)table.ColumnDefinitions[j].Width * _widthZoomRatio;
                }

                if (double.IsNaN(table.RowDefinitions[i].Height))
                {
                    continue;
                }

                //workaround fix SyncFusion table row KeepNext issue
                var height = (float)table.RowDefinitions[i].Height;
                var document = table.GetReportTemplateDocument();
                var maxHeight = document.GetPageClientHeight();
                if (height > maxHeight)
                    height = (float)maxHeight;

                wordTable.Rows[i].Height = height;
            }
        }

        private static void SetTableCell(ReportInfo reportInfo, WTableCell wordCell, ICell cell,
            Dictionary<RTTable, bool> validDictionary)
        {
            wordCell.ChildEntities.Clear();
            //if (cell.Blocks.Any()&& cell.Blocks.All(x => x is InputImageList))
            //    wordCell.OwnerRow.Height = 0;

            foreach (var blockElement in cell.Blocks)
            {
                if (blockElement is Paragraph paragraph)
                {
                    ParagraphGenerater.Generate(reportInfo, paragraph, wordCell);
                    continue;
                }
                cell.HorizontalAlignment = HorizontalLayout.Left;
                blockElement.HorizontalAlignment = HorizontalLayout.Left;
                if (blockElement is RTTable cellRtTable && validDictionary[cellRtTable])
                {
                    Generate(reportInfo, cellRtTable, wordCell, _widthZoomRatio);
                }

                if (blockElement is InputImageList inputImageList)
                {
                    GenerateInputImageList(reportInfo, inputImageList, wordCell);
                }
            }

            var wParagraphs = wordCell.ChildEntities.OfType<IWParagraph>().ToArray();
            if (wParagraphs.Length > 0)
                wParagraphs.First().ParagraphFormat.BeforeSpacing =
                    wParagraphs.Last().ParagraphFormat.AfterSpacing = 3;
        }

        private static void MergeCell(IWTable wordTable, CellPostion cellPostion, RTTable table)
        {
            if (cellPostion.ColumnSpan > 1)
            {
                for (var i = 0; i < cellPostion.RowSpan; i++)
                {
                    wordTable.ApplyHorizontalMerge(cellPostion.Row + i, cellPostion.Column,
                      cellPostion.Column + cellPostion.ColumnSpan - 1);
                }
            }

            if (cellPostion.RowSpan > 1)
            {
                for (var i = 0; i < cellPostion.ColumnSpan; i++)
                    wordTable.ApplyVerticalMerge(cellPostion.Column + i, cellPostion.Row,
                        cellPostion.Row + cellPostion.RowSpan - 1);
            }

            var cell = table.GetCell(cellPostion.Row, cellPostion.Column);
            var invertColor = table.GetReportTemplateDocument().InvertColor;
            for (var i = cellPostion.Row; i < cellPostion.RowSpan + cellPostion.Row; i++)
            {
                wordTable.Rows[i].HeightType = cell.HeightType == WidthTypeEnum.FixedValue ? TableRowHeightType.Exactly : TableRowHeightType.AtLeast;
                for (var j = cellPostion.Column; j < cellPostion.Column + cellPostion.ColumnSpan; j++)
                {
                    var wordCell = wordTable.Rows[i].Cells[j];
                    wordCell.CellFormat.BackColor = invertColor
                        ? Color.FromArgb(cell.Background.A, 255 - cell.Background.R, 255 - cell.Background.G,
                            255 - cell.Background.B)
                        : Color.FromArgb(cell.Background.A, cell.Background.R, cell.Background.G, cell.Background.B);
                    SetCellAlignment(cell, wordCell);

                    SetCellBorderStyle(wordCell.CellFormat.Borders.Left, cell.Borders.Left, invertColor);

                    SetCellBorderStyle(wordCell.CellFormat.Borders.Top, cell.Borders.Top, invertColor);

                    SetCellBorderStyle(wordCell.CellFormat.Borders.Right, cell.Borders.Right, invertColor);

                    SetCellBorderStyle(wordCell.CellFormat.Borders.Bottom, cell.Borders.Bottom, invertColor);
                }
            }
        }

        private static void SetCellBorderStyle(Border border, RTBorder rtBorder, bool invertColor)
        {
            if (rtBorder.BorderStyle == RTBorderStyle.None || rtBorder.Thickness == 0)
            {
                return;
            }

            border.LineWidth = rtBorder.Thickness;
            border.Color = invertColor
                ? Color.FromArgb(255 - rtBorder.Color.R, 255 - rtBorder.Color.G, 255 - rtBorder.Color.B)
                : Color.FromArgb(rtBorder.Color.R, rtBorder.Color.G, rtBorder.Color.B);
            SetBorderType(border, rtBorder.BorderStyle);
        }

        private static void SetCellAlignment(ICell cell, WTableCell wordTableCell)
        {
            switch (cell.VerticalAlignment)
            {
                case VerticalLayout.Center:
                    wordTableCell.CellFormat.VerticalAlignment = VerticalAlignment.Middle;
                    break;
                case VerticalLayout.Top:
                    wordTableCell.CellFormat.VerticalAlignment = VerticalAlignment.Top;
                    break;
                case VerticalLayout.Bottom:
                    wordTableCell.CellFormat.VerticalAlignment = VerticalAlignment.Bottom;
                    break;
            }
        }

        private static void SetBorderType(Border border, RTBorderStyle rtBorderStyle)
        {
            switch (rtBorderStyle)
            {
                case RTBorderStyle.Solid:
                    border.BorderType = BorderStyle.Single;
                    break;
                case RTBorderStyle.Dash:
                    //return BorderStyle.DashDotStroker;
                    border.BorderType = BorderStyle.DashSmallGap;
                    break;
                case RTBorderStyle.Dot:
                    border.BorderType = BorderStyle.Dot;
                    break;
                case RTBorderStyle.DashDot:
                    border.BorderType = BorderStyle.DotDash;
                    break;
                case RTBorderStyle.DashDotDot:
                    border.BorderType = BorderStyle.DotDotDash;
                    break;
                case RTBorderStyle.Double:
                    border.BorderType = BorderStyle.Double;
                    break;
                case RTBorderStyle.None:
                    border.BorderType = BorderStyle.None;
                    break;
            }
        }
    }
}
