using System.Text;
using SkiaSharp;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Log;
using ReportService.ReportConverter;
using Vinno.vCloud.Report.Models;
using Vinno.IUS.Common.IO;
using System.Data;
using System.Net;

namespace ReportService
{
    public class ReportPreviewHelper
    {
        private static readonly System.Net.Http.HttpClient _httpClient = new System.Net.Http.HttpClient();
        private static readonly ReportPreviewImagesGenerater _reportPreviewImagesGenerater = new ReportPreviewImagesGenerater();
        private static readonly string _reportPreviewFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ReportPreviewTemp");

        /// <summary>
        /// pdf转img
        /// </summary>
        /// <param name="pdfFileUrl"></param>
        /// <param name="isPhysicalPath"></param>
        /// <returns></returns>
        public static List<string> GetImageReportPreviewByPDFFile(string pdfFileUrl, bool isPhysicalPath = true)
        {
            if (!Directory.Exists(_reportPreviewFolder))
            {
                Directory.CreateDirectory(_reportPreviewFolder);
            }
            else
            {
                //删除过时文件
                FileHelper.DeleteFile(_reportPreviewFolder, DateTime.Now.AddHours(-6));
            }
            DirectoryInfo reportFolder = new DirectoryInfo(_reportPreviewFolder);
            string fileName = $"{Guid.NewGuid().ToString("N")}";
            string filePath = Path.Combine(_reportPreviewFolder, fileName);

            var files = reportFolder.GetFiles().Where(f => f.FullName.Contains(fileName) && f.Extension == $".jpg");
            if (files != null && files.Any())
            {
                foreach (var file in files)
                {
                    file.Delete();
                }
            }
            var curPhysicalFilePath = pdfFileUrl;
            if (!isPhysicalPath)
            {
                //那就是网络路径
                curPhysicalFilePath = DownloadAsync(_reportPreviewFolder, pdfFileUrl).Result;
            }
            var imageFiles = new List<string>();
            var imgList = _reportPreviewImagesGenerater.ConvertPDFReportToImage(curPhysicalFilePath);
            if (imgList != null && imgList.Count() > 0)
            {
                int i = 0;
                foreach (var image in imgList)
                {
                    using (var bitmap = SkiaSharp.SKBitmap.Decode(image.ImageBuffer.GetBytes()))
                    {
                        var imagePath = $"{filePath}_{i}.jpg";
                        using (var fileStream = new FileStream(imagePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                        {
                            bitmap.Encode(fileStream, SkiaSharp.SKEncodedImageFormat.Jpeg, 100);
                            imageFiles.Add(imagePath);
                            i++;
                        }
                    }
                }
            }
            if (File.Exists(curPhysicalFilePath))
            {
                File.Delete(curPhysicalFilePath);
            }
            return imageFiles;
        }

        /// <summary>
        /// 生成报告预览图
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <returns></returns>
        public static List<string> GetReportPreviewPath(ReportDTO reportInfo, UploadFileTypeEnum fileType)
        {
            try
            {
                if (!Directory.Exists(_reportPreviewFolder))
                {
                    Directory.CreateDirectory(_reportPreviewFolder);
                }
                else
                {
                    //删除过时文件
                    FileHelper.DeleteFile(_reportPreviewFolder, DateTime.Now.AddHours(-6));
                }
                DirectoryInfo reportFolder = new DirectoryInfo(_reportPreviewFolder);
                string fileName = $"{reportInfo.ReportCode}_{Guid.NewGuid().ToString("N")}";
                string filePath = Path.Combine(_reportPreviewFolder, fileName);

                var files = reportFolder.GetFiles().Where(f => f.FullName.Contains(fileName) && f.Extension == $".{fileType.ToString().ToLower()}");
                if (files != null && files.Any())
                {
                    foreach (var file in files)
                    {
                        file.Delete();
                    }
                }
                //生成报告json文件
                ReportJsonSerializer.Serialize($"{filePath}.json", reportInfo);
                var oldReportInfo = Vinno.vCloud.Report.Models.ReportJsonSerializer.DeSerialize($"{filePath}.json");
                oldReportInfo = GetReportInfoAsync(oldReportInfo).Result;
                if (fileType == UploadFileTypeEnum.PDF)
                {
                    var bytes = _reportPreviewImagesGenerater.ConvertReportToPdf(oldReportInfo);
                    if (bytes != null)
                    {
                        var pdfFile = filePath + ".pdf";
                        File.WriteAllBytes(pdfFile, bytes);
                        if (File.Exists(pdfFile))
                        {
                            return new List<string> { pdfFile };
                        }
                    }
                }
                else
                {
                    var imageFiles = new List<string>();
                    var reportImagesInfo = _reportPreviewImagesGenerater.GenerateReportPreviewImages(oldReportInfo);
                    if (reportImagesInfo != null && reportImagesInfo.Count() > 0)
                    {
                        int i = 0;
                        foreach (var image in reportImagesInfo)
                        {
                            using (var bitmap = SKBitmap.Decode(image.ImageBuffer.GetBytes()))
                            {
                                var imagePath = $"{filePath}_{i}.jpg";
                                using (var fileStream = new FileStream(imagePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                                {
                                    bitmap.Encode(fileStream, SKEncodedImageFormat.Jpeg, 100);
                                    imageFiles.Add(imagePath);
                                    i++;
                                }
                            }
                        }
                    }
                    return imageFiles;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"GetReportPreviewPath err {ex}");
            }
            return new List<string>();
        }

        public static string GetReportPreview(ReportDTO reportInfo, UploadFileTypeEnum fileType)
        {
            try
            {
                var files = GetReportPreviewPath(reportInfo, fileType);
                if (files != null && files.Any())
                {
                    if (files.Count == 1)
                    {
                        return files.FirstOrDefault();
                    }
                    else
                    {
                        return MergeImages(files, reportInfo.ReportCode);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportService ReportPreviewHelper GetReportPreview error, ex:{ex}");
            }
            return string.Empty;
        }

        public static string GetReportPreview(List<string> files, string reportCode)
        {
            try
            {
                if (files != null && files.Any())
                {
                    if (files.Count == 1)
                    {
                        return files.FirstOrDefault();
                    }
                    else
                    {
                        return MergeImages(files, reportCode);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportService ReportPreviewHelper GetReportPreview error, ex:{ex}");
            }
            return string.Empty;
        }

        private static string MergeImages(List<string> files, string reportId)
        {
            var images = new List<SKImage>();
            try
            {
                var totalHeight = 0;
                var maxWidth = 0;
                foreach (var fullName in files)
                {
                    SKImage image = SKImage.FromEncodedData(SKData.Create(fullName));
                    images.Add(image);
                    totalHeight += image.Height;
                    if (maxWidth < image.Width)
                    {
                        maxWidth = image.Width;
                    }
                }
                totalHeight += (files.Count - 1) * 5;

                using (SKBitmap bitmap = new SKBitmap(maxWidth, totalHeight))
                {
                    using (SKCanvas canvas = new SKCanvas(bitmap))
                    {
                        canvas.Clear(SKColor.Empty);
                        var y = 0;
                        foreach (var image in images)
                        {
                            var rect = SKRect.Create(0, y, image.Width, image.Height);
                            canvas.DrawImage(image, rect);
                            y = y + image.Height + 5;
                        }
                    }
                    var filePath = Path.Combine(_reportPreviewFolder, $"{reportId}_{Guid.NewGuid():N}.jpg");
                    using (SKImage img = SKImage.FromBitmap(bitmap))
                    {
                        using (SKData p = img.Encode(SKEncodedImageFormat.Jpeg, 100))
                        {
                            using (var stream = File.Create(filePath))
                            {
                                stream.Write(p.ToArray(), 0, p.ToArray().Length);
                            }
                        }
                    }
                    return filePath;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"report preview merge image err,{ex}");
            }
            finally
            {
                foreach (var image in images)
                {
                    image?.Dispose();
                }
            }
            return string.Empty;
        }

        /// <summary>
        /// get report image from storage server
        /// </summary>
        /// <param name="reportInfo">reportInfo entitys</param>
        /// <returns></returns>
        private static async Task<ReportInfo> GetReportInfoAsync(ReportInfo reportInfo)
        {
            foreach (var elementValue in reportInfo.ElementValues.Values)
            {
                if (elementValue is ImageListElementValue imageListElementValue)
                {
                    var fileImageValues = imageListElementValue.Value;
                    if (fileImageValues != null)
                    {
                        foreach (var fileImage in fileImageValues)
                        {
                            if (fileImage is FileImageValue fileImageValue)
                            {
                                if (!string.IsNullOrWhiteSpace(fileImageValue.Value))
                                {
                                    fileImageValue.ImageBuffer = new ByteBuffer(await GetFileAsync(fileImageValue.Value));
                                }
                                else
                                {
                                    Logger.WriteLineWarn($"GetFileAsync error, file url is empty, reportId:{reportInfo.Id}");
                                }
                            }
                        }
                    }
                }
                else if (elementValue is FileImageValue fileImageValue)
                {
                    fileImageValue.ImageBuffer = new ByteBuffer(await GetFileAsync(fileImageValue.Value));
                }
            }
            return reportInfo;
        }

        private static async Task<byte[]> GetFileAsync(string token)
        {
            try
            {
                byte[] data = new byte[0];
                var filePath = await DownloadAsync(_reportPreviewFolder, token);
                if (!string.IsNullOrWhiteSpace(filePath))
                {
                    data = File.ReadAllBytes(filePath);
                    File.Delete(filePath);
                }
                return data;
            }
            catch (Exception e)
            {
                Logger.WriteLineError($"GetFileAsync {token} error:{e}");
                return new byte[0];
            }
        }

        /// <summary>
        /// 下载文件
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        private static async Task<string> DownloadAsync(string downloadFolder, string url)
        {
            if (string.IsNullOrEmpty(url))
            {
                throw new NoNullAllowedException("url");
            }
            if (!Directory.Exists(downloadFolder))
            {
                Directory.CreateDirectory(downloadFolder);
            }
            var fileName = Path.GetFileName(url);
            var targetPath = Path.Combine(downloadFolder, fileName);
            if (File.Exists(targetPath))
            {
                File.Delete(targetPath);
            }
            long fileSize = 0;
            using (var request = new HttpRequestMessage())
            {
                request.RequestUri = new Uri(url);
                request.Method = HttpMethod.Get;
                var response = await _httpClient.SendAsync(request);
                if (response != null && response.StatusCode == HttpStatusCode.OK)
                {
                    var contentLength = response.Content.Headers.ContentLength;
                    fileSize = contentLength == null ? 0 : contentLength.Value;
                }
            }
            if (fileSize <= 0)
            {
                throw new NotSupportedException($"fileSize is {fileSize}");
            }
            byte[] bytes = await _httpClient.GetByteArrayAsync(url);
            File.WriteAllBytes(targetPath, bytes);
            return targetPath;
        }

        /// <summary>
        /// 下载文件
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        public static async Task<byte[]> DownloadAsync(string url)
        {
            try
            {
                if (!string.IsNullOrWhiteSpace(url))
                {
                    if (!Directory.Exists(_reportPreviewFolder))
                    {
                        Directory.CreateDirectory(_reportPreviewFolder);
                    }
                    var fileName = Path.GetFileName(url);
                    var targetPath = Path.Combine(_reportPreviewFolder, fileName);
                    if (File.Exists(targetPath))
                    {
                        return File.ReadAllBytes(targetPath);
                    }
                    else
                    {

                        long fileSize = 0;
                        using (var request = new HttpRequestMessage())
                        {
                            request.RequestUri = new Uri(url);
                            request.Method = HttpMethod.Get;
                            var response = await _httpClient.SendAsync(request);
                            if (response != null && response.StatusCode == HttpStatusCode.OK)
                            {
                                var contentLength = response.Content.Headers.ContentLength;
                                fileSize = contentLength == null ? 0 : contentLength.Value;
                            }
                        }
                        if (fileSize > 0)
                        {
                            byte[] bytes = await _httpClient.GetByteArrayAsync(url);
                            File.WriteAllBytes(targetPath, bytes);
                            return bytes;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportService download report failed, report url:{url}, ex:{ex}");
            }
            return Array.Empty<byte>();
        }

        /// <summary>
        /// 获取文件保存路径
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        public static string GetSaveFilePath()
        {
            if (!Directory.Exists(_reportPreviewFolder))
            {
                Directory.CreateDirectory(_reportPreviewFolder);
            }
            else
            {
                //删除过时文件
                FileHelper.DeleteFile(_reportPreviewFolder, DateTime.Now.AddDays(-1));
            }
            DirectoryInfo reportFolder = new DirectoryInfo(_reportPreviewFolder);
            string fileName = $"{Guid.NewGuid().ToString("N")}";
            string filePath = Path.Combine(_reportPreviewFolder, fileName);
            return filePath;
        }
    }
}