using System.Collections.Generic;
using System;
using System.Text.Json;
using System.Diagnostics;
using System.Net;
using ReportService.Posters;
using ReportService.ReportConverter;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Report;
using WingInterfaceLibrary.Request.Storage;
using WingServerCommon.Config;
using WingServerCommon.Log;
using WingServerCommon.Service;
using WingServerCommon.Utilities.Executors;
using System.Net.Http.Headers;
using SkiaSharp;
using WingInterfaceLibrary.Interface;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.DTO.Record;

namespace ReportService.Service
{
    public partial class ReportService : JsonRpcService, IReportService
    {
        private readonly SequenceExecutor<ReportPushRecordDTO> _reportPostHistoryExecutor = new SequenceExecutor<ReportPushRecordDTO>("ReportPostHistory");
        private readonly SequenceExecutor<ReportConverterDetail> _reportConverterExecutor = new SequenceExecutor<ReportConverterDetail>("ReportConverter");
        private readonly SequenceExecutor<ReportConverterDetail> _ecgReportConverterExecutor = new SequenceExecutor<ReportConverterDetail>("EcgReportConverter");
        private readonly SequenceExecutor<GetReportTemplateDBRequest> _reportTemplateConverterExecutor = new SequenceExecutor<GetReportTemplateDBRequest>("ReportTemplateConverter");
        private static readonly object _locker = new object();

        public class ReportConverterDetail
        {
            public string ReportCode { get; set; }

            /// <summary>
            /// 0-全部生成，1-只生成dpf,2-只生成jpg
            /// </summary>
            /// <value></value>
            public int Type { get; set; } = 0;

            /// <summary>
            /// 是否覆盖
            /// </summary>
            /// <value></value>
            public bool IsCover { get; set; } = true;

            /// <summary>
            /// 是否自动创建vital报告
            /// </summary>
            /// <value></value>
            public bool IsAutoCreateReport { get; set; } = false;
        }

        /// <summary>
        /// 加载报告转发配置到内存
        /// /// </summary>
        /// <returns></returns>
        private void LoadReportPosterFromDb()
        {
            Task.Run(async () =>
           {
               try
               {
                   await Task.Delay(5000);
                   var posters = await _reportPosterDBService.GetReportPosterAllAsync();
                   foreach (var poster in posters)
                   {
                       ReportPoster cachePoster;
                       if (poster.PosterType == ReportPosterTypeEnum.BJAid)
                       {
                           cachePoster = new BJAidReportPoster(poster);
                       }
                       else
                       {
                           cachePoster = new ReportPoster(poster);
                       }
                       CacheMaintenance.Instance.Get<IReportPostersManager>().Add(cachePoster);
                   }
                   if (ConfigurationManager.IsMaster || !ConfigurationManager.IsDistributed)
                   {
                       var records = await _reportPosterDBService.GetAllReportPushRecordsByStateAsync(PushDataStateEnum.UnDo);
                       foreach (var record in records)
                       {
                           JoinQueue(record);
                       }
                   }
               }
               catch (Exception ex)
               {
                   Logger.WriteLineWarn($"ReportService LoadReportPosterFromDb err, {ex}");
               }
           });
        }

        /// <summary>
        /// 刷新报告配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RefreshReportPosterFromDb(RefreshReportPosterRequest request)
        {
            try
            {
                var poster = await _reportPosterDBService.GetReportPosterByCodeAsync(new GetReportPosterByCodeDBRequest
                {
                    ReportPosterCode = request.ReportPosterCode
                });
                if (poster != null)
                {
                    ReportPoster cachePoster;
                    if (poster.PosterType == ReportPosterTypeEnum.BJAid)
                    {
                        cachePoster = new BJAidReportPoster(poster);
                    }
                    else
                    {
                        cachePoster = new ReportPoster(poster);
                    }
                    CacheMaintenance.Instance.Get<IReportPostersManager>().Add(cachePoster);
                }
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"RefreshReportPosterFromDb error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 查询报告预览图
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> FindReportPreviewImageAsync(FindReportPreviewImageRequest request)
        {
            var reportCode = request.RecordCode;
            var language = request.LanguageCode;
            var reportInfo = await FindReportByCodeAsync(reportCode, language);
            var reportPreviewList = reportInfo.ReportPreviewList?.Where(x => x.FileType == UploadFileTypeEnum.JPG);
            if (reportPreviewList == null || !reportPreviewList.Any())
            {
                DoReportConverter(reportCode, UploadFileTypeEnum.JPG);
                await Task.Delay(100);
                reportInfo = await FindReportByCodeAsync(reportCode, language);
                reportPreviewList = reportInfo.ReportPreviewList?.Where(x => x.FileType == UploadFileTypeEnum.JPG);
                if (reportPreviewList == null || !reportPreviewList.Any())
                {
                    return string.Empty;
                }
            }
            var reportPreview = reportPreviewList.OrderByDescending(x => x.Language == language).FirstOrDefault();
            return reportPreview.FileToken;
        }

        /// <summary>
        /// 查询报告预览图
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportPreviewDTO> ConvertVitalReportPreviewImageAsync(ConvertVitalReportPreviewImageRequest request)
        {
            if(request.ReportInfo == null){ return null; }
            return DoReportConverter(request.ReportInfo, UploadFileTypeEnum.JPG);
        }

        /// <summary>
        /// 查询报告预览图
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportPreviewDTO> ConvertVitalReportPreviewPDFAsync(ConvertVitalReportPreviewImageRequest request)
        {
            if(request.ReportInfo == null){ return null; }
            return DoReportConverter(request.ReportInfo, UploadFileTypeEnum.PDF);
        }

        /// <summary>
        /// 加入发送队列
        /// </summary>
        /// <param name="history"></param>
        private void JoinQueue(ReportPushRecordDTO history)
        {
            _reportPostHistoryExecutor.Add(DoPosterByHistory, history);
        }

        private bool DoPosterByHistory(ReportPushRecordDTO history)
        {
            try
            {
                _reportPosterObserver.Post(_storageService, history);
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoPosterHistory err {ex}");
            }
            return true;
        }

        /// <summary>
        /// 加入队列
        /// </summary>
        /// <param name="recordCode"></param>
        private void JoinEcgQueue(string recordCode, int type = 0, bool isCover = true, bool isAutoCreateReport = true)
        {
            var reportConverterDetail = new ReportConverterDetail { ReportCode = recordCode, Type = type, IsCover = isCover, IsAutoCreateReport = isAutoCreateReport };
            _ecgReportConverterExecutor.Add(DoEcgReportConverter, reportConverterDetail);
        }

        /// <summary>
        /// ecg 报告生成
        /// </summary>
        private bool DoEcgReportConverter(ReportConverterDetail reportConverterDetail)
        {
            var type = reportConverterDetail.Type;
            var reportCode = reportConverterDetail.ReportCode;
            var isCover = reportConverterDetail.IsCover;
            var isAutoCreateReport = reportConverterDetail.IsAutoCreateReport;
            if (type == 0 || type == 2)
            {
                DoReportConverter(reportCode, UploadFileTypeEnum.JPG, isCover, true, isAutoCreateReport);
            }
            if (type == 0 || type == 1)
            {
                DoReportConverter(reportCode, UploadFileTypeEnum.PDF, isCover, true, isAutoCreateReport);
            }
            return true;
        }
        
       

        /// <summary>
        /// 加入队列
        /// </summary>
        /// <param name="recordCode"></param>
        private void JoinQueue(string recordCode, int type = 0, bool isCover = true)
        {
            var reportConverterDetail = new ReportConverterDetail { ReportCode = recordCode, Type = type, IsCover = isCover };
            _reportConverterExecutor.Add(DoReportConverter, reportConverterDetail);
        }

        private bool DoReportConverter(ReportConverterDetail reportConverterDetail)
        {
            var type = reportConverterDetail.Type;
            var reportCode = reportConverterDetail.ReportCode;
            var isCover = reportConverterDetail.IsCover;
            if (type == 0 || type == 2)
            {
                DoReportConverter(reportCode, UploadFileTypeEnum.JPG, isCover);
            }
            if (type == 0 || type == 1)
            {
                DoReportConverter(reportCode, UploadFileTypeEnum.PDF, isCover);
            }
            return true;
        }

        private bool DoReportConverter(string reportCode, UploadFileTypeEnum fileType, bool isCover = true, bool isEcgReport = false, bool isAutoCreateReport = false)
        {
            try
            {
                var reportInfo = FindReportByCodeAsync(reportCode, "zh-CN").Result;
                if (reportInfo == null || string.IsNullOrWhiteSpace(reportInfo.ReportCode))
                {
                    return true;
                }
                //如果不覆盖
                if (!isCover)
                {
                    if (fileType == UploadFileTypeEnum.PDF)
                    {
                        if (reportInfo.ReportPreviewList != null && reportInfo.ReportPreviewList.Count > 0 && reportInfo.ReportPreviewList.Any(x => x.FileType == UploadFileTypeEnum.PDF))
                        {
                            return true;
                        }
                    }
                    else if (fileType == UploadFileTypeEnum.JPG)
                    {
                        if (reportInfo.ReportPreviewList != null && reportInfo.ReportPreviewList.Count > 0 && reportInfo.ReportPreviewList.Any(x => x.FileType == UploadFileTypeEnum.JPG))
                        {
                            return true;
                        }
                    }
                }
                Logger.WriteLineInfo($"report convert to {fileType.ToString()} start, patientName:{CryptionHelper.Decrypt(reportInfo.PatientName)}, recordCode:{reportInfo.RecordCode}, reportCode:{reportInfo.ReportCode}");
                if (reportInfo.DiagnosisLabels.Contains("VAid"))
                {
                    var languageConfigDatas = LanguageSetting.GetLanguageConfigParammeter("LanguageConfigs", true);
                    if (languageConfigDatas?.Count > 0)
                    {
                        foreach(var item in languageConfigDatas)
                        {
                            if (item == "zh-CN")
                            {
                                SaveReportPreview(reportInfo, item, fileType, isEcgReport, isAutoCreateReport);
                            }
                            else
                            {
                                reportInfo = FindReportByCodeAsync(reportCode, item).Result;
                                if (reportInfo != null && !string.IsNullOrWhiteSpace(reportInfo.ReportCode))
                                {
                                    SaveReportPreview(reportInfo, item, fileType, isEcgReport, isAutoCreateReport);
                                }
                            }
                        }
                    }
                    else
                    {
                        SaveReportPreview(reportInfo, "en-US", fileType, isEcgReport, isAutoCreateReport);
                    }
                }
                else
                {
                    SaveReportPreview(reportInfo, "", fileType, isEcgReport, isAutoCreateReport);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoReportConverter err {ex}");
            }
            return true;
        }

        private ReportPreviewDTO DoReportConverter(ReportDTO reportInfo, UploadFileTypeEnum fileType)
        {
            try
            {
                if (reportInfo != null || !string.IsNullOrWhiteSpace(reportInfo.ReportCode))
                {
                    return SaveReportPreview(reportInfo, fileType);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoReportConverter err {ex}");
            }
            return null;
        }

        private ReportPreviewDTO SaveReportPreview(ReportDTO reportInfo, UploadFileTypeEnum fileType)
        {
            var pathList = ReportPreviewHelper.GetReportPreviewPath(reportInfo, fileType);
            if (pathList.Any())
            {
                var thumbnailToken = string.Empty;
                var imageUrlList = new List<string>();
                for (var i = 0; i < pathList.Count; i++)
                {
                    var localPath = pathList[i];
                    var imageUrl = UploadFileAsync(localPath, Path.GetFileName(localPath)).Result;
                    if (!string.IsNullOrWhiteSpace(imageUrl))
                    {
                        imageUrlList.Add(imageUrl);
                    }
                    if (i == 0 && fileType == UploadFileTypeEnum.JPG)
                    {
                        thumbnailToken = CreateThumbnailFile(localPath, 100, 30).Result;
                    }
                }
                return new ReportPreviewDTO
                {
                    FileToken = string.Join(";", imageUrlList),
                    ThumbnailToken = thumbnailToken,
                    FileType = fileType
                };
            }
            return null;
        }

        private void SaveReportPreview(ReportDTO reportInfo, string language, UploadFileTypeEnum fileType, bool isEcgReport = false, bool isAutoCreateReport = false)
        {
            var pathList = ReportPreviewHelper.GetReportPreviewPath(reportInfo, fileType);
            if (pathList.Any())
            {
                var thumbnailToken = string.Empty;
                var imageUrlList = new List<string>();
                for (var i = 0; i < pathList.Count; i++)
                {
                    var localPath = pathList[i];
                    var imageUrl = UploadFileAsync(localPath, Path.GetFileName(localPath)).Result;
                    if (!string.IsNullOrWhiteSpace(imageUrl))
                    {
                        imageUrlList.Add(imageUrl);
                    }
                    if (i == 0 && fileType == UploadFileTypeEnum.JPG)
                    {
                        thumbnailToken = CreateThumbnailFile(localPath, 100, 30).Result;
                    }
                }
                _reportDBService.SaveReportPreviewImageAsync(new SaveReportPreviewImageDBRequest
                {
                    ReportCode = reportInfo.ReportCode,
                    FileToken = string.Join(";", imageUrlList),
                    ThumbnailToken = thumbnailToken,
                    FileType = fileType,
                    Language = language
                }).GetAwaiter().GetResult();
                if (isEcgReport)
                {
                    if (isAutoCreateReport)
                    {
                        if (fileType == UploadFileTypeEnum.PDF)
                        {
                            var req = new WingInterfaceLibrary.Request.Vital.VitalReportConvertDetailRequest() {
                                RecordCode = reportInfo.RecordCode,
                                ReportCode =  reportInfo.ReportCode,
                                Type = 1
                            };
                            var r2 = CreateRemoteHEIEcgReport(req).Result;
                        }
                    }
                }
                else 
                {
                    if (fileType == UploadFileTypeEnum.PDF)
                    {
                        var req = new WingInterfaceLibrary.Request.Vital.CreateHEIUltrasonicExamAndReportRequest() {
                            UltrasonicCode = reportInfo.RecordCode,
                            Type = 1
                        };
                        var r1 = CreateHEIUltrasonicExamAndReport(req).Result;
                    }
                }
            }
        }

        private async Task<string> CreateThumbnailFile(string localJpg, int requestHeight, int quality)
        {
            if (!File.Exists(localJpg))
            {
                return string.Empty;
            }
            try
            {
                var compressedData = File.ReadAllBytes(localJpg);
                using (var ms = new MemoryStream(compressedData))
                {
                    using (var image = SKBitmap.Decode(ms))
                    {
                        var ratio = (double)requestHeight / image.Height;
                        if (ratio < 1)
                        {
                            var width = (int)(image.Width * ratio);
                            var height = (int)(image.Height * ratio);
                            SKBitmap tempImage = image;
                            if (image.Width != width || image.Height != height)
                            {
                                tempImage = image.Resize(new SKImageInfo(width, height), SKFilterQuality.High);
                            }
                            try
                            {
                                using (var map = new SKPixmap(new SKImageInfo(tempImage.Width, tempImage.Height, tempImage.ColorType), tempImage.GetPixels()))
                                {
                                    using (var stream = new SKDynamicMemoryWStream())
                                    {
                                        SKPixmap.Encode(stream, map, SKEncodedImageFormat.Jpeg, quality);
                                        compressedData = stream.CopyToData().ToArray();
                                    }
                                }
                            }
                            catch (Exception ex)
                            {
                                throw ex;
                            }
                            finally
                            {
                                tempImage.Dispose();
                            }
                        }
                    }
                }
                var imageUrl = await DoUploadFile($"{Guid.NewGuid():N}.jpg", compressedData);
                return imageUrl;
            }
            catch (Exception ex)
            {
                return string.Empty;
            }
        }

        public async Task<bool> RetryPushReportRecordsAsync(RetryPushReportRecordsByCodesRequest request)
        {
            var records = await _reportPosterDBService.GetReportPushRecordsByCodesAsync(request.Codes);
            foreach (var record in records)
            {
                await _reportPosterDBService.ResetRecordAsync(record.PushRecordCode);
                JoinQueue(record);
            }
            return true;
        }

        /// <summary>
        /// 查询报告详情
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        private ReportDTO FindReporByCode(string code, string language)
        {
            return FindReportByCodeAsync(code, language).Result;
        }

        /// <summary>
        /// 查询检查详情
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        private RecordInfoDTO FindRecordByCode(string code)
        {
            return _recordsInfoDBService.FindRecordInfoByCodeAsync(code).Result;
        }

        /// <summary>
        /// 更新报告转发记录状态
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        private bool UpdateReportPushRecord(UpdateReportPushRecordDBRequest request)
        {
            try
            {
                var fileName = $"{Guid.NewGuid():N}.dat";
                var fileBytes = System.Text.Encoding.UTF8.GetBytes(request.PushResultToken);
                request.PushResultToken = DoUploadFile(fileName, fileBytes).Result;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UpdateReportPushRecord upload push result err, record code: {request.PushRecordCode} {ex}");
            }
            _reportPosterDBService.UpdateReportPushRecordAsync(request);
            return true;
        }

        /// <summary>
        /// 查询报告转发配置
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        private CacheReportPostersDTO FindReportPosterByCode(string code)
        {
            var posters = FindReportPostersWithCodesAsync(new List<string> { code }).Result;
            return posters.FirstOrDefault();
        }

        private IList<CacheReportPostersDTO> FindReportPostersWithCodes(IList<string> codes)
        {
            var posters = FindReportPostersWithCodesAsync(codes).Result;
            return posters;
        }

        private async Task<IList<CacheReportPostersDTO>> FindReportPostersWithCodesAsync(IList<string> codes)
        {
            var dbDTOs = await _reportPosterDBService.GetReportPostersWithCodesAsync(codes as List<string>);
            var list = new List<CacheReportPostersDTO>();
            foreach (var dto in dbDTOs)
            {
                CacheReportPostersDTO poster = null;
                if (dto.PosterType == ReportPosterTypeEnum.BJAid)
                {
                    poster = new BJAidReportPoster(dto);
                }
                else
                {
                    poster = new ReportPoster(dto);
                }
                list.Add(poster);
            }
            return list;
        }

        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="filePath"></param>
        /// <param name="fileName"></param>
        /// <returns></returns>
        private async Task<string> UploadFileAsync(string filePath, string fileName)
        {
            var fileToken = "";
            using (var fileStream = new FileStream(filePath, FileMode.Open))
            {
                var size = fileStream.Length;

                byte[] buffer = new byte[fileStream.Length];
                await fileStream.ReadAsync(buffer, 0, buffer.Length);
                fileToken = await DoUploadFile(fileName, buffer);
            }
            return fileToken;
        }

        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="fileData"></param>
        /// <returns></returns>
        async Task<string> DoUploadFile(string fileName, byte[] fileData)
        {
            var requestHeads = new Dictionary<string, string>();
            var defaultToken = await _authenticationService.GetServerDefaultTokenAsync();
            var authorizationRet = await _storageService.GetAuthorizationAsync(
                new FileServiceRequest
                {
                    Token = defaultToken,
                    FileName = fileName,
                });
            requestHeads.Add("Authorization", authorizationRet.Authorization);
            var fileUrl = authorizationRet.StorageUrl;
            Logger.WriteLineInfo($"DoUploadFile fileUrl:{fileUrl}");
            using (var request = new HttpRequestMessage())
            {
                var fileExtension = Path.GetExtension(fileName);
                var mimeType = FileHelper.GetMimeType(fileExtension);
                var contentType = MediaTypeHeaderValue.Parse(mimeType);
                using (UploadContent content = new UploadContent(fileData, contentType))
                {
                    request.RequestUri = new Uri(fileUrl);
                    request.Method = HttpMethod.Put;
                    request.Content = content;
                    foreach (var head in requestHeads)
                    {
                        request.Headers.TryAddWithoutValidation(head.Key, head.Value);
                    }
                    var result = await ExecuteRequest(request);
                    if (!result)
                    {
                        throw new Exception("Upload file failed!");
                    }
                }
            }
            return fileUrl;
        }

        private UploadFileTypeEnum GetUploadFileType(string fileName)
        {
            if (fileName.EndsWith(".vid"))
            {
                return UploadFileTypeEnum.VID;
            }
            else if (fileName.EndsWith(".jpg"))
            {
                return UploadFileTypeEnum.JPG;
            }
            else
            {
                return UploadFileTypeEnum.MP4;
            }
        }

        /// <summary>
        /// 执行请求
        /// </summary>
        /// <param name="httpRequestMessage"></param>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        private async Task<bool> ExecuteRequest(HttpRequestMessage httpRequestMessage)
        {
            try
            {
                var response = await _httpClient.SendAsync(httpRequestMessage);
                if (response != null && response.StatusCode == HttpStatusCode.OK)
                {
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        #region 报告模板 预览 服务端生成图片功能 
        /// <summary>
        /// 加入队列，执行请求
        /// </summary>
        /// <param name="reportTemplateId"></param>
        private void JoinTemplateQueue(GetReportTemplateDBRequest request)
        {
            _reportTemplateConverterExecutor.Add(DoReportTemplateConverter, request);
        }

        /// <summary>
        /// 执行报告模板
        /// </summary>
        /// <param name="reportTemplateId"></param>
        private bool DoReportTemplateConverter(GetReportTemplateDBRequest request)
        {
            var languageConfigDatas = LanguageSetting.GetLanguageConfigParammeter("LanguageConfigs", true);
            if (languageConfigDatas.Contains(request.LanguageCode)) 
            {
                DoReportTemplateConverter(request.ReportTemplateCode, UploadFileTypeEnum.JPG, request.LanguageCode);
            }
            else
            {
                DoReportTemplateConverter(request.ReportTemplateCode, UploadFileTypeEnum.JPG, "en-US");
            }
            return true;
        }

        /// <summary>
        /// 报告模板转换
        /// </summary>
        /// <param name="reportTemplateId"></param>
        private bool DoReportTemplateConverter(string reportTemplateId, UploadFileTypeEnum fileType, string language)
        {
            try
            {
                var reportTemplateInfo = QueryReportTemplateByCodeAsync(reportTemplateId, language).Result;
                if (reportTemplateInfo == null || string.IsNullOrWhiteSpace(reportTemplateInfo.ReportCode))
                {
                    return true;
                }
                SaveReportTemplatePreview(reportTemplateInfo, language, fileType);
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoReportTemplateConverter err {ex}");
            }
            return true;
        }

        /// <summary>
        /// 获取报告模板数据
        /// </summary>
        /// <param name="reportTemplateId">模板id</param>
        /// <param name="language">语言</param>
        /// <returns></returns>
        private async Task<ReportDTO> QueryReportTemplateByCodeAsync(string reportTemplateId, string language)
        {
            var reportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(reportTemplateId) ?? new ReportTemplateDTO();
            if (!string.IsNullOrEmpty(reportTemplateInfo.ReportTemplateCode))
            {
                if (string.IsNullOrWhiteSpace(language))
                {
                    language = "en-US";
                }
                if (string.IsNullOrWhiteSpace(reportTemplateInfo.ReportDatasJson))
                {
                    reportTemplateInfo.ReportDatasJson = "";
                }
                else
                {
                    //处理过滤数据，根据语言
                    var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportTemplateInfo.ReportDatasJson, new ReportElementDTOConverter());
                    var newReportElements = reportElements?.FindAll(x => x.AILanguage == language) ?? new List<ReportElementDTO>();
                    reportTemplateInfo.ReportDatasJson = Newtonsoft.Json.JsonConvert.SerializeObject(newReportElements);
                }
                reportTemplateInfo.ReportTemplateJson = reportTemplateInfo.ReportTemplateJson;
            }
            var reportInfo = new ReportDTO()
            {
                ReportTemplateJson = reportTemplateInfo.ReportTemplateJson,
                ReportDatasJson = reportTemplateInfo.ReportDatasJson,
                ReportCode = reportTemplateInfo.ReportTemplateCode,
                UpdateTime = DateTime.UtcNow,
            };
            return reportInfo;
        }

        /// <summary>
        /// 保存报告模板预览图
        /// </summary>
        /// <param name="reportInfo">报告模板信息</param>
        /// <param name="language">语言</param>
        /// <param name="fileType">文件类型</param>
        /// <returns></returns>
        private void SaveReportTemplatePreview(ReportDTO reportInfo, string language, UploadFileTypeEnum fileType)
        {
            var pathList = ReportPreviewHelper.GetReportPreviewPath(reportInfo, fileType);
            if (pathList.Any())
            {
                var thumbnailToken = string.Empty;
                var imageUrlList = new List<string>();
                for (var i = 0; i < pathList.Count; i++)
                {
                    var localPath = pathList[i];
                    var imageFileName = Path.GetFileName(localPath);
                    var imageUrl = UploadFileAsync(localPath, imageFileName).Result;
                    if (!string.IsNullOrWhiteSpace(imageUrl))
                    {
                        var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                        var fileName = Path.GetFileName(imageUrl);
                        var result = _fileTransfer.CopyToOtherNode(imageUrl, fileName).Result;
                        var newImageUrl = imageUrl.ToUrlToken();
                        imageUrlList.Add(newImageUrl);
                    }
                    if (i == 0 && fileType == UploadFileTypeEnum.JPG)
                    {
                        var thumbnailUrl = CreateThumbnailFile(localPath, 100, 30).Result;
                        if (!string.IsNullOrEmpty(thumbnailUrl))
                        {
                            var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                            var fileName = Path.GetFileName(thumbnailUrl);
                            var result = _fileTransfer.CopyToOtherNode(imageUrl, fileName).Result;
                            thumbnailToken = thumbnailUrl.ToUrlToken();
                        }
                    }
                }
                _reportDBService.SaveReportTemplatePreviewImageAsync(new SaveReportPreviewImageDBRequest
                {
                    ReportCode = reportInfo.ReportCode,
                    FileToken = string.Join(";", imageUrlList),
                    ThumbnailToken = thumbnailToken,
                    FileType = fileType,
                    Language = language
                });
            }
        }
        #endregion 

    }
}