using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Request;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.Request.Report;
using WingServerCommon.Mapper;

namespace ReportService.Service
{
    public partial class ReportService : JsonRpcService, IReportService
    {
        /// <summary>
        /// 设备端查询检查关联报告集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<DeviceExamReportDTO>> GetDeviceExamReportsAsync(GetDeviceExamReportsRequest request)
        {
            var deviceCode = await ValidateTokenAsync(request.Token);
            var recordCode = request.RecordCode;
            var languageCode = request.LanguageCode;

            var examReports = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = recordCode });
            if (examReports != null && examReports.Any())
            {
                var isEncryptedShow = false;
                if (!string.IsNullOrWhiteSpace(deviceCode))
                {
                    var deviceCache = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Get(deviceCode);
                    if (deviceCache != null && !string.IsNullOrWhiteSpace(deviceCache.OrganizationCode))
                    {
                        var organizationCache = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(deviceCache.OrganizationCode);
                        if (organizationCache != null)
                        {
                            isEncryptedShow = organizationCache.IsEncryptedShow;
                        }
                    }
                }
                var reports = await GetExamReportsAsync(examReports, languageCode, isEncryptedShow);
                reports = reports.Where(v => v.ReportPreviewList != null && v.ReportPreviewList.Count > 0)?.ToList() ?? new List<ReportDTO>();
                reports = reports.OrderByDescending(x => x.DiagnosisLabels == null || !x.DiagnosisLabels.Contains("VAid")).ThenByDescending(x => x.UpdateTime).ToList();

                var resultData = reports.MappingTo<List<DeviceExamReportDTO>>();
                foreach (var item in resultData)
                {
                    var reportUrl = string.Format(_reportUrl, item.ReportCode, languageCode);
                    item.ShareUrl = reportUrl;
                }
                return resultData;
            }
            return new List<DeviceExamReportDTO>();
        }

        /// <summary>
        /// 设备端查询报告详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<DeviceExamReportDTO> GetDeviceExamReportByCodeAsync(GetDeviceExamReportByCodeRequest request)
        {
            var deviceCode = await ValidateTokenAsync(request.Token);
            var reportCode = request.ReportCode;
            var languageCode = request.LanguageCode;

            var reportDO = await _reportDBService.GetReportByCodeAsync(new GetReportByCodeDBRequest { ReportCode = reportCode });
            if (reportDO != null && !string.IsNullOrWhiteSpace(reportDO.ReportCode))
            {
                var isEncryptedShow = false;
                if (!string.IsNullOrWhiteSpace(deviceCode))
                {
                    var deviceCache = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Get(deviceCode);
                    if (deviceCache != null && !string.IsNullOrWhiteSpace(deviceCache.OrganizationCode))
                    {
                        var organizationCache = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(deviceCache.OrganizationCode);
                        if (organizationCache != null)
                        {
                            isEncryptedShow = organizationCache.IsEncryptedShow;
                        }
                    }
                }
                var reports = await GetExamReportsAsync(new List<ReportDTO> { reportDO }, languageCode, isEncryptedShow);
                if (reports != null && reports.Any())
                {
                    var resultData = reports.FirstOrDefault().MappingTo<DeviceExamReportDTO>();
                    var shareUrl = string.Format(_reportUrl, resultData.ReportCode, languageCode);
                    resultData.ShareUrl = shareUrl;
                    if (request.IncludeReportBytes)
                    {
                        var reportUrl = resultData.ReportPreviewList?.FirstOrDefault(x => x.FileToken.EndsWith(".jpg") || x.FileToken.EndsWith(".dat"))?.FileToken;
                        resultData.ReportBytes = await ReportPreviewHelper.DownloadAsync(reportUrl);
                    }
                    return resultData;
                }
            }
            return null;
        }

        private async Task<List<ReportDTO>> GetExamReportsAsync(List<ReportDTO> reports, string languageCode, bool isEncryptedShow)
        {
            foreach (var item in reports)
            {
                if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                {
                    foreach (var preview in item.ReportPreviewList)
                    {
                        if (string.IsNullOrWhiteSpace(preview.ThumbnailToken))
                        {
                            preview.ThumbnailToken = preview.FileToken;
                        }
                    }
                }
                item.PatientName = CryptionHelper.NameEncrypt(item.PatientName, isEncryptedShow);
                if (item.DiagnosisLabels.Contains("VAid"))
                {
                    var language = "en-US";
                    if (!string.IsNullOrWhiteSpace(languageCode))
                    {
                        language = languageCode;
                    }
                    var reportPreviewList = new List<ReportPreviewDTO>();
                    if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                    {
                        reportPreviewList = item.ReportPreviewList.Where(v => v.Language == language)?.ToList() ?? new List<ReportPreviewDTO>();
                        if (!reportPreviewList.Any())
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => v.Language == "en-US")?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                        if (!reportPreviewList.Any() && item.ReportPreviewList.Any(rp => rp.FileToken.Contains(".dat")))
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => string.IsNullOrWhiteSpace(v.Language))?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                    }
                    item.ReportPreviewList = reportPreviewList;
                }
                if (item.ReportPreviewList == null || !item.ReportPreviewList.Any() || !item.ReportPreviewList.Any(x => x.FileToken.ToLower().Contains(".dat") || x.FileToken.ToLower().Contains(".jpg")))
                {
                    JoinQueue(item.ReportCode, 0, false);
                }
            }
            return reports;
        }
    }
}