using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.DBVitalRequest;
using WingServerCommon.Config;
using WingServerCommon.Service;
using WingServerCommon.Config.Parameters;
using Newtonsoft.Json.Linq;
using JsonRpcLite.Log;

namespace ReportService.Service
{
    /// <summary>
    /// 生化血常规报告动态生成检测项管理类
    /// </summary>
    public partial class ReportService : JsonRpcService, IReportService
    {
        /// <summary>
        /// 生化检测，检测项Key与模板组件绑定关系
        /// </summary>
        /// <value></value>
        public List<ExamElementTagModel> _biochemicalTags = new List<ExamElementTagModel>
        {
            new ExamElementTagModel { ExamDataKey = "Total_Protein", ElementId = "4D916526-9947-4EDE-8630-337632A35627", ElementKey = "FetusA"},
            new ExamElementTagModel { ExamDataKey = "directBili", ElementId = "B5A2AAE5-4DDB-4945-9515-82DB367B04F2", ElementKey = "FetusB"},
            new ExamElementTagModel { ExamDataKey = "GGT", ElementId = "35C22E6B-D15E-43CD-88F9-99AA79C3575D", ElementKey = "FetusC"},
            new ExamElementTagModel { ExamDataKey = "UricAcid", ElementId = "24498307-BB99-458B-8EC0-AC8434DE491D", ElementKey = "FetusD"},
            new ExamElementTagModel { ExamDataKey = "Urea", ElementId = "AEC6FDB6-1CD4-4C43-896A-2E914B837545", ElementKey = "FetusE"},
            new ExamElementTagModel { ExamDataKey = "AGratio", ElementId = "04FA2451-BDD0-45E4-931B-3D01E95123E0", ElementKey = "Gravida"},
            new ExamElementTagModel { ExamDataKey = "ALP", ElementId = "96C987B0-D0C3-445B-BD45-CB10A276E6D7", ElementKey = "Para"},
            new ExamElementTagModel { ExamDataKey = "UB", ElementId = "5AAC30D9-FCE3-4FD0-998C-3A9E2D54F312", ElementKey = "BSA"},
            new ExamElementTagModel { ExamDataKey = "Globulin", ElementId = "EA33699C-E86D-497B-A9A9-B8AFAF4A11C7", ElementKey = "AB"},
        };

        /// <summary>
        /// 血常规检测，检测项Key与模板组件绑定关系
        /// </summary>
        /// <value></value>
        public List<ExamElementTagModel> _bloodTags = new List<ExamElementTagModel>
        {
            new ExamElementTagModel { ExamDataKey = "ALC", ElementId = "F4340A5E-16FA-4521-B79D-C14412F24AD5", ElementKey = "LeftTumor"},
            new ExamElementTagModel { ExamDataKey = "AMC", ElementId = "F4340A5E-16FA-A321-B79D-C14412F24AD5", ElementKey = "RightTumor"},
            new ExamElementTagModel { ExamDataKey = "ANC", ElementId = "d02481bc-c9fb-4514-a757-1b6cf1528186", ElementKey = "ApplyForHospital"},
            new ExamElementTagModel { ExamDataKey = "AEC", ElementId = "53a84c3d-a374-4902-aba3-f226a4e8d01a", ElementKey = "ConsultationHospital"},
            new ExamElementTagModel { ExamDataKey = "EOS", ElementId = "D2340A5E-16FA-4321-B79D-C14412F24AD5", ElementKey = "BedNumber"},

            new ExamElementTagModel { ExamDataKey = "Bas", ElementId = "4D916526-9947-4EDE-8630-337632A35627", ElementKey = "FetusA"},
            new ExamElementTagModel { ExamDataKey = "BasRatio", ElementId = "B5A2AAE5-4DDB-4945-9515-82DB367B04F2", ElementKey = "FetusB"},
            new ExamElementTagModel { ExamDataKey = "ALY", ElementId = "35C22E6B-D15E-43CD-88F9-99AA79C3575D", ElementKey = "FetusC"},
            new ExamElementTagModel { ExamDataKey = "ALYRatio", ElementId = "24498307-BB99-458B-8EC0-AC8434DE491D", ElementKey = "FetusD"},
            new ExamElementTagModel { ExamDataKey = "LIC", ElementId = "A1843C2E-B60A-483F-A5F0-B26F475A3611", ElementKey = "PatientTemperature"},
            new ExamElementTagModel { ExamDataKey = "LICRatio", ElementId = "A1843C2E-B60A-483F-A5F0-B26F475A3612", ElementKey = "PatientPulseRate"},
            new ExamElementTagModel { ExamDataKey = "RDW-CV", ElementId = "A1843C2E-B60A-483F-A5F0-B26F475A3613", ElementKey = "PatientRespiratoryRate"},
            new ExamElementTagModel { ExamDataKey = "RDW-SD", ElementId = "A1843C2E-B60A-483F-A5F0-B26F475A3614", ElementKey = "PatientWeightIndex"},
            new ExamElementTagModel { ExamDataKey = "P-LCC", ElementId = "A1843C2E-B60A-483F-A5F0-B26F475A3616", ElementKey = "PatientLeftBloodPressure"},
            new ExamElementTagModel { ExamDataKey = "P-LCR", ElementId = "A1843C2E-B60A-483F-A5F0-B26F475A3617", ElementKey = "PatientRightBloodPressure"},
        };

        private List<ExamElementTagModel> GetExamTagsByKey(string key)
        {
            return key == "HEIBiochemical" ? _biochemicalTags : _bloodTags;;
        }

        /// <summary>
        /// 检测项偏高偏低提示处理
        /// </summary>
        /// <param name="key"></param>
        /// <param name="dictionary"></param>
        /// <param name="json"></param>
        private void FillResultPrompt(string key, Dictionary<string, string> dictionary, JObject json)
        {
            try
            {
                var tags = GetExamTagsByKey(key);
                foreach (var tag in tags)
                {
                    var elementKey = tag.ElementKey;
                    var examValue = json.ContainsKey(tag.ExamDataKey) ? json[tag.ExamDataKey].ToString() : "";
                    if (!dictionary.ContainsKey(elementKey))
                    {
                        dictionary.Add(elementKey, GetResultPromptByElementKey(key, elementKey, examValue));
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteWarning($"ReportService ExamTag FillResultPrompt failed. key:{key}, ex:{ex}");
            }
        }

        /// <summary>
        /// 检测项补全，用于从字典里获取翻译和单位信息
        /// </summary>
        /// <param name="key"></param>
        /// <param name="request"></param>
        private void FillExamDataKey(string key, GetDictionaryNameAndUnitByKeysDBRequest request)
        {
            try
            {
                var tags = GetExamTagsByKey(key);
                foreach (var tag in tags)
                {
                    if (!request.Keys.Contains(tag.ExamDataKey))
                    {
                        request.Keys.Add(tag.ExamDataKey);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteWarning($"ReportService ExamTag FillExamDataKey failed. key:{key}, ex:{ex}");
            }
        }

        /// <summary>
        /// 用于健康指导显示异常项
        /// </summary>
        /// <param name="key"></param>
        /// <param name="dataItemList"></param>
        /// <param name="json"></param>
        /// <param name="dics1"></param>
        private void RefreshDateItems(string key, List<DataItemDTO> dataItemList, JObject json, List<DictionaryWithUnitDTO> dics1)
        {
            try
            {
                var tags = GetExamTagsByKey(key);
                foreach (var tag in tags)
                {
                    var dateItemKey = $"{key}_{tag.ExamDataKey}";
                    var examDataKey = tag.ExamDataKey;
                    var elementKey = tag.ElementKey;
                    var examValue = json.ContainsKey(tag.ExamDataKey) ? json[tag.ExamDataKey].ToString() : "";
                    var unit = (dics1.FirstOrDefault(c => c.Key.Contains(examDataKey)) ?? new DictionaryWithUnitDTO()).Unit;
                    if (dataItemList.Exists(c => c.Key == dateItemKey))
                    {
                        dataItemList.FirstOrDefault(c => c.Key == dateItemKey).Value = GetResultPromptMsgByKey(key, elementKey, examValue, unit);
                    }
                    else
                    {
                        dataItemList.Add(new DataItemDTO()
                        {
                            Key = dateItemKey,
                            Value = GetResultPromptMsgByKey(key, elementKey, examValue, unit)
                        });
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteWarning($"ReportService ExamTag RefreshDateItems failed. key:{key}, ex:{ex}");
            }
        }

        /// <summary>
        /// 报告内容自动填充，包含翻译文件Key
        /// </summary>
        /// <param name="key"></param>
        /// <param name="reportDatas"></param>
        /// <param name="json"></param>
        /// <param name="item"></param>
        private void FillReportData(string key, List<ReportElementDTO> reportDatas, JObject json, string item)
        {
            try
            {
                var tags = GetExamTagsByKey(key);
                foreach (var tag in tags)
                {
                    var elementKey = tag.ElementKey;
                    var examValue = json.ContainsKey(tag.ExamDataKey) ? json[tag.ExamDataKey].ToString() : "";
                    var element = new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", $"Element{elementKey}Key").Value, tag.ElementId, elementKey, examValue, item);
                    reportDatas.Add(element);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteWarning($"ReportService ExamTag FillReportData failed. key:{key}, ex:{ex}");
            }
        }
    }

    public class ExamElementTagModel
    {
        /// <summary>
        /// 检测结果内检测项Key
        /// </summary>
        /// <value></value>
        public string ExamDataKey { get; set; }

        /// <summary>
        /// 模板绑定内容Key
        /// </summary>
        /// <value></value>
        public string ElementKey { get; set; }

        /// <summary>
        /// 模板绑定组件id
        /// </summary>
        /// <value></value>
        public string ElementId { get; set; }
    }
}