﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The struct defines the element type.
    /// </summary>
    [JsonObject]
    [DataContract(Namespace = "")]
    [Serializable]
    public struct ElementType : IEquatable<ElementType>
    {
        /// <summary>
        /// The space element. Used as white space.
        /// </summary>
        public static readonly ElementType Space = new ElementType("Space");

        /// <summary>
        /// The line element.Always used as paragraph seperator.
        /// </summary>
        public static readonly ElementType Line = new ElementType("Line");

        /// <summary>
        /// The static image element defined in report template.such as Logo
        /// </summary>
        public static readonly ElementType StaticImage = new ElementType("StaticImage");

        /// <summary>
        /// The input image element used for user fill image data.
        /// </summary>
        public static readonly ElementType InputImage = new ElementType("InputImage");

        /// <summary>
        /// The image list. Used to show selected image list.
        /// </summary>
        public static readonly ElementType ImageList = new ElementType("ImageList");

        /// <summary>
        /// The AtlasElement.
        /// </summary>
        public static readonly ElementType AtlasElement = new ElementType("AtlasElement");

        /// <summary>
        /// The static text element.e.g. PatientInfo description:PatientName
        /// </summary>
        public static readonly ElementType StaticText = new ElementType("StaticText");

        /// <summary>
        /// The input text is used for user to fill data.
        /// </summary>
        public static readonly ElementType InputText = new ElementType("InputText");

        /// <summary>
        /// The single selected element is used for use to choose one item with ComboBox or others control.
        /// </summary>
        public static readonly ElementType SingleSelected = new ElementType("SingleSelected");
        public static readonly ElementType SingleSelectedVertical = new ElementType("SingleSelectedVertical");

        /// <summary>
        /// The multi selected element is used for use to do multi selection.
        /// </summary>
        public static readonly ElementType MultiSelected = new ElementType("MultiSelected");
        public static readonly ElementType MultiSelectedVertical = new ElementType("MultiSelectedVertical");

        /// <summary>
        /// The date time element.
        /// </summary>
        public static readonly ElementType DateTime= new ElementType("DateTime");

        /// <summary>
        /// The page number element.
        /// </summary>
        public static readonly ElementType PageNumber= new ElementType("PageNumber");

        /// <summary>
        /// The paragraph element.
        /// </summary>
        public static readonly ElementType Paragraph = new ElementType("Paragraph");

        /// <summary>
        /// The grid element.Used for layout arrangement.
        /// </summary>
        public static readonly ElementType RTGrid = new ElementType("RTGrid");

        /// <summary>
        /// The cell element.
        /// </summary>
        public static readonly ElementType RTCell = new ElementType("RTCell");

        /// <summary>
        /// The table element.
        /// </summary>
        public static readonly ElementType RTTable = new ElementType("RTTable");

        /// <summary>
        /// Gets the element type name.
        /// </summary>
        [DataMember]
        public string Name { get; set; }

        /// <summary>
        /// Initialize a <see cref="ElementType"/>
        /// </summary>
        /// <param name="name">The element type name.</param>
        public ElementType(string name)
        {
            Name = name;
        }

        /// <summary>
        /// Tests whether the specified <see cref="ElementType"/> structure is identical to this type.
        /// </summary>
        /// <param name="other">The <see cref="ElementType" /> structure to compare to the current structure.</param>
        /// <returns>If the specified <see cref="ElementType"/> structure is identical to the current structure, returns true; otherwise, returns false.</returns>
        public bool Equals(ElementType other)
        {
            return string.Equals(Name, other.Name);
        }

        /// <summary>
        /// Tests whether the specified object is a <see cref="ElementType" /> structure and is equivalent to this type.
        /// </summary>
        /// <param name="obj">The object to compare to this <see cref="ElementType" /> structure.</param>
        /// <returns>If the specified object is a <see cref="ElementType"/> structure and is identical to the current structure, returns true; otherwise, returns false. </returns>
        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            return obj is ElementType && Equals((ElementType)obj);
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns>A hash code for the current <see cref="ElementType"/>.</returns>
        public override int GetHashCode()
        {
            return (Name != null ? Name.GetHashCode() : 0);
        }
    }
}