﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common.Args;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The column definition of grid.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTColumnDefinition
    {
        /// <summary>
        /// Gets or sets the column width.Unit is pt.
        /// </summary>
        [DataMember]
        public double Width { get; set; }
    }

    /// <summary>
    /// The row definition of grid.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTRowDefinition
    {
        private double _minHeight = 18;
        /// <summary>
        /// Gets or sets the row height.Unit is pt.
        /// </summary>
        [DataMember]
        public double Height { get; set; }

        [DataMember]
        public WidthTypeEnum Type { get; set; }
        [DataMember]
        public double MinHeight
        {
            get => _minHeight;
            set
            {
                if (_minHeight != value)
                {
                    _minHeight = value;
                }
            }
        }
    }

    /// <summary>
    /// The cells collection changed arguments.
    /// </summary>
    public class CellsChangedArgs
    {
        /// <summary>
        /// Gets the added cells.
        /// </summary>
        public IEnumerable<KeyValuePair<CellPostion, ICell>> AddedCells { get; }

        /// <summary>
        /// Gets the removed cells.
        /// </summary>
        public IEnumerable<KeyValuePair<CellPostion, ICell>> RemovedCells { get; }

        /// <summary>
        /// Initialize a <see cref="CellsChangedArgs"/>
        /// </summary>
        /// <param name="addedCells">The added cells.</param>
        /// <param name="removedCells">The removed cells.</param>
        public CellsChangedArgs(IEnumerable<KeyValuePair<CellPostion, ICell>> addedCells, IEnumerable<KeyValuePair<CellPostion, ICell>> removedCells)
        {
            AddedCells = addedCells;
            RemovedCells = removedCells;
        }
    }

    /// <summary>
    /// Define the inferface of Grid block. Used for UI layout.
    /// </summary>
    public interface IGrid : IBlockElement
    {
        /// <summary>
        /// The event occurs when column added or removed.
        /// </summary>
        event EventHandler<CollectionChangedArgs<RTColumnDefinition>> ColumnDefinitionCollectionChanged;

        /// <summary>
        /// The event occurs when row added or removed.
        /// </summary>
        event EventHandler<CollectionChangedArgs<RTRowDefinition>> RowDefinitionCollectionChanged;
        event EventHandler<CollectionChangedArgs<RTRowDefinition>> RowHeightChanged;
        event EventHandler<CollectionChangedArgs<RTColumnDefinition>> ColumnWidthChanged;

        /// <summary>
        /// The event occurs when cell added, removed or merged.
        /// </summary>
        event EventHandler<CellsChangedArgs> CellsChanged;
        event EventHandler CellsClear;

        /// <summary>
        /// Gets the column definitions.
        /// </summary>
        IReadOnlyList<RTColumnDefinition> ColumnDefinitions { get; }

        /// <summary>
        /// Gets the row definitions.
        /// </summary>
        IReadOnlyList<RTRowDefinition> RowDefinitions { get; }

        /// <summary>
        /// Gets all cells.
        /// </summary>
        IReadOnlyDictionary<CellPostion, ICell> Cells { get; }

        /// <summary>
        /// Merge cell
        /// </summary>
        /// <param name="startRowIndex">The start row index.</param>
        /// <param name="endRowIndex">The end row index.</param>
        /// <param name="startColumnIndex">The start column index.</param>
        /// <param name="endColumnIndex">The end column index.</param>
        void MergeCell(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex);

        /// <summary>
        /// Initialize cells for grid.
        /// </summary>
        /// <param name="row">The row count.</param>
        /// <param name="column">The column count.</param>
        void SetCells(int row, int column);

        /// <summary>
        /// Gets cell position, includes row index, cell index, row span and column span.
        /// </summary>
        /// <param name="cell">The searched cell.</param>
        /// <returns>The <see cref="CellPostion"/></returns>
        CellPostion GetCellPosition(ICell cell);

        /// <summary>
        /// Get cell by cell position.
        /// </summary>
        /// <param name="rowIndex">The row index.</param>
        /// <param name="columnIndex">The column index.</param>
        /// <returns>The <see cref="ICell"/></returns>
        ICell GetCell(int rowIndex, int columnIndex);
    }
}