﻿using System;
using System.Collections.Generic;
using System.IO;
using Vinno.IUS.Common.Args;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The interface defines the data struct of report template document.
    /// </summary>
    public interface IReportTemplateDocument : IReportElement, IReportTemplateBody
    {
        /// <summary>
        /// The event occurs when page size changed.
        /// </summary>
        event EventHandler PageSizeChanged;

        /// <summary>
        /// The event occurs when page headerheight or footerheight changed.
        /// </summary>
        event EventHandler HeightChanged;

        /// <summary>
        /// The event when new element added.
        /// </summary>
        event EventHandler<IElement> CurrentElementChanged;

        /// <summary>
        /// The event occurs when headers collection changes.
        /// </summary>
        event EventHandler<CollectionChangedArgs<IBlockElement>> HeadersCollectionChanged;

        /// <summary>
        /// The event occurs when footers collection changes.
        /// </summary>
        event EventHandler<CollectionChangedArgs<IBlockElement>> FootersCollectionChanged;

        /// <summary>
        /// Gets or sets the update time.
        /// </summary>
        DateTime UpdateTime { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the report template is created by user.
        /// </summary>
        bool IsCustom { get; set; }

        /// <summary>
        /// Gets the author of the report template.
        /// </summary>
        string Author { get; }

        /// <summary>
        /// Gets the report template name.
        /// </summary>
        string Name { get; set; }

        /// <summary>
        /// Gets or sets the page padding. default is 20mm
        /// </summary>
        RTThickness PagePadding { get; set; }

        /// <summary>
        /// Gets or sets the page size.
        /// </summary>
        RTPageSize PageSize { get; set; }

        /// <summary>
        /// get or set the distance between the header and the top of the page
        /// </summary>
        double HeaderDistance { get; set; }

        /// <summary>
        /// get or set the distance between the footer and the bottom of the page
        /// </summary>
        double FooterDistance { get; set; }

        /// <summary>
        /// Gets or sets the height of header.
        /// </summary>
        double? HeaderHeight { get;  set; }

        /// <summary>
        /// Gets or sets the height of footer.
        /// </summary>
        double? FooterHeight { get;  set; }

        /// <summary>
        /// Gets the customer tags of the report template.
        /// </summary>
        Dictionary<string, ElementTag> Tags { get; }

        /// <summary>
        /// Gets the report template header.
        /// </summary>
        IReadOnlyList<IBlockElement> Header { get; }

        /// <summary>
        /// Gets the report template footer.
        /// </summary>
        IReadOnlyList<IBlockElement> Footer { get; }

        /// <summary>
        /// Gets the report template BaseFontSize.
        /// </summary>
        double BaseFontSize { get; set; }

        bool InvertColor { get; set; }

        string Version { get; set; }

        /// <summary>
        /// key1=language, key2=keyword
        /// </summary>
        Dictionary<string, Dictionary<string, string>> CustomizeTranslation { get; set; }

        HashSet<string> CustomizeComments { get; set; }

        HashSet<string> CustomizeReportComments { get; set; }

        HashSet<CustomizeCalculation> CustomizeCalculations { get; set; }

        /// <summary>
        /// Save report template to file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <param name="reportTemplateSerializer">The report template serializer.</param>
        void Save(string filePath, IReportTemplateSerializer reportTemplateSerializer);

        /// <summary>
        /// Save report template to stream.
        /// </summary>
        /// <param name="stream">The stream.</param>
        /// <param name="reportTemplateSerializer">The report template serializer.</param>
        void Save(Stream stream, IReportTemplateSerializer reportTemplateSerializer);

        /// <summary>
        /// Add a block for footer.
        /// </summary>
        /// <param name="block">The new added block.</param>
        void AddFooterBlock(IBlockElement block);

        /// <summary>
        /// Remove a block from footer.
        /// </summary>
        /// <param name="block">The removed block.</param>
        void RemoveFooterBlock(IBlockElement block);

        /// <summary>
        /// Add a block for header.
        /// </summary>
        /// <param name="block">The new added block.</param>
        void AddHeaderBlock(IBlockElement block);

        /// <summary>
        /// Remove a block from header.
        /// </summary>
        /// <param name="block">The removed block.</param>
        void RemoveHeaderBlock(IBlockElement block);

        /// <summary>
        /// Add a input tag.
        /// </summary>
        /// <param name="tag">The new added input tag.</param>
        void AddTag(ElementTag tag);

        /// <summary>
        /// Remove a input tag.
        /// </summary>
        /// <param name="tag">The removed input tag.</param>
        void RemoveTag(ElementTag tag);

        /// <summary>
        /// Get all used tags.
        /// </summary>
        /// <returns></returns>
        IEnumerable<ElementTag> GetAllTags();

        /// <summary>
        /// Raise current element changed event.
        /// </summary>
        void RaiseCurrentElementChanged(IElement element);

        /// <summary>
        /// Raise page size changed event.
        /// </summary>
        void RaisePageSizeChanged();

        /// <summary>
        /// Raise page headerheight or footerheight changed event.
        /// </summary>
        void RaiseHeightChanged();

        /// <summary>
        /// Insert header block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        void InsertHeaderBlock(IBlockElement block, int index);

        /// <summary>
        /// Insert footer block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        void InsertFooterBlock(IBlockElement block, int index);

        double GetPageClientWidth();

        /// <summary>
        /// Get page client height assuming <see cref="Header"/>/<see cref="Footer"/> both having visible content/>
        /// </summary>
        /// <returns></returns>
        double GetPageClientHeight();

        void UpdateElementProperty(IReportTemplateDocument reportTemplateDocument);
    }
}
