﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.Interfaces
{
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class MeasureTag
    {
        public const string ReportTemplatePrefix = "ReportTemplate.";
        private SortedSet<string> _internalAvailableMethods;

        [DataMember]
        public string[] AvailableMethods
        {
            get => InternalAvailableMethods.ToArray();
            set => InternalAvailableMethods = value == null ? null : new SortedSet<string>(value);
        }

        [DataMember] public string BaseType { get; set; }

        [DataMember] public string CalculationId { get; set; }

        private SortedSet<string> InternalAvailableMethods
        {
            get => _internalAvailableMethods ?? new SortedSet<string>();
            set => _internalAvailableMethods = value;
        }

        [JsonIgnore] public bool IsCustomize => UserId?.StartsWith(ReportTemplatePrefix) ?? false;

        [DataMember] public string MatchId { get; set; }

        [DataMember] public string Method { get; set; }

        [DataMember] public string Mode { get; set; }

        [Obsolete] [DataMember] public string Output { get; set; }

        [DataMember] public string OutputV2 { get; set; }

        [DataMember] public string Unit { get; set; }

        [DataMember] public string UserId { get; set; }

        [JsonIgnore] public string UserIdWithoutPrefix => UserId?.Replace(ReportTemplatePrefix, "");

        [DataMember] public string MeasureResultType { get; set; }

        protected bool Equals(MeasureTag other)
        {
            return InternalAvailableMethods.SetEquals(other.InternalAvailableMethods) &&
                   string.Equals(BaseType, other.BaseType) &&
                   string.Equals(CalculationId, other.CalculationId) &&
                   string.Equals(MatchId, other.MatchId) &&
                   string.Equals(Method, other.Method) &&
                   string.Equals(Mode, other.Mode) &&
#pragma warning disable 612
                   string.Equals(Output, other.Output) &&
#pragma warning restore 612
                   string.Equals(OutputV2, other.OutputV2) &&
                   string.Equals(Unit, other.Unit) &&
                   string.Equals(UserId, other.UserId)&&
                   string.Equals(MeasureResultType,other.MeasureResultType);
        }

        public override bool Equals(object obj)
        {
            if (obj == null)
                return false;

            if (ReferenceEquals(this, obj))
                return true;

            if (obj.GetType() != GetType())
                return false;

            var other = (MeasureTag)obj;
            return Equals(other);
        }

        public string GetDescrption(Func<string, string> factoryIdTranslator)
        {
            var idDescription = IsCustomize ? UserIdWithoutPrefix : factoryIdTranslator(UserIdWithoutPrefix);
            var outputV2 = string.IsNullOrEmpty(CalculationId)
                ? OutputV2
                : CalculationId;
            outputV2 = outputV2?.Replace(ReportTemplatePrefix, "").Split('|').Last();
#pragma warning disable 612
            return $"{idDescription}_{outputV2 ?? Output}";
#pragma warning restore 612
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = SortedSet<string>.CreateSetComparer().GetHashCode(InternalAvailableMethods);
                hashCode = (hashCode * 397) ^ (BaseType != null ? BaseType.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (CalculationId != null ? CalculationId.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (MatchId != null ? MatchId.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (Method != null ? Method.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (Mode != null ? Mode.GetHashCode() : 0);
#pragma warning disable 612
                hashCode = (hashCode * 397) ^ (Output != null ? Output.GetHashCode() : 0);
#pragma warning restore 612
                hashCode = (hashCode * 397) ^ (OutputV2 != null ? OutputV2.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (Unit != null ? Unit.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (UserId != null ? UserId.GetHashCode() : 0);
                return hashCode;
            }
        }

        private int GetMatchScore(MeasureXmlData data)
        {
            if (Mode != data.ModeId ||
                BaseType != data.TypeId)
                return int.MinValue;

            if (OutputV2 != null)
            {
                if (Method != null && Method != data.MethodId ||
                    OutputV2 != data.OutputId)
                    return int.MinValue;

                var calculationId = CalculationId ?? "";
                if (calculationId != data.CalculationId)
                    return int.MinValue;
            }

            var score = -1;
            var matchId = MatchId ?? "";
            if (matchId != data.MatchId)
                score--;

            if (Unit != data.UnitId)
                score--;

            if (Method != null && Method != data.MethodId)
                score--;

            if (data.OutputId == OutputV2)
                return score; //[-4, -1]

            score -= 4;
#pragma warning disable 612
            var output = Output;
#pragma warning restore 612
            if (data.CalculationId == output)
                return score; //[-8, -5]

            score -= 4;
            if (data.OutputDescriptionKey == output)
                return score; //[-12, -9]

            score -= 4;
            if (data.OutputDescriptionValue.Contains(output))
                return score; //[-16, -13]

            score -= 4;
            if (data.OutputId == output)
                return score; //[-20, -17]

            score -= 4;
            return data.OutputId.Contains(output)
                ? score //[-24, -21]
                : int.MinValue;
        }

        public MeasureTagMatchInfo GetMeasrueMatchInfo(int fetusIndex)
        {
            string worksheetMode;
            switch (Mode)
            {
                case "Tissue":
                case "Flow":
                case "EI":
                    worksheetMode = "2D";
                    break;

                case "Tissue3D":
                    worksheetMode = "3D/4D";
                    break;

                case "TissueTM":
                    worksheetMode = "M";
                    break;

                case "Doppler":
                    worksheetMode = "Doppler";
                    break;

                case "PWVTM":
                    worksheetMode = "PWV";
                    break;

                case "STRAINRATETM":
                    worksheetMode = "Strain TM";
                    break;

                default:
                    worksheetMode = Mode;
                    break;
            }

            var userId = UserId;
#pragma warning disable 612
            var outputId = OutputV2 ?? Output;
#pragma warning restore 612
            var calculationId = CalculationId;
            if (!string.IsNullOrEmpty(calculationId))
            {
                worksheetMode = IsCustomize ? "CustomizeCalculation" : "Calculation";
                userId = calculationId;
                outputId = calculationId.Split('|').Last();
            }

            var matchId = MatchId;
            var valueId = $"{userId}_{outputId}";
            if (!string.IsNullOrEmpty(matchId))
            {
                worksheetMode = matchId.Split('*').First();
                var index = worksheetMode.Length + 1;
                valueId = index > matchId.Length
                    ? null
                    : matchId.Substring(index).Replace('|', '_');
            }

            return new MeasureTagMatchInfo
            {
                FetusIndex = fetusIndex,
                WorkshetModeName = worksheetMode,
                ValueId = valueId,
                UnitId = Unit
            };
        }

        public IEnumerable<MeasureXmlData> Match(MeasureXmlData[] datas)
        {
            return datas.Select(x => (Data: x, Score: GetMatchScore(x)))
                .Where(x => x.Score > int.MinValue)
                .OrderByDescending(x => x.Score)
                .Select(x => x.Data);
        }
    }
}