﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The class define the page size information for report template.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTPageSize : IEquatable<RTPageSize>
    {
        public const double Cm2Px = 3600d / sbyte.MaxValue;

        /// <summary>
        /// Specifies A4 format page size.
        /// </summary>
        public static readonly RTPageSize A4 = new RTPageSize(21 * Cm2Px, 29.7 * Cm2Px);

        /// <summary>
        /// Specifies B5 format page size.
        /// </summary>
        public static readonly RTPageSize B5 = new RTPageSize(517, 730);

        /// <summary>
        /// Gets the height, in pt.
        /// </summary>
        [DataMember]
        public double Height { get; private set; }

        /// <summary>
        /// Gets the width, in pt.
        /// </summary>
        [DataMember]
        public double Width { get; private set; }

        /// <summary>
        /// Initialize <see cref="RTPageSize"/>
        /// </summary>
        /// <param name="width">The page width.</param>
        /// <param name="height">The page height.</param>
        public RTPageSize(double width, double height)
        {
            Height = height;
            Width = width;
        }

        /// <summary>
        /// Tests whether the specified <see cref="RTPageSize"/> structure is identical to this pagesize.
        /// </summary>
        /// <param name="other">The <see cref="RTPageSize" /> structure to compare to the current structure.</param>
        /// <returns>If the width and height are equal, returns true; otherwise, returns false.</returns>
        public bool Equals(RTPageSize other)
        {
            return Height.Equals(other.Height) && Width.Equals(other.Width);
        }

        /// <summary>
        /// Tests whether the specified object is a <see cref="RTPageSize" /> structure and is equivalent to this page size.
        /// </summary>
        /// <param name="obj">The object to compare to this <see cref="RTPageSize" /> structure.</param>
        /// <returns>If the specified object is a <see cref="RTPageSize"/> structure and is identical to the current structure, returns true; otherwise, returns false. </returns>
        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            return obj is RTPageSize && Equals((RTPageSize)obj);
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns>A hash code for the current <see cref="RTPageSize" />.</returns>
        public override int GetHashCode()
        {
            unchecked
            {
                return (Height.GetHashCode() * 397) ^ Width.GetHashCode();
            }
        }
    }
}