﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// Describes the thickness of a frame around a rectangle.
    /// Four values describe the Left, Top, Right, and Bottom sides of the rectangle, respectively.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTThickness : IEquatable<RTThickness>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="RTThickness" /> structure that has specific lengths applied to each side of the rectangle.
        /// </summary>
        /// <param name="left">The thickness for the left side of the rectangle.</param>
        /// <param name="top">The thickness for the upper side of the rectangle.</param>
        /// <param name="right">The thickness for the right side of the rectangle</param>
        /// <param name="bottom">The thickness for the lower side of the rectangle.</param>
        public RTThickness(double left, double top, double right, double bottom)
        {
            Left = left;
            Top = top;
            Right = right;
            Bottom = bottom;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RTThickness" /> structure that has the specified uniform length on each side.
        /// </summary>
        /// <param name="uniformLength">The uniform length applied to all four sides of the bounding rectangle.</param>
        public RTThickness(double uniformLength)
        {
            Left = uniformLength;
            Top = uniformLength;
            Right = uniformLength;
            Bottom = uniformLength;
        }

        public RTThickness() { }

        /// <summary>
        /// Gets or sets the width, in pt, of the left side of the bounding rectangle.
        /// </summary>
        [DataMember]
        public double Left { get; set; }

        /// <summary>
        /// Gets or sets the width, in pt, of the upper side of the bounding rectangle.
        /// </summary>
        [DataMember]
        public double Top { get; set; }

        /// <summary>
        /// Gets or sets the width, in pt, of the right side of the bounding rectangle.
        /// </summary>
        [DataMember]
        public double Right { get; set; }

        /// <summary>
        /// Gets or sets the width, in pt, of the lower side of the bounding rectangle.
        /// </summary>
        [DataMember]
        public double Bottom { get; set; }

        /// <summary>
        /// Tests whether the specified <see cref="RTThickness"/> structure is identical to this thickness.
        /// </summary>
        /// <param name="other">The <see cref="RTThickness" /> structure to compare to the current structure.</param>
        /// <returns>If the specified <see cref="RTThickness"/> structure is identical to the current structure, returns true; otherwise, returns false.</returns>
        public bool Equals(RTThickness other)
        {
            return Left.Equals(other.Left) && Top.Equals(other.Top) && Right.Equals(other.Right) && Bottom.Equals(other.Bottom);
        }

        /// <summary>
        /// Tests whether the specified object is a <see cref="RTThickness" /> structure and is equivalent to this thickness.
        /// </summary>
        /// <param name="obj">The object to compare to this <see cref="RTThickness" /> structure.</param>
        /// <returns>If the specified object is a <see cref="RTThickness"/> structure and is identical to the current structure, returns true; otherwise, returns false. </returns>
        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            return obj is RTThickness && Equals((RTThickness)obj);
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns>A hash code for the current <see cref="RTThickness"/>.</returns>
        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = Left.GetHashCode();
                hashCode = (hashCode * 397) ^ Top.GetHashCode();
                hashCode = (hashCode * 397) ^ Right.GetHashCode();
                hashCode = (hashCode * 397) ^ Bottom.GetHashCode();
                return hashCode;
            }
        }
    }
}