﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.JsonConverters
{
    public abstract class AbstractElementConverter : JsonConverter
    {
        protected void InitializeElement(IElement element, JToken jToken)
        {
            element.Id = jToken["Id"].ToString();
            if (int.TryParse(jToken["Index"].ToString(), out var index))
            {
                element.Index = index;
            }

            var serializerSetting = new JsonSerializerSettings
            {
                Converters = new List<JsonConverter> { new ElementTagJsonConverter() }
            };
            element.Tag = JsonConvert.DeserializeObject<ElementTag>(jToken["Tag"].ToString(), serializerSetting);
            element.MeasureTag = JsonConvert.DeserializeObject<MeasureTag>(jToken["MeasureTag"]?.ToString() ?? "");
            if(Enum.TryParse<HorizontalLayout>(jToken["HorizontalAlignment"].ToString(), out var horizontalAlignment))
            {
                element.HorizontalAlignment = horizontalAlignment;
            }
            if (Enum.TryParse<VerticalLayout>(jToken["VerticalAlignment"].ToString(), out var verticalAlignment))
            {
                element.VerticalAlignment = verticalAlignment;
            }
            
            element.Margin = JsonConvert.DeserializeObject<RTThickness>(jToken["Margin"].ToString());
        }
    }

    /// <summary>
    /// Block element json converter.
    /// </summary>
    public class BlockJsonConverter : AbstractElementConverter
    {
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            var jToken = JToken.ReadFrom(reader);
            var elementType = jToken["ElementType"]["Name"].ToString();
            var elementTagSerializerSetting = new JsonSerializerSettings
            {
                Converters = new List<JsonConverter> { new ElementTagJsonConverter() }
            };
            switch (elementType)
            {
                case "Paragraph":
                    var paragraph = new Paragraph(null);
                    InitializeElement(paragraph, jToken);
                    if (float.TryParse(jToken["LineSpace"].ToString(), out var lineSpace))
                    {
                        paragraph.LineSpace = lineSpace;
                    }

                    var paragraphSerializerSettings = new JsonSerializerSettings
                    {
                        ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                        Converters =
                            new List<JsonConverter>
                            {
                                new StringEnumConverter(),
                                new ElementJsonConverter()
                            }
                    };
                    var elements =
                        JsonConvert.DeserializeObject<List<IElement>>(jToken["Elements"].ToString(), paragraphSerializerSettings);
                    foreach (var element in elements)
                    {
                        if (element is IInLineElemnt iInLineElemnt)
                        {
                            iInLineElemnt.Parent = paragraph;
                            paragraph.AddElement(iInLineElemnt);
                        }
                    }
                    return paragraph;
                case "RTTable":
                    var table =new RTTable(null);
                    InitializeElement(table, jToken);
                    var columnDefinitions =
                        JsonConvert.DeserializeObject<List<RTColumnDefinition>>(jToken["ColumnDefinitions"].ToString());
                    var rowDefinitions =
                        JsonConvert.DeserializeObject<List<RTRowDefinition>>(jToken["RowDefinitions"].ToString());
                    table.SetColumnDefinitions(columnDefinitions);
                    table.SetRowDefinitions(rowDefinitions);
                    var readOnlyCellDictionaryJsonConverter = new JsonSerializerSettings
                    {
                        ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                        Converters =
                            new List<JsonConverter>
                            {
                                new ReadOnlyCellDictionaryJsonConverter()
                            }
                    };
                    var cells =
                        JsonConvert.DeserializeObject<IReadOnlyDictionary<CellPostion, ICell>>(
                            jToken["Cells"].ToString(), readOnlyCellDictionaryJsonConverter);
                    foreach (var cell in cells)
                    {
                        cell.Value.Parent  = table;
                    }
                    table.AutoHide = JsonConvert.DeserializeObject<bool?>(jToken["AutoHide"]?.ToString().ToLower() ?? "") ??
                                     table.AutoHide;
                    table.IsAverageColumnWidth = JsonConvert.DeserializeObject<bool?>(jToken["IsAverageColumnWidth"]?.ToString().ToLower() ?? "") ??
                                     table.IsAverageColumnWidth;
                    table.AllowBreakAcrossPages = JsonConvert.DeserializeObject<bool?>(jToken["AllowBreakAcrossPages"]?.ToString().ToLower() ?? "") ??
                                     table.AllowBreakAcrossPages;
                    table.IsFoldOccupy = JsonConvert.DeserializeObject<bool?>(jToken["IsFoldOccupy"]?.ToString().ToLower() ?? "") ??
                                     table.IsFoldOccupy;
                    table.Cells = cells;
                    return table;
                case "ImageList":
                    return JsonConvert.DeserializeObject<InputImageList>(jToken.ToString(), elementTagSerializerSetting);
                case "AtlasElement":
                    return JsonConvert.DeserializeObject<AtlasElement>(jToken.ToString(), elementTagSerializerSetting);
            }

            return null;
        }

        public override bool CanConvert(Type objectType)
        {
            return typeof(IBlockElement).IsAssignableFrom(objectType);
        }
    }

    /// <summary>
    /// Element json converter.
    /// </summary>
    public class ElementJsonConverter : AbstractElementConverter
    {
        /// <summary>Writes the JSON representation of the object.</summary>
        /// <param name="writer">The <see cref="T:Newtonsoft.Json.JsonWriter" /> to write to.</param>
        /// <param name="value">The value.</param>
        /// <param name="serializer">The calling serializer.</param>
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
           
        }

        /// <summary>Reads the JSON representation of the object.</summary>
        /// <param name="reader">The <see cref="T:Newtonsoft.Json.JsonReader" /> to read from.</param>
        /// <param name="objectType">Type of the object.</param>
        /// <param name="existingValue">The existing value of object being read.</param>
        /// <param name="serializer">The calling serializer.</param>
        /// <returns>The object value.</returns>
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            var jToken = JToken.ReadFrom(reader);
            var value = jToken.ToString();
            var elementType = jToken["ElementType"]["Name"].ToString();
            var serializerSetting = new JsonSerializerSettings
            {
                Converters = new List<JsonConverter> { new StringEnumConverter(), new RTColorJsonConverter(), new ElementTagJsonConverter() }
            };
            switch (elementType)
            {
                case "Space":
                    var space = JsonConvert.DeserializeObject<Space>(value);
                    if (space.SpaceUnitCount > 0)
                        space.SpaceWidth = space.SpaceUnitCount * 9;
                    return space;
                case "Line":
                    return JsonConvert.DeserializeObject<Line>(value, serializerSetting);
                case "StaticImage":
                    return JsonConvert.DeserializeObject<StaticImage>(value, serializerSetting);
                case "InputImage":
                    return JsonConvert.DeserializeObject<InputImage>(value, serializerSetting);
                case "StaticText":
                    return JsonConvert.DeserializeObject<StaticText>(value, serializerSetting);
                case "InputText":
                    var inputText = JsonConvert.DeserializeObject<InputText>(value, serializerSetting);
                    SetLineWidth(inputText);
                    return inputText;
                case "SingleSelected":
                    var singleSelected = JsonConvert.DeserializeObject<SingleSelected>(value, serializerSetting);
                    SetLineWidth(singleSelected);
                    return singleSelected;
                //case "SingleSelectedVertical":
                //    var singleSelectedVertical = JsonConvert.DeserializeObject<SingleSelectedVertical>(value, serializerSetting);
                //    SetLineWidth(singleSelectedVertical);
                //    return singleSelectedVertical;
                case "MultiSelected":
                    var multiSelected = JsonConvert.DeserializeObject<MultiSelected>(value, serializerSetting);
                    SetLineWidth(multiSelected);
                    return multiSelected;
                case "MultiSelectedVertical":
                    var mutliSelectedVertical = JsonConvert.DeserializeObject<MultiSelectedVertical>(value, serializerSetting);
                    SetLineWidth(mutliSelectedVertical);
                    return mutliSelectedVertical;
                case "DateTime":
                    return JsonConvert.DeserializeObject<DateTimeElement>(value, serializerSetting);
                case "PageNumber":
                    return JsonConvert.DeserializeObject<PageNumberElement>(value, serializerSetting);
            }

            return null;
        }

        private void SetLineWidth(IElement element)
        {
            if (element is ITextElement text)
            {
                if (text.LineWidth <= 0 && text.LineLength > 0)
                {
                    if ((ElementTag.ReportSummary.Equals(text.Tag) || ElementTag.ReportDescription.Equals(text.Tag))&& text.LineLength>46)
                    {
                        text.LineWidth = 46 * 10.5;
                    }
                    else
                    {
                        text.LineWidth = text.LineLength * 10.5;
                    }
                }                
            }
        }

        /// <summary>
        /// Determines whether this instance can convert the specified object type.
        /// </summary>
        /// <param name="objectType">Type of the object.</param>
        /// <returns>
        /// 	<c>true</c> if this instance can convert the specified object type; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanConvert(Type objectType)
        {
            return typeof(IElement).IsAssignableFrom(objectType);
        }
    }
}