﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.JsonConverters
{
    public class ReadOnlyCellDictionaryJsonConverter : JsonConverter
    {
        /// <summary>Writes the JSON representation of the object.</summary>
        /// <param name="writer">The <see cref="T:Newtonsoft.Json.JsonWriter" /> to write to.</param>
        /// <param name="value">The value.</param>
        /// <param name="serializer">The calling serializer.</param>
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            if (value is IReadOnlyDictionary<CellPostion, ICell> dictionary)
            {
                //Loop over the Dictionary array and write each one
                writer.WriteStartArray();
                foreach (var entry in dictionary)
                {
                    //Write Key (Vector) 
                    serializer.Serialize(writer, entry.Key);
                    //Write Value (Collection)
                    serializer.Serialize(writer, entry.Value);
                }
                writer.WriteEndArray();
                return;
            }
            writer.WriteStartObject();
            writer.WriteEndObject();
        }

        /// <summary>Reads the JSON representation of the object.</summary>
        /// <param name="reader">The <see cref="T:Newtonsoft.Json.JsonReader" /> to read from.</param>
        /// <param name="objectType">Type of the object.</param>
        /// <param name="existingValue">The existing value of object being read.</param>
        /// <param name="serializer">The calling serializer.</param>
        /// <returns>The object value.</returns>
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue,
            JsonSerializer serializer)
        {
            //Where to re-create the json data into 
            var dict = new Dictionary<CellPostion, ICell>();
            if (reader.TokenType == JsonToken.StartArray)
            {
                var jArray = JArray.Load(reader);
                if (jArray != null && jArray.Count >= 2)
                {
                    //Do the loop faster with +=2
                    for (var i = 0; i < jArray.Count; i += 2)
                    {
                        //first item = key
                        var firstData = jArray[i + 0].ToString();
                        //second item = value
                        var secondData = jArray[i + 1].ToString();

                        //Create Vector2Int key data 
                        var cellPosition = JsonConvert.DeserializeObject<CellPostion>(firstData);

                        //Create Collection value data
                        var settings = new JsonSerializerSettings
                        {
                            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                            Converters = new List<JsonConverter>
                            {
                                new StringEnumConverter(),
                                new CellJsonConverter(),
                                new ElementTagJsonConverter()
                            }
                        };
                        var cell = JsonConvert.DeserializeObject<RTCell>(secondData, settings);

                        //Add both Key and Value to the Dictionary if key doesnt exit yet
                        if (!dict.ContainsKey(cellPosition))
                            dict.Add(cellPosition, cell);
                    }
                }
            }

            return dict;
        }

        /// <summary>
        /// Determines whether this instance can convert the specified object type.
        /// </summary>
        /// <param name="objectType">Type of the object.</param>
        /// <returns>
        /// 	<c>true</c> if this instance can convert the specified object type; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanConvert(Type objectType)
        {
            return typeof(IReadOnlyDictionary<CellPostion, ICell>).IsAssignableFrom(objectType);
        }
    }
}