﻿using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.JsonConverters
{
    public class ReportTemplateDocumentJsonConverter : JsonConverter
    {
        /// <summary>Writes the JSON representation of the object.</summary>
        /// <param name="writer">The <see cref="T:Newtonsoft.Json.JsonWriter" /> to write to.</param>
        /// <param name="value">The value.</param>
        /// <param name="serializer">The calling serializer.</param>
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
        }

        /// <summary>Reads the JSON representation of the object.</summary>
        /// <param name="reader">The <see cref="T:Newtonsoft.Json.JsonReader" /> to read from.</param>
        /// <param name="objectType">Type of the object.</param>
        /// <param name="existingValue">The existing value of object being read.</param>
        /// <param name="serializer">The calling serializer.</param>
        /// <returns>The object value.</returns>
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            var jToken = JToken.ReadFrom(reader);
            if (string.IsNullOrEmpty(jToken.ToString()))
            {
                return null;
            }
            var reportTemplateDocument = new ReportTemplateDocument();
            reportTemplateDocument.Id = jToken["Id"].ToString();
            if (bool.TryParse(jToken["IsCustom"].ToString(), out var isCustom))
            {
                reportTemplateDocument.IsCustom = isCustom;
            }

            if (DateTime.TryParse(jToken["UpdateTime"]?.ToString(), out var updateTime))
            {
                reportTemplateDocument.UpdateTime = updateTime;
            }

            if (jToken["Version"] == null || string.IsNullOrEmpty(jToken["Version"].ToString()))
            {
                reportTemplateDocument.Version = "1.0_1.5.0.8564";
            }
            else
            {
                reportTemplateDocument.Version = jToken["Version"].ToString();
            }

            reportTemplateDocument.Name = jToken["Name"].ToString();
            reportTemplateDocument.Author = jToken["Author"].ToString();
            reportTemplateDocument.PagePadding= JsonConvert.DeserializeObject<RTThickness>(jToken["PagePadding"].ToString());
            reportTemplateDocument.PageSize = JsonConvert.DeserializeObject<RTPageSize>(jToken["PageSize"].ToString());
            if (jToken["HeaderHeight"] != null && !string.IsNullOrEmpty(jToken["HeaderHeight"].ToString()))
            {
                reportTemplateDocument.HeaderHeight = Convert.ToDouble(jToken["HeaderHeight"].ToString());
            }
            else
            {
                reportTemplateDocument.HeaderHeight = null;
            }
            if (jToken["FooterHeight"] != null && !string.IsNullOrEmpty(jToken["FooterHeight"].ToString()))
            {
                reportTemplateDocument.FooterHeight = Convert.ToDouble(jToken["FooterHeight"].ToString());
            }
            else
            {
                reportTemplateDocument.FooterHeight = null;
            }

            if (jToken["HeaderDistance"] != null && !string.IsNullOrEmpty(jToken["HeaderDistance"].ToString()))
            {
                reportTemplateDocument.HeaderDistance = Convert.ToDouble(jToken["HeaderDistance"].ToString());
            }

            if (jToken["FooterDistance"] != null && !string.IsNullOrEmpty(jToken["FooterDistance"].ToString()))
            {
                reportTemplateDocument.FooterDistance = Convert.ToDouble(jToken["FooterDistance"].ToString());
            }
            reportTemplateDocument.BaseFontSize =
                JsonConvert.DeserializeObject<double?>(jToken["BaseFontSize"]?.ToString() ?? "") ??
                reportTemplateDocument.BaseFontSize;
            reportTemplateDocument.InvertColor =
                JsonConvert.DeserializeObject<bool?>(jToken["InvertColor"]?.ToString().ToLower() ?? "") ??
                reportTemplateDocument.InvertColor;
            var tags =
                JsonConvert.DeserializeObject<Dictionary<string, ElementTag>>(jToken["Tags"].ToString());
            if (tags != null && tags.Any())
            {
                foreach (var tag in tags)
                {
                    reportTemplateDocument.Tags.Add(tag.Key, tag.Value);
                }
            }

            var value = jToken["CustomizeTranslation"]?.ToString() ?? "";
            reportTemplateDocument.CustomizeTranslation = JsonConvert
                .DeserializeObject<Dictionary<string, Dictionary<string, string>>>(value);
            value = jToken["CustomizeComments"]?.ToString() ?? "";
            reportTemplateDocument.CustomizeComments = JsonConvert.DeserializeObject<HashSet<string>>(value);
            value = jToken["CustomizeReportComments"]?.ToString() ?? "";
            reportTemplateDocument.CustomizeReportComments = JsonConvert.DeserializeObject<HashSet<string>>(value);
            value = jToken["CustomizeCalculations"]?.ToString() ?? "";
            reportTemplateDocument.CustomizeCalculations =
                JsonConvert.DeserializeObject<HashSet<CustomizeCalculation>>(value);
            var settings = new JsonSerializerSettings
            {
                ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                Converters =
                    new List<JsonConverter> { new BlockJsonConverter() }
            };

            var headers =
                JsonConvert.DeserializeObject<List<IBlockElement>>(jToken["Header"].ToString(), settings);
            foreach (var element in headers)
            {
                element.Parent = reportTemplateDocument;
                reportTemplateDocument.AddHeaderBlock(element);
            }

            var blocks =
                JsonConvert.DeserializeObject<List<IBlockElement>>(jToken["Blocks"].ToString(), settings);
            foreach (var element in blocks)
            {
                element.Parent = reportTemplateDocument;
                reportTemplateDocument.AddBlock(element);
            }

            var footers =
                JsonConvert.DeserializeObject<List<IBlockElement>>(jToken["Footer"].ToString(), settings);
            foreach (var element in footers)
            {
                element.Parent = reportTemplateDocument;
                reportTemplateDocument.AddFooterBlock(element);
            }
            return reportTemplateDocument;
        }

        /// <summary>
        /// Determines whether this instance can convert the specified object type.
        /// </summary>
        /// <param name="objectType">Type of the object.</param>
        /// <returns>
        /// 	<c>true</c> if this instance can convert the specified object type; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanConvert(Type objectType)
        {
            return typeof(IReportTemplateDocument).IsAssignableFrom(objectType);
        }
    }
}