﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Vinno.IUS.Common.Log;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The report template element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public abstract class Element: IElement
    {
        private ElementTag _tag;
        private MeasureTag _measureTag;
        public event EventHandler OnElementPropertyChanged;

        /// <summary>
        /// Gets or sets the Id.
        /// </summary>
        [DataMember]
        public string Id { get; set; }

        /// <summary>
        /// Gets or sets the index.
        /// </summary>
        [DataMember]
        public int Index { get; set; }

        /// <summary>
        /// Gets the parent element.
        /// </summary>
        [DataMember]
        public IReportElement Parent { get; set; }

        /// <summary>
        /// Gets or sets the element tag.
        /// </summary>
        [DataMember]
        public ElementTag Tag
        {
            get => _tag;
            set
            {
                if (_tag != value)
                {
                    _tag = value;
                }
            }
        }

        [DataMember]
        public MeasureTag MeasureTag
        {
            get => _measureTag;
            set
            {
                if (_measureTag != value)
                {
                    _measureTag = value;
                }
            }
        }


        /// <summary>
        /// Gets or sets the HorizontalAlignment.
        /// </summary>
        [DataMember]
        public HorizontalLayout HorizontalAlignment { get; set; }

        /// <summary>
        /// Gets or sets the VerticalAlignment.
        /// </summary>
        [DataMember]
        public VerticalLayout VerticalAlignment { get; set; }

        /// <summary>
        /// Gets or sets the margin.
        /// </summary>
        [DataMember]
        public RTThickness Margin { get; set; }

        /// <summary>
        /// Gets or sets the element type.
        /// </summary>
        [DataMember]
        public ElementType ElementType { get; protected set; } 

        /// <summary>
        /// Initialize a <see cref="Element"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        protected Element(IReportElement parent)
        {
            Id = Guid.NewGuid().ToString();
            Margin = new RTThickness(0);
            Parent = parent;
            HorizontalAlignment = HorizontalLayout.Stretch;
            VerticalAlignment = VerticalLayout.Stretch;
        }

        /// <summary>
        /// Get ReportTemplateDocument
        /// </summary>
        public ReportTemplateDocument GetReportTemplateDocument()
        {
            return GetReportTemplateDocument(this);
        }

        public abstract IElement Clone();

        /// <summary>
        /// Get Fetus Tag
        /// </summary>
        public ElementTag GetFetusTag()
        {
            return GetLogicalAncestry().OfType<RTTable>()
                       .Select(x => x.Tag)
                       .FirstOrDefault(ElementTag.FetusTags.Contains)
                   ?? ElementTag.FetusA;
        }

        public MeasureTagMatchInfo GetMeasrueMatchInfo()
        {
            var fetusTag = GetFetusTag();
            var fetusIndex = ElementTag.FetusTags.IndexOf(fetusTag);
            if (MeasureTag == null)
                throw new NullReferenceException("MeasureTag");

            return MeasureTag.GetMeasrueMatchInfo(fetusIndex);
        }

        /// <summary>
        /// Get Logical Ancestry
        /// </summary>
        public IEnumerable<IElement> GetLogicalAncestry()
        {
            var element = Parent as IElement;
            while (element != null)
            {
                yield return element;
                element = element.Parent as IElement;
            }
        }

        protected void InitializeCloneElement(IElement element)
        {
            element.Id = Id;
            element.Tag = Tag;
            element.MeasureTag = MeasureTag;
            element.Index = Index;
            element.HorizontalAlignment = HorizontalAlignment;
            element.VerticalAlignment = VerticalAlignment;
            element.Margin = new RTThickness(Margin.Left, Margin.Top, Margin.Right, Margin.Bottom);
        }

        private ReportTemplateDocument GetReportTemplateDocument(IElement element)
        {
            if (element == null)
                return null;
            if (element.Parent is ReportTemplateDocument reportTemplateDocument)
            {
                return reportTemplateDocument;
            }

            return GetReportTemplateDocument(element.Parent as IElement);
        }

        /// <summary>
        /// Judge whether the tag exists 
        /// </summary>
        public virtual bool HasTagIn(ReportInfo reportInfo)
        {
            if (Tag != null && reportInfo.ElementValues.ContainsKey(Tag))
                return true;

            if (MeasureTag == null)
                return false;

            var fetusTag = GetFetusTag();
            if (reportInfo.MeasureElementValues?.Count > 0 && reportInfo.MeasureElementValues.ContainsKey(fetusTag))
            {
                return reportInfo.MeasureElementValues[fetusTag].ContainsKey(MeasureTag);
            }
            else
            {
                return false;
            }
        }       

        public virtual bool IsVisibleIn(ReportInfo reportInfo)
        {
            return true;
        }

        public virtual bool IsFoldVisibleIn(ReportInfo reportInfo)
        {
            return true;
        }

        /// <summary>
        /// Delete self
        /// </summary>
        public virtual void Delete()
        {
            if (Parent == null)
            {
                return;
            }

            var reportTemplateDocument = GetReportTemplateDocument();
            if (Parent is Paragraph paragraph)
            {
                if (paragraph.Elements.Count == 1)
                {
                    paragraph.Delete();
                }
                else
                {
                    paragraph.RemoveElement(this);
                }
                reportTemplateDocument.RaiseCurrentElementChanged(null);
            }

            if (Parent is RTCell cell)
            {
                cell.RemoveBlock(this as IBlockElement);
                reportTemplateDocument.RaiseCurrentElementChanged(null);
            }

            if (Parent is IReportTemplateDocument)
            {
                if (reportTemplateDocument.Blocks.Any(b => b.Id == Id))
                {
                    reportTemplateDocument.RemoveBlock(this as IBlockElement);
                    reportTemplateDocument.RaiseCurrentElementChanged(null);
                    return;
                }

                if (reportTemplateDocument.Footer.Any(b => b.Id == Id))
                {
                    reportTemplateDocument.RemoveFooterBlock(this as IBlockElement);
                    reportTemplateDocument.RaiseCurrentElementChanged(null);
                    return;
                }

                if (reportTemplateDocument.Header.Any(b => b.Id == Id))
                {
                    reportTemplateDocument.RemoveHeaderBlock(this as IBlockElement);
                    reportTemplateDocument.RaiseCurrentElementChanged(null);
                }
            }
        }
        /// <summary>
        /// UnDelete self
        /// </summary>
        public virtual void UnDelete(bool? isHeader,int originalIndex)
        {
            try
            {
                if (Parent == null)
                {
                    return;
                }

                var reportTemplateDocument = GetReportTemplateDocument();
                if (Parent is Paragraph paragraph)
                {
                    if (paragraph.Parent is RTCell rtcell && rtcell.Blocks.Count == 0)
                    {
                        rtcell.InsertBlock(paragraph, 0);
                    }
                    else
                    {
                        if (paragraph.IsEmptyFirst)
                        {
                            paragraph.InsertElement(this as IInLineElemnt, Index);
                        }
                        else
                        {
                            if (paragraph.Elements.Count == 1 && paragraph.Elements[0].Id == Id)
                            {
                                reportTemplateDocument.InsertBlock(paragraph, paragraph.Index);
                            }
                            else
                            {
                                paragraph.InsertElement(this as IInLineElemnt, Index);
                            }
                        }
                    }
                }

                if (Parent is ICell cell)
                {
                    cell.InsertBlock(this as IBlockElement, Index);
                }

                if (Parent is IReportTemplateDocument)
                {
                    if (isHeader == null)
                    {
                        if (this is Paragraph par&& par.IsEmptyFirst)
                        {
                            par.ClearElement();
                            reportTemplateDocument.InsertBlock(par, originalIndex);
                        }
                        else
                        {
                            reportTemplateDocument.InsertBlock(this as IBlockElement, originalIndex);
                        }
                    }
                    else
                    {
                        if (isHeader.Value)
                        {
                            reportTemplateDocument.InsertHeaderBlock(this as IBlockElement, originalIndex);
                        }
                        else
                        {
                            reportTemplateDocument.InsertFooterBlock(this as IBlockElement, originalIndex);
                        }
                    }
                }
                reportTemplateDocument.RaiseCurrentElementChanged(this);
            }
            catch (Exception e)
            {
                Logger.WriteLineError($"UnDelete element error {e}");
            }
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public virtual void UpdateElementProperty(IElement element)
        {
            var reportTemplateDocument = GetReportTemplateDocument();
            Id = element.Id;
            Tag = element.Tag;
            MeasureTag = element.MeasureTag;
            Index = element.Index;
            HorizontalAlignment = element.HorizontalAlignment;
            VerticalAlignment = element.VerticalAlignment;
            if (element.Margin != null)
            {
                Margin.Left = element.Margin.Left;
                Margin.Top = element.Margin.Top;
                Margin.Right = element.Margin.Right;
                Margin.Bottom = element.Margin.Bottom;
            }

            OnElementPropertyChanged?.Invoke(this, EventArgs.Empty);
            reportTemplateDocument.RaiseCurrentElementChanged(null);
            reportTemplateDocument.RaiseCurrentElementChanged(this);
        }
    }
}