﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common;
using Vinno.IUS.Common.Args;
using Vinno.IUS.Common.IO;
using Vinno.IUS.Common.Log;

namespace Vinno.vCloud.Report.Models
{
    [DataContract(Namespace = "")]
    public enum ValueType
    {
        [EnumMember]
        String,
        [EnumMember]
        Integer,
        [EnumMember]
        Float,
        [EnumMember]
        Datetime,
        [EnumMember]
        FileImage,
        [EnumMember]
        BufferImage,
        [EnumMember]
        ImageList,
        [EnumMember]
        StringList,
        [EnumMember]
        CommentString,
        [EnumMember]
        CommentStringList, 
    }

    /// <summary>
    /// Define the element value class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public abstract class ElementValue<T> : IElementValue
    {
        private T _value;

        public event EventHandler ValueChanged;

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        [DataMember]
        public T Value
        {
            get => _value;
            set
            {
                if (ValueEquals(value))
                {
                    return;
                }

                _value = value;
                ValueChanged?.Invoke(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Gets the value type.
        /// </summary>
        [DataMember]
        public ValueType ValueType { get; protected set; }

        protected ElementValue(T value)
        {
            Value = value;
        }

        protected abstract bool ValueEquals(T value);

        public object GetValue()
        {
            return Value;
        }

        public void SetValue(object value)
        {
            _value = ConvertValue(value);
        }

        /// <summary>
        /// Set the value and raise the value changed.
        /// </summary>
        /// <param name="value">The new value.</param>
        public void SetValueWithValueChanged(string value)
        {
            Value = ConvertValue(value);
        }

        public abstract IElementValue Clone();

        protected abstract T ConvertValue(object value);      
    }

    public interface IElementValue
    {
        event EventHandler ValueChanged;

        ValueType ValueType { get; }

        object GetValue();

        /// <summary>
        /// Set the value without the value changed.
        /// </summary>
        void SetValue(object value);

        /// <summary>
        /// Set the value and raise the value changed.
        /// </summary>
        /// <param name="value">The new value.</param>
        void SetValueWithValueChanged(string value);

        IElementValue Clone();
    }


    /// <summary>
    /// The text value class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class TextElementValue : ElementValue<string>
    {
        public TextElementValue(string value) : base(value)
        {
            ValueType = ValueType.String;
        }

        public override IElementValue Clone()
        {
            return new TextElementValue(Value);
        }

        protected override bool ValueEquals(string value)
        {
            return value == Value;
        }

        protected override string ConvertValue(object value)
        {
            return value?.ToString();
        }
    }

    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class IntegerElementValue : ElementValue<int?>
    {
        public IntegerElementValue(int? value) : base(value)
        {
            ValueType = ValueType.Integer;
        }

        protected override bool ValueEquals(int? value)
        {
            return value == Value;
        }

        public override IElementValue Clone()
        {
            return new IntegerElementValue(Value);
        }

        protected override int? ConvertValue(object value)
        {
            if (value == null)
            {
                return null;
            }

            if (int.TryParse(value.ToString(), out var intValue))
            {
                return intValue;
            }

            return null;
        }
    }

    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class FloatElementValue : ElementValue<float?>
    {
        private const double Tolerance = 0.0001;

        public FloatElementValue(float? value) : base(value)
        {
            ValueType = ValueType.Integer;
        }

        public override IElementValue Clone()
        {
            return new FloatElementValue(Value);
        }

        protected override bool ValueEquals(float? value)
        {
            if (value == null && Value != null || value != null && Value == null)
            {
                return false;
            }

            if (value == null && Value == null)
            {
                return true;
            }

            return value != null && Math.Abs(value.Value - Value.Value) < Tolerance;
        }

        protected override float? ConvertValue(object value)
        {
            if (value == null)
            {
                return null;
            }

            if (float.TryParse(value.ToString(), out var floatValue))
            {
                return floatValue;
            }

            return null;
        }
    }

    /// <summary>
    /// The date time element value.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class DateTimeElementValue : ElementValue<DateTime?>
    {
        public DateTimeElementValue(DateTime? dateTime) : base(dateTime)
        {
            ValueType = ValueType.Datetime;
        }

        public override IElementValue Clone()
        {
            return new DateTimeElementValue(Value);
        }

        protected override bool ValueEquals(DateTime? value)
        {
            return value == Value;
        }

        protected override DateTime? ConvertValue(object value)
        {
            if (value == null)
            {
                return null;
            }

            if (DateTime.TryParse(value.ToString(), out var dateTime))
            {
                return dateTime;
            }

            return null;
        }
    }

    /// <summary>
    /// The image value.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public abstract class ImageElementValue<T> : ElementValue<T>, IImageElementValue
    {
        protected ImageElementValue(T value) : base(value)
        {
        }
    }

    public interface IImageElementValue : IElementValue
    {
    }

    /// <summary>
    /// The file image value.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class FileImageValue : ImageElementValue<string>
    {
        /// <summary>
        /// image buffer
        /// </summary>
        [JsonIgnore]
        public IBuffer ImageBuffer { get; set; }

        [DataMember]
        public string ExamDataId { get; set; }

        public FileImageValue(string value) : base(value)
        {
            ValueType = ValueType.FileImage;
        }

        public override IElementValue Clone()
        {
            return new FileImageValue(Value) { ExamDataId = ExamDataId ,ImageBuffer=ImageBuffer};
        }

        protected override bool ValueEquals(string value)
        {
            return value == Value;
        }

        protected override string ConvertValue(object value)
        {
            return value?.ToString();
        }
    }

    /// <summary>
    /// The buffer image value.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class BufferImageValue : ImageElementValue<byte[]>
    {
        public BufferImageValue(byte[] value) : base(value)
        {
            ValueType = ValueType.BufferImage;
        }

        public override IElementValue Clone()
        {
            return new BufferImageValue(Value);
        }

        protected override bool ValueEquals(byte[] value)
        {
            return value == Value;
        }

        protected override byte[] ConvertValue(object value)
        {
            return (byte[])value;
        }
    }

    /// <summary>
    /// The list value class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public abstract class ListElementValue<T> : IElementValue where T : IElementValue
    {
        private NotificationList<T> _listElementValues;

        public event EventHandler<CollectionChangedArgs<T>> ListValueChanged;

        public event EventHandler ValueChanged;

        [DataMember]
        public IEnumerable<T> Value
        {
            get => _listElementValues.ToList();
            set => _listElementValues = new NotificationList<T>(value);
        }

        /// <summary>
        /// Gets the value type.
        /// </summary>
        [DataMember]
        public ValueType ValueType { get; protected set; }

        protected ListElementValue(IEnumerable<T> value)
        {
            _listElementValues = new NotificationList<T>(value, CollectionChangedCallback);
        }

        public abstract IElementValue Clone();

        private void CollectionChangedCallback(object sender, CollectionChangedArgs<T> collectionChangedArgs)
        {
            ListValueChanged?.Invoke(this, collectionChangedArgs);
        }

        public object GetValue()
        {
            return _listElementValues.ToList();
        }

        public void SetValue(object value)
        {
            _listElementValues.Clear();
            if (value is IEnumerable<T> lists)
            {
                foreach (var item in lists)
                {
                    _listElementValues.Add(item);
                }
            }
        }

        public void AddValue(T value)
        {
            if (value == null)
            {
                Logger.WriteLineError($"Trying to add a null value in element value collection");
                return;
            }
            if (_listElementValues.Any(v => v.GetValue() == value.GetValue()))
            {
                return;
            }

            _listElementValues.Add(value);
        }

        public void RemoveValue(T value)
        {
            if (value == null)
            {
                Logger.WriteLineError($"Trying to get a null value in element value collection");
                return;
            }
            if (_listElementValues == null)
            {
                Logger.WriteLineError($"Empty colletion appeared while get a value from current colletion");
                return;
            }
            var existedValue = _listElementValues.FirstOrDefault(v => v.GetValue().Equals(value.GetValue()));
            if (existedValue != null)
            {
                _listElementValues.Remove(existedValue);
            }
        }

        public void SetValueWithValueChanged(string value)
        {
        }
    }

    /// <summary>
    /// The image list value.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class ImageListElementValue : ListElementValue<IImageElementValue>
    {
        /// <summary>
        /// Initialize a <see cref="ImageListElementValue"/>
        /// </summary>
        public ImageListElementValue(IEnumerable<IImageElementValue> imageElementValues) : base(imageElementValues)
        {
            ValueType = ValueType.ImageList;
        }

        public override IElementValue Clone()
        {
            return new ImageListElementValue(Value);
        }
    }

    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class StringListElementValue : ListElementValue<TextElementValue>
    {
        /// <summary>
        /// Initialize a <see cref="ImageListElementValue"/>
        /// </summary>
        public StringListElementValue(IEnumerable<TextElementValue> textElementValues) : base(textElementValues)
        {
            ValueType = ValueType.StringList;
        }

        public override IElementValue Clone()
        {
            return new StringListElementValue(Value);
        }
    }

    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class CommentTextElementValue : TextElementValue
    {
        [DataMember]
        public bool IsTitie { get; set; }

        [DataMember]
        public bool IsBold { get; set; }

        [DataMember]
        public bool IsItalic { get; set; }

        [DataMember]
        public bool IsUnderline { get; set; }

        [DataMember]
        public bool IsNoWrap { get; set; }
        public CommentTextElementValue(string value, bool isTitile) : base(value)
        {
            ValueType = ValueType.CommentString;
            IsTitie = isTitile;
        }
        public CommentTextElementValue(string value, bool isTitile, bool isBold, bool isItalic, bool isUnderline, bool isNoWrap) : base(value)
        {
            ValueType = ValueType.CommentString;
            IsTitie = isTitile;
            IsBold = isBold;
            IsItalic = isItalic;
            IsUnderline = isUnderline;
            IsNoWrap = isNoWrap;
        }

        public override IElementValue Clone()
        {
            return new CommentTextElementValue(Value, IsTitie, IsBold, IsItalic, IsUnderline, IsNoWrap);
        }
    }

    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class ReportCommentsElementValue : ListElementValue<CommentTextElementValue>
    {
        public ReportCommentsElementValue(IEnumerable<CommentTextElementValue> textElementValues) : base(textElementValues)
        {
            ValueType = ValueType.CommentStringList;
        }

        public override IElementValue Clone()
        {
            return new ReportCommentsElementValue(Value);
        }
    }
}