﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The input image list element.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class InputImageList: Element, IBlockElement, IInputImageListElement
    {
        private int _column;
        public const float ImageListCellSpacing = 1.5f;
        public event EventHandler ColumnChanged;

        public IReadOnlyList<IElement> Elements { get; private set; }

        [JsonIgnore]
        [DataMember]
        private List<IElement> DataContractElements
        {
            get => new List<IElement>(Elements);
            set => Elements = value;
        }

        /// <summary>
        /// Gets or sets the image width.
        /// </summary>
        [DataMember]
        public double ImageWidth { get; set; }

        /// <summary>
        /// Gets or sets the image height.
        /// </summary>
        [DataMember]
        public double ImageHeight { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the image has border or not.
        /// </summary>
        [DataMember]
        public bool HasImageBorder { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate the image count for each row.
        /// </summary>
        [DataMember]
        public int Column
        {
            get => _column;
            set
            {
                if (_column!=value)
                {
                    _column = value;
                    ColumnChanged?.Invoke(_column, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Ges or sets the value to indicate whether the element is readonly or not.
        /// </summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>
        /// </summary>
        [DataMember]
        public bool TransparentBackground { get; set; }

        [DataMember]
        public bool HasGap { get; set; }

        /// <summary>
        /// Initialize a <see cref="InputImageList"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public InputImageList(IReportElement parent) : base(parent)
        {
            Elements = new List<IElement>();
            ElementType = ElementType.ImageList;
            Column = 2;
            HorizontalAlignment = HorizontalLayout.Left;
            HasGap = true;
        }

        /// <summary>
        /// Get default element value.
        /// </summary>
        /// <returns></returns>
        public IElementValue GetDefaultValue()
        {
            return new ImageListElementValue(new List<IImageElementValue>());
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var inputImageList = new InputImageList(null);
            InitializeCloneElement(inputImageList);
            inputImageList.ImageHeight = ImageHeight;
            inputImageList.ImageWidth = ImageWidth;
            inputImageList.HasImageBorder = HasImageBorder;
            inputImageList.Column = Column;
            inputImageList.IsReadOnly = IsReadOnly;
            inputImageList.TransparentBackground = TransparentBackground;
            inputImageList.HasGap = HasGap;
            return inputImageList;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is InputImageList inputImageList)
            {
                ImageHeight = inputImageList.ImageHeight;
                ImageWidth = inputImageList.ImageWidth;
                HasImageBorder = inputImageList.HasImageBorder;
                Column = inputImageList.Column;
                IsReadOnly = inputImageList.IsReadOnly;
                TransparentBackground = inputImageList.TransparentBackground;
                HasGap = inputImageList.HasGap;
                base.UpdateElementProperty(inputImageList);
            }
        }
    }
}