﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Xml.Linq;
using Vinno.IUS.Common.Utilities;

namespace Vinno.vCloud.Report.Models
{
    [DataContract]
    public class MeasureXmlData
    {
        private static readonly ILookup<string, string> CalculationIdsByRef;
        private static readonly Dictionary<string, CustomizeCalculation> CalculationsById;
        private static readonly List<MeasureXmlData> SimpleDatas;
        private static readonly Dictionary<string, string> Translator;
        private static readonly Dictionary<string, XElement> Types;

        static MeasureXmlData()
        {
            Types = GetDocument("MeasureBaseTypes.xml")
                        .Element("BaseTypes")
                        ?.Elements("Item")
                        .ToDictionary(x => x.GetAttributeValue("Id"))
                    ?? new Dictionary<string, XElement>();
            UnitExs = GetDocument("MeasureUnits.xml")
                .Elements("MeasureUnits")
                .Elements("UnitExtension.UnitEx")
                .ToDictionary(x => x.GetElementValue("Unit"));
            var noUnitValue = TranslateHelper.Translate("NoUnit");
            UnitExs["None"].SetElementValue("Description", noUnitValue);
            UnitExs["fraction"].SetElementValue("Description", noUnitValue);
            UnitExGroups = UnitExs.Values
                .ToLookup(x => x.GetElementValue("BaseUnit"));
            Translator = GetDocument("English.xml")
                .Descendants("LocaleItem")
                .GroupBy(x => x.GetAttributeValue("key"), x => x.Value)
                .Where(x => x.Key.StartsWith("Measure."))
                .ToDictionary(x => x.Key, x => x.First());
            var calculationDocument = GetDocument("MeasureCalculations.xml");
            var validCalculationIds = calculationDocument.Descendants("Group")
                .SelectMany(x => x.GetAttributeValue("Calculations").SplitSemicolon())
                .ToDictionary(x => x);
            var calculations = calculationDocument.Descendants("Calculation")
                .Select(FromCalculation)
                .Where(x => validCalculationIds.ContainsKey(x.Id))
                .ToList();
            calculations.ForEach(x => x.ParseFormula());
            CalculationIdsByRef = calculations.SelectMany(x => x.ReferencedItems)
                .ToLookup(x => $"{x.Category}*{x.Name}", x => x.Calculation.Id);
            CalculationsById = calculations.ToDictionary(x => x.Id);
            var calculationDatas = calculations.SelectMany(FromCalculation)
                .ToList();
            CalculationDatasById = calculationDatas.ToLookup(x => x.CalculationId);
            var modes = GetDocument("Items_VReport.xml")
                .Elements("Application")
                .Elements("MeasureMode")
                .Where(x => x.GetAttributeValue("Id") != "Undefined")
                .ToList();
            SimpleDatas = new List<MeasureXmlData>();
            SimpleDatas = modes.SelectMany(x => FromMode(x, false))
                .ToList();
            XmlDatas = modes
                .SelectMany(x => FromMode(x, true))
                .Concat(SimpleDatas)
                .Concat(calculationDatas)
                .ToArray();

            #region DebugMissingTag

            //var serializer = new DataContractSerializer(typeof(MeasureTag[]), "MeasureTags", "");
            //MeasureTag[] missingTags;
            //using (var reader = XmlReader.Create("AllMeasureTag-Old.xml"))
            //{
            //    missingTags = (MeasureTag[]) serializer.ReadObject(reader);
            //}

            //missingTags = missingTags.Where(x => !x.Match(XmlDatas).Any()).ToArray();
            //var settings = new XmlWriterSettings
            //{
            //    Indent = true,
            //    OmitXmlDeclaration = true
            //};
            //using (var writer = XmlWriter.Create("MissingMeasureTags.xml", settings))
            //{
            //    serializer.WriteStartObject(writer, missingTags);
            //    writer.WriteAttributeString("xmlns", "arrays", null,
            //        "http://schemas.microsoft.com/2003/10/Serialization/Arrays");
            //    serializer.WriteObjectContent(writer, missingTags);
            //    serializer.WriteEndObject(writer);
            //}

            #endregion

            #region DebugTagDiff

            //var tags = XmlDatas.Select(x => new MeasureTag
            //    {
            //        BaseType = x.TypeId,
            //        CalculationId = x.CalculationId,
            //        Method = x.MethodId,
            //        MatchId = x.MatchId,
            //        Mode = x.ModeId,
            //        OutputV2 = x.OutputId,
            //        Unit = x.UnitId,
            //        UserId = x.CategoryDescription == TryTranslate("Common")
            //            ? "ReportTemplate." + x.TypeId
            //            : x.ItemId
            //    }).Distinct()
            //    .OrderBy(x => x.Mode)
            //    .ThenBy(x => x.UserId)
            //    .ThenBy(x => x.Method)
            //    .ThenBy(x => x.OutputV2)
            //    .ThenBy(x => x.CalculationId)
            //    .ThenBy(x => x.Unit)
            //    .ToArray();
            //var serializer = new DataContractSerializer(typeof(MeasureTag[]), "MeasureTags", "");
            //var settings = new XmlWriterSettings
            //{
            //    Indent = true,
            //    OmitXmlDeclaration = true
            //};
            //using (var writer = XmlWriter.Create("AllMeasureTag.xml", settings))
            //{
            //    serializer.WriteStartObject(writer, tags);
            //    writer.WriteAttributeString("xmlns", "arrays", null,
            //        "http://schemas.microsoft.com/2003/10/Serialization/Arrays");
            //    serializer.WriteObjectContent(writer, tags);
            //    serializer.WriteEndObject(writer);
            //}

            #endregion
        }

        public MeasureXmlData()
        {
        }

        public MeasureXmlData(MeasureXmlData source)
        {
            CalculationDescription = source.CalculationDescription;
            CalculationId = source.CalculationId;
            CategoryDescription = source.CategoryDescription;
            CategoryId = source.CategoryId;
            DefaultMethodDescription = source.DefaultMethodDescription;
            DefaultMethodId = source.DefaultMethodId;
            DefaultOutputDescriptionValue = source.DefaultOutputDescriptionValue;
            DefaultUnitDescription = source.DefaultUnitDescription;
            DefaultUnitId = source.DefaultUnitId;
            ItemDescription = source.ItemDescription;
            ItemId = source.ItemId;
            MatchId = source.MatchId;
            MethodDescription = source.MethodDescription;
            MethodId = source.MethodId;
            ModeDescription = source.ModeDescription;
            ModeId = source.ModeId;
            OutputDescriptionKey = source.OutputDescriptionKey;
            OutputDescriptionValue = source.OutputDescriptionValue;
            OutputId = source.OutputId;
            OutputIdForCc = source.OutputIdForCc;
            TypeDescription = source.TypeDescription;
            TypeId = source.TypeId;
            UnitDescription = source.UnitDescription;
            UnitId = source.UnitId;
            Obsolete = source.Obsolete;
            VisibleDefaultOutputDescriptionValue = source.VisibleDefaultOutputDescriptionValue;
        }

        public static ILookup<string, MeasureXmlData> CalculationDatasById { get; }

        [DataMember] public string CalculationDescription { get; set; }

        [DataMember] public string CalculationId { get; set; }

        [DataMember] public string CategoryDescription { get; set; }

        [DataMember] public string CategoryId { get; set; }

        [DataMember] public string DefaultMethodDescription { get; set; }

        [DataMember] public string DefaultMethodId { get; set; }

        [DataMember] public string DefaultOutputDescriptionValue { get; set; }

        [DataMember] public string DefaultUnitDescription { get; set; }

        [DataMember] public string DefaultUnitId { get; set; }

        [DataMember] public string ItemDescription { get; set; }

        [DataMember] public string ItemId { get; set; }

        [DataMember] public string MatchId { get; set; }

        [DataMember] public string MethodDescription { get; set; }

        [DataMember] public string MethodId { get; set; }

        [DataMember] public string ModeDescription { get; set; }

        [DataMember] public string ModeId { get; set; }

        [DataMember] public bool Obsolete { get; set; }

        /// <summary>
        ///     For compatibility
        /// </summary>
        [DataMember]
        public string OutputDescriptionKey { get; set; }

        [DataMember] public string OutputDescriptionValue { get; set; }

        [DataMember] public string OutputId { get; set; }

        [DataMember] public string OutputIdForCc { get; set; }

        [DataMember] public string TypeDescription { get; set; }

        [DataMember] public string TypeId { get; set; }

        [DataMember] public string UnitDescription { get; set; }

        public static ILookup<string, XElement> UnitExGroups { get; }

        public static Dictionary<string, XElement> UnitExs { get; }

        [DataMember] public string UnitId { get; set; }

        [DataMember] public string VisibleDefaultOutputDescriptionValue { get; set; }

        public static MeasureXmlData[] XmlDatas { get; }

        private static void AddOutput(XElement item, string id, string unit)
        {
            var outputId = new XAttribute("Id", id);
            var outputUnit = new XAttribute("Unit", unit);
            var outputElement = new XElement("OutPut", outputId, outputUnit);
            item.Add(outputElement);
        }

        private static CustomizeCalculation FromCalculation(XElement calculation)
        {
            return new CustomizeCalculation
            {
                Description = TryTranslateDescription(calculation),
                Formula = calculation.GetAttributeValue("Formula"),
                Id = calculation.GetAttributeValue("Id"),
                Unit = calculation.GetAttributeValue("Unit") ?? "None"
            };
        }

        private static IEnumerable<MeasureXmlData> FromCalculation(CustomizeCalculation calculation)
        {
            var id = calculation.Id;
            var template = new MeasureXmlData
            {
                DefaultUnitId = calculation.Unit,
                ItemDescription = calculation.Description
            };
            template.ModeId =
                template.CategoryId = "Calculation";
            template.ModeDescription =
                template.CategoryDescription = TranslateHelper.Translate("Measure.Calculation");
            template.MatchId =
                template.MethodId =
                    template.MethodDescription =
                        template.DefaultMethodId =
                            template.DefaultMethodDescription = "";
            template.TypeId =
                template.TypeDescription =
                    template.ItemId =
                        template.CalculationId =
                            template.CalculationDescription =
                                template.OutputId =
                                    template.OutputDescriptionValue =
                                        template.VisibleDefaultOutputDescriptionValue = id;
            return FromDefaultUnit(template);
        }

        private static List<MeasureXmlData> FromCalculation(string calId, MeasureXmlData template)
        {
            template = new MeasureXmlData(template)
            {
                CalculationId = calId,
                CalculationDescription = ""
            };
            CalculationsById.TryGetValue(calId, out var calculation);
            if (calculation != null)
            {
                template.MatchId = "";
                template.CalculationDescription = calculation.Description;
                template.DefaultUnitId = calculation.Unit;
            }

            return FromDefaultUnit(template);
        }

        public static List<MeasureXmlData> FromDefaultUnit(MeasureXmlData template)
        {
            var defaultUnit = UnitExs[template.DefaultUnitId];
            template.DefaultUnitDescription = defaultUnit.GetElementValue("Description");
            var baseUnitId = defaultUnit.GetElementValue("BaseUnit");
            return UnitExGroups[baseUnitId]
                .Select(x => new MeasureXmlData(template)
                {
                    UnitId = x.GetElementValue("Unit"),
                    UnitDescription = x.GetElementValue("Description")
                }).OrderByDescending(x => x.UnitId == x.DefaultUnitId)
                .ToList();
        }

        private static List<MeasureXmlData> FromItem(XElement item, MeasureXmlData template)
        {
            template = new MeasureXmlData(template)
            {
                CategoryId = item.GetAttributeValue("Categories"),
                ItemId = item.GetAttributeValue("Id"),
                TypeId = item.GetAttributeValue("BaseType")
            };
            var categoryIds = template.CategoryId.SplitSemicolon().Select(TryTranslate);
            template.CategoryDescription = string.Join("/", categoryIds);
            template.ItemDescription = TryTranslateDescription(item);
            var type = Types[template.TypeId];
            var typeDescription = TryTranslateDescription(type);
            template.TypeDescription = TryTranslate(typeDescription);
            if (HasGaEdd(item) || HasGaEdd(type))
            {
                AddOutput(item, "GA", "week_day");
                AddOutput(item, "EDD", "Tick");
            }

            var methodIds = item.GetAttributeValue("AvailableMethods")
                                ?.SplitSemicolon()
                                .ToDictionary(x => x) ??
                            new Dictionary<string, string> {{"", ""}};
            template.DefaultMethodId = item.GetAttributeValue("WorkingMethod");
            if (template.DefaultMethodId == null || !methodIds.ContainsKey(template.DefaultMethodId))
                template.DefaultMethodId = methodIds.Keys.First();

            template.DefaultMethodDescription = TryTranslate(template.DefaultMethodId);
            var methods = item.Elements("Item")
                .Where(x => methodIds.ContainsKey(x.GetAttributeValue("Id")))
                .ToList();
            var itemBaseOutputs = item.Descendants("OutPut").Except(methods.Descendants("OutPut")).ToList();
            return methods.Count == 0
                ? FromMethod(item, itemBaseOutputs, template)
                : methods.SelectMany(x => FromMethod(x, itemBaseOutputs, template))
                    .OrderByDescending(x => x.MethodId == x.DefaultMethodId)
                    .ToList();
        }

        private static List<MeasureXmlData> FromMethod(XElement method, List<XElement> itemBaseOutputs,
            MeasureXmlData template)
        {
            var methodId = method.GetAttributeValue("Id");
            if (methodId == template.ItemId)
                methodId = "";

            template = new MeasureXmlData(template)
            {
                MethodId = methodId,
                MethodDescription = TryTranslate(methodId)
            };
            var methodOutputs = method.Descendants("OutPut").ToArray();
            var blackList = methodOutputs
                .GroupBy(output => GetOutputId(output, "_"))
                .Where(x => !x.Any(OutputIsValid))
                .Select(x => x.Key)
                .ToDictionary(x => x);
            if (template.CategoryId == "Common")
            {
                var uncommonOutputs = method.Descendants("Calculator")
                    .Where(x => x.GetAttributeValue("Id") != "Calculator")
                    .Descendants("OutPut");
                methodOutputs = methodOutputs.Except(uncommonOutputs).ToArray();
            }

            var outputs = itemBaseOutputs.Concat(methodOutputs)
                .Where(OutputIsValid)
                .GroupBy(output => GetOutputId(output, "_"))
                .Where(x => !blackList.ContainsKey(x.Key))
                .Select(x => x.Last())
                .ToArray();
            var defaultOutputId = method.Element("Calculator")?.GetAttributeValue("PrimaryOutput") ??
                                  method.Element("Calculator")?.GetAttributeValue("Outputs")?.SplitSemicolon()[0];
            var defaultOutput = outputs.FirstOrDefault(x => x.GetAttributeValue("Id") == defaultOutputId) ??
                                outputs.FirstOrDefault();
            template.DefaultOutputDescriptionValue = defaultOutput == null
                ? null
                : GetOutputDescription(defaultOutput);
            var visibleOutputs = outputs.Where(x => x.GetAttributeValue("CalculationIDs") == null)
                .ToList();
            defaultOutput = visibleOutputs.FirstOrDefault(x => x.GetAttributeValue("Id") == defaultOutputId) ??
                            visibleOutputs.FirstOrDefault();
            template.VisibleDefaultOutputDescriptionValue = defaultOutput == null
                ? null
                : GetOutputDescription(defaultOutput);
            return outputs.SelectMany(x => FromOutPut(x, template))
                .OrderByDescending(x => x.OutputDescriptionValue == x.DefaultOutputDescriptionValue)
                .ToList();
        }

        private static List<MeasureXmlData> FromMode(XElement mode, bool complex)
        {
            var modeId = mode.GetAttributeValue("Id");
            var template = new MeasureXmlData
            {
                ModeId = modeId,
                ModeDescription = TryTranslate(modeId)
            };
            return mode.Elements("Folder")
                .Elements("Item")
                .Where(x => IsValidItem(x, complex))
                .SelectMany(x => FromItem(x, template))
                .ToList();
        }

        private static List<MeasureXmlData> FromOutPut(XElement output, MeasureXmlData template)
        {
            template = new MeasureXmlData(template)
            {
                MatchId = "",
                OutputId = GetOutputId(output, "_"),
                OutputIdForCc = GetOutputId(output, "|"),
                OutputDescriptionValue = GetOutputDescription(output),
                DefaultUnitId = output.GetAttributeValue("Unit") ?? "None"
            };
            var (targetItem, targetItemId) = output.AncestorsAndSelf()
                .Select(x => (x, x.GetAttributeValue("ReferenceMeasureTerm")))
                .FirstOrDefault(x => x.Item2 != null);
            if (targetItem != null)
            {
                var modeDescription = targetItem.GetAttributeValue("ReferenceMeasureTermMode") ??
                                      template.ModeDescription;
                var targetOutputId = template.OutputIdForCc.Split('|').Last();
                template.MatchId = $"{modeDescription}*{targetItemId}|{targetOutputId}";
            }

            var element = output;
            if (output.Parent is XElement parent &&
                parent.Name == "Calculator" &&
                parent.Elements("OutPut").First() == output &&
                parent.PreviousNode == null &&
                parent.Parent is XElement parent2 &&
                parent2.Name == "Item" &&
                parent2.Parent is XElement parent3 &&
                parent3.Name != "Folder")
            {
                var methods = parent3.GetAttributeValue("AvailableMethods")
                    ?.SplitSemicolon();

                var id = parent2.GetAttributeValue("Id");
                if (methods == null || !methods.Contains(id))
                    element = parent2;
            }

            template.OutputDescriptionKey = element.GetAttributeValue("Id");
            if (template.OutputDescriptionValue != template.DefaultOutputDescriptionValue ||
                output.PreviousNode != null)
            {
                var value = TryTranslateDescription(element);
                if (value != null)
                    template.OutputDescriptionKey = value;
            }

            var calIds = output.GetAttributeValue("CalculationIDs")
                ?.SplitSemicolon()
                .ToList();
            var obsolete = calIds != null;
            template.Obsolete = obsolete;
            if (!obsolete)
            {
                var key = template.MatchId == ""
                    ? $"{template.ModeDescription}*{template.ItemId}|{template.OutputIdForCc}"
                    : template.MatchId;
                key = key.Replace("Flow*", "2D*").Replace("EI*", "2D*");
                calIds = CalculationIdsByRef[key].ToList();
                calIds.Insert(0, "");
            }

            return calIds.SelectMany(x => FromCalculation(x, template))
                .ToList();
        }

        private static XDocument GetDocument(string path)
        {
            var stream = Assembly.GetEntryAssembly()
                ?.GetManifestResourceStream($"ReportTemplateDesigner.Resources.{path}");
            return XDocument.Load(stream);
        }

        private static string GetOutputDescription(XElement output)
        {
            var ancestorsAndSelf = GetOutputElements(output);
            // ReSharper disable once PossibleNullReferenceException
            if (ancestorsAndSelf.Count > 1 && output.Parent.Elements("OutPut").Count() == 1)
                ancestorsAndSelf.RemoveAt(ancestorsAndSelf.Count - 1);

            var values = ancestorsAndSelf.Select(TryTranslateDescription);
            return string.Join("|", values);
        }

        private static List<XElement> GetOutputElements(XElement output)
        {
            var ancestorsAndSelf = output.AncestorsAndSelf()
                .Where(x => x == output || x.Name == "Item")
                .Reverse()
                .ToList();
            for (var i = 0; i < ancestorsAndSelf.Count - 1; i++)
            {
                var methods = ancestorsAndSelf[i].GetAttributeValue("AvailableMethods")
                    ?.SplitSemicolon();
                var id = ancestorsAndSelf[i + 1].GetAttributeValue("Id");
                if (methods?.Contains(id) == true)
                    ancestorsAndSelf.RemoveAt(i + 1);
            }

            ancestorsAndSelf.RemoveAt(0);
            return ancestorsAndSelf;
        }

        private static string GetOutputId(XElement output, string separator)
        {
            var values = GetOutputElements(output).Select(x => x.GetAttributeValue("Id"));
            return string.Join(separator, values);
        }

        private static bool HasGaEdd(XElement element)
        {
            return element.Elements("Calculator")
                .Elements("Processor")
                .Select(x => x.GetAttributeValue("Id"))
                .Any(x => x == "GAEDD");
        }

        private static bool IsValidItem(XElement item, bool complex)
        {
            return complex ==
                   item.DescendantsAndSelf()
                       .Any(x => x.GetAttributeValue("ContainSimpleMeasureItem") == bool.TrueString);
        }

        private static bool OutputIsValid(XElement output)
        {
            if (output.GetAttributeValue("Placeholder") == bool.TrueString)
                return false;

            if (output.GetAttributeValue("CalculationIDs") != null)
                return true;

            var isEnabled = output.GetAttributeValue("IsEnabled");
            return !string.Equals(isEnabled, bool.FalseString, StringComparison.CurrentCultureIgnoreCase);
        }

        public static string TryTranslate(string key)
        {
            return Translator.TryGetValue("Measure." + key, out var value) ? value : key;
        }

        public static string TryTranslateDescription(XElement element)
        {
            var value = element?.GetAttributeValue("Description") ?? element?.GetAttributeValue("Id");
            var description = TryTranslate(value);
            if (element?.Parent?.Name == "Folder")
                return description;

            var prefix = element?.GetAttributeValue("Prefix") + " ";
            return prefix.TrimStart() + description;
        }
    }

    public static class MeasureXmlDataHelper
    {
        public static string GetAttributeValue(this XElement element, string name)
        {
            return element.Attribute(name)?.Value;
        }

        public static string GetElementValue(this XElement element, string name)
        {
            return element.Element(name)?.Value;
        }

        public static string[] SplitSemicolon(this string src)
        {
            return src.Split(new[] {";"}, StringSplitOptions.RemoveEmptyEntries);
        }

        public static Dictionary<string, XElement> ToIdDictionary(this IEnumerable<XElement> element)
        {
            return element.ToDictionary(x => GetAttributeValue(x, "Id"));
        }
    }
}