﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common;
using Vinno.IUS.Common.Args;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The paragraph element.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class Paragraph:Element, IParagraph
    {
        private NotificationList<IElement> _elements;

        /// <summary>
        /// Gets or sets the line spacing for the paragraph.
        /// </summary>
        [DataMember]
        public float LineSpace { get; set; }

        [JsonIgnore]
        public bool IsEmptyFirst { get; set; }

        /// <summary>
        /// The event occurs when elements collection changed.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<IElement>> ElementsCollectionChanged;

        /// <summary>
        /// Gets the elements.
        /// </summary>
        public IReadOnlyList<IElement> Elements => _elements.ToList();

        [JsonIgnore]
        [DataMember]
        private IEnumerable<IElement> DataContractElements
        {
            get => _elements;
            set => _elements = new NotificationList<IElement>(value, OnElementsCollectionChanged);
        }

        /// <summary>
        /// Initialize a <see cref="Paragraph"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public Paragraph(IReportElement parent) : base(parent)
        {
            ElementType = ElementType.Paragraph;
            _elements = new NotificationList<IElement>(OnElementsCollectionChanged);
        }

        /// <summary>
        /// Add a element.
        /// </summary>
        /// <param name="element">The added element.</param>
        public void AddElement(IInLineElemnt element)
        {
            var index = _elements.Count;
            element.Index = index;
            _elements.Add(element);
        }

        /// <summary>
        /// Insert element in specific index.
        /// </summary>
        /// <param name="element">The elemenet to be inserted.</param>
        /// <param name="index">The index.</param>
        public void InsertElement(IInLineElemnt element, int index)
        {
            var updatedElements = _elements.Where(b => b.Index >= index);
            foreach (var updatedElement in updatedElements)
            {
                updatedElement.Index = updatedElement.Index + 1;
            }

            element.Index = index;
            _elements.Insert(index, element);
        }

        /// <summary>
        /// Remove element.
        /// </summary>
        /// <param name="element">The removed element.</param>
        public void RemoveElement(IElement element)
        {
            _elements.Remove(element);
            var updatedElements = _elements.Where(e => e.Index >= element.Index);
            foreach (var updatedElement in updatedElements)
            {
                updatedElement.Index = updatedElement.Index - 1;
            }
        }
        /// <summary>
        /// Clear child elements
        /// </summary>
        public void ClearElement()
        {
            _elements.Clear();
        }

        /// <summary>
        /// Remove element by element index.
        /// </summary>
        /// <param name="elementIndex">The element index.</param>
        public void RemoveElement(int elementIndex)
        {
            var element = Elements[elementIndex];
            _elements.Remove(element);
            var updatedElements = _elements.Where(e => e.Index >= elementIndex);
            foreach (var updatedElement in updatedElements)
            {
                updatedElement.Index = updatedElement.Index - 1;
            }
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var paragraph = new Paragraph(null);
            InitializeCloneElement(paragraph);
            paragraph.LineSpace = LineSpace;
            foreach (var element in Elements)
            {
                if (element is IInLineElemnt)
                {
                    var cloneElement = element.Clone() as IInLineElemnt;
                    if (cloneElement != null)
                    {
                        cloneElement.Parent = paragraph;
                        paragraph.AddElement(cloneElement);
                    }
                }
            }
            return paragraph;
        }

        private void OnElementsCollectionChanged(object sender, CollectionChangedArgs<IElement> e)
        {
            ElementsCollectionChanged?.Invoke(this, new CollectionChangedArgs<IElement>(e.RemovedItems, e.AddedItems));
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is Paragraph paragraph)
            {
                LineSpace = paragraph.LineSpace;
                base.UpdateElementProperty(paragraph);
            }
        }
    }
}