﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common;
using Vinno.IUS.Common.Args;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The RTCell element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTCell: Element, ICell
    {
        private NotificationList<IBlockElement> _blocks;
        private RTColor _background;
        private Borders _borders;
        private string _groupField;

        /// <summary>
        /// The event occurs when blocks collection changes.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<IBlockElement>> BlocksCollectionChanged;

        /// <summary>
        /// Gets all the report template blocks.
        /// </summary>
        public IReadOnlyList<IBlockElement> Blocks => _blocks.ToList();

        [JsonIgnore]
        [DataMember]
        private IEnumerable<IBlockElement> DataContractBlocks
        {
            get => _blocks;
            set => _blocks = new NotificationList<IBlockElement>(value, OnBlocksCollectionChanged);
        }

        /// <summary>
        /// Gets or sets the background.
        /// </summary>
        [DataMember]
        public RTColor Background
        {
            get => _background;
            set
            {
                if (!_background.Equals(value))
                {
                    _background = value;
                    OnCellBackGroundChanged?.Invoke(this, EventArgs.Empty);
                }
            }
        }

        [DataMember]
        public WidthTypeEnum WidthType { get; set; }
        [DataMember]
        public WidthTypeEnum HeightType { get; set; }

        [DataMember]
        public string GroupField { get; set; }

        /// <summary>
        /// Gets the cell borders.
        /// </summary>
        [DataMember]
        public Borders Borders
        {
            get => _borders;
            set
            {
                if (_borders != value)
                {
                    _borders = value;
                    OnBordersChanged?.Invoke(this, EventArgs.Empty);
                }
            }
        }

        public event EventHandler OnCellBackGroundChanged;
        public event EventHandler OnBordersChanged;

        /// <summary>
        /// Initialize a <see cref="RTCell"/>
        /// </summary>
        /// <param name="parent">The parent of cell.</param>
        public RTCell(IGrid parent) : base(parent)
        {
            WidthType = WidthTypeEnum.FixedValue;
            HeightType = WidthTypeEnum.MinValue;
            ElementType = ElementType.RTCell;
            _blocks = new NotificationList<IBlockElement>(OnBlocksCollectionChanged);
            _background = RTColor.Transparent;
            _borders = new Borders();
        }

        /// <summary>
        /// Add a block.
        /// </summary>
        /// <param name="block">The new added block.</param>
        public void AddBlock(IBlockElement block)
        {
            var index = _blocks.Count;
            block.Index = index;
            _blocks.Add(block);
        }

        /// <summary>
        /// Insert block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        public void InsertBlock(IBlockElement block, int index)
        {
            var updatedBlocks = _blocks.Where(b => b.Index >= index);
            foreach (var updatedBlock in updatedBlocks)
            {
                updatedBlock.Index = updatedBlock.Index + 1;
            }

            block.Index = index;
            _blocks.Insert(index, block);
        }
        
        /// <summary>
        /// Get all child elements
        /// </summary>
        /// <returns></returns>
        public IEnumerable<IElement> GetAllElements()
        {
            var elements = new List<IElement>();
            foreach (var block in Blocks)
            {
                if (block is Paragraph paragraph)
                {
                    elements.AddRange(paragraph.Elements);
                }

                if (block is RTGrid rtGrid)
                {
                    elements.AddRange(rtGrid.Elements);
                }

                if (block is InputImageList || block is AtlasElement)
                {
                    elements.Add(block);
                }
            }

            return elements;
        }

        /// <summary>
        /// Remove a block.
        /// </summary>
        /// <param name="block">The removed block.</param>
        public void RemoveBlock(IBlockElement block)
        {
            if (_blocks.Contains(block))
            {
                _blocks.Remove(block);
            }

            var updatedBlocks = _blocks.Where(b => b.Index >= block.Index);
            foreach (var updatedBlock in updatedBlocks)
            {
                updatedBlock.Index = updatedBlock.Index - 1;
            }
        }

        /// <summary>
        /// Clear all blocks.
        /// </summary>
        public void ClearBlocks()
        {
            _blocks.Clear();
        }

        private void OnBlocksCollectionChanged(object sender, CollectionChangedArgs<IBlockElement> e)
        {
            BlocksCollectionChanged?.Invoke(this, new CollectionChangedArgs<IBlockElement>(e.RemovedItems, e.AddedItems));
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var cell = new RTCell(null);
            InitializeCloneElement(cell);
            cell.Background = Background;
            cell.WidthType = WidthType; 
            cell.GroupField = GroupField;
            cell.HeightType = HeightType;
            cell.Borders = Borders.Clone();
            foreach (var block in Blocks)
            {
                var cloneBlock = block.Clone() as IBlockElement;
                if (cloneBlock != null)
                {
                    cloneBlock.Parent = cell;
                    cell.AddBlock(cloneBlock);
                }
            }
            return cell;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is RTCell cell)
            {
                Background = cell.Background;
                Borders = cell.Borders;
                WidthType = cell.WidthType;
                GroupField = cell.GroupField;
                HeightType = cell.HeightType;
                ClearBlocks();
                foreach (var block in cell.Blocks)
                {
                    block.Parent = this;
                    _blocks.Add(block);
                }
                base.UpdateElementProperty(cell);
            }
        }
    }
}