﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common;
using Vinno.IUS.Common.Args;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTGrid: Element, IGrid
    {
        private const double Tolerance = 0.0001;
        private const double MinHeight = 18;
        private const double MinWidth = 21;
        private Dictionary<CellPostion, ICell> _cells;

        private NotificationList<RTColumnDefinition> _columnDefinitions;

        private NotificationList<RTRowDefinition> _rowDefinitions;

        /// <summary>
        /// The event occurs when column added or removed.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<RTColumnDefinition>> ColumnDefinitionCollectionChanged;
        public event EventHandler<CollectionChangedArgs<RTColumnDefinition>> ColumnWidthChanged;
        
        /// <summary>
        /// The event occurs when row added or removed.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<RTRowDefinition>> RowDefinitionCollectionChanged;
        public event EventHandler<CollectionChangedArgs<RTRowDefinition>> RowHeightChanged;

        /// <summary>
        /// The event occurs when cell added, removed or merged.
        /// </summary>
        public event EventHandler<CellsChangedArgs> CellsChanged;
        public event EventHandler CellsClear;

        [JsonIgnore]
        public bool IsDelete { get; set; }
        [JsonIgnore]
        public bool IsInsertColumn { get; set; }
        [JsonIgnore]
        public int DeleteRowIndex { get; set; }
        [JsonIgnore]
        public int DeleteColumnIndex { get; set; }
        [JsonIgnore]
        public double DivideWidth { get; set; }
        [JsonIgnore]
        public bool? IsMergeCell { get; set; }
        [JsonIgnore]
        public int MergeDeleteStartColumnIndex { get; set; }
        [JsonIgnore]
        public int MergeDeleteEndColumnIndex { get; set; }

        //[JsonIgnore]
        //public bool IsUndoRedo { get; set; }

        /// <summary>
        /// Gets the column definitions.
        /// </summary>
        public IReadOnlyList<RTColumnDefinition> ColumnDefinitions
        {
            get => _columnDefinitions.ToList();
            set
            {
                if (_columnDefinitions != value)
                {
                    _columnDefinitions = new NotificationList<RTColumnDefinition>(value, OnColumnDefinitionsCollectionChangedCallback);
                }
            }
        }

        [JsonIgnore]
        [DataMember]
        private IEnumerable<RTColumnDefinition> DataContractColumnDefinitions
        {
            get => _columnDefinitions;
            set => _columnDefinitions = new NotificationList<RTColumnDefinition>(value, OnColumnDefinitionsCollectionChangedCallback);
        }

        /// <summary>
        /// Gets the row definitions.
        /// </summary>
        public IReadOnlyList<RTRowDefinition> RowDefinitions
        {
            get => _rowDefinitions.ToList();
            set
            {
                if (_rowDefinitions != value)
                {
                    _rowDefinitions = new NotificationList<RTRowDefinition>(value, OnRowDefinitionsCollectionChangedCallback);
                }
            }
        }

        [JsonIgnore]
        [DataMember]
        private IEnumerable<RTRowDefinition> DataContractRowDefinitions
        {
            get => _rowDefinitions;
            set => _rowDefinitions = new NotificationList<RTRowDefinition>(value, OnRowDefinitionsCollectionChangedCallback);
        }

        /// <summary>
        /// Gets all cells.
        /// </summary>
        public IReadOnlyDictionary<CellPostion, ICell> Cells
        {
            get => _cells;
            set
            {
                if (_cells != value)
                {
                    _cells = (Dictionary<CellPostion, ICell>) value;
                }
            }
        }

        [JsonIgnore]
        [DataMember]
        private Dictionary<CellPostion, ICell> DataContractCells
        {
            get => _cells;
            set => _cells = value;
        }

        /// <summary>
        /// Gets all the elements.
        /// </summary>
        [JsonIgnore]
        public IReadOnlyList<IElement> Elements => _cells.Values.SelectMany(v => v.GetAllElements()).ToList();

        /// <summary>
        /// Initialize a <see cref="RTGrid"/>
        /// </summary>
        /// <param name="parent">The parent of <see cref="RTGrid"/>.</param>
        public RTGrid(IReportElement parent) : base(parent)
        {
            HorizontalAlignment = HorizontalLayout.Left;
            ElementType = ElementType.RTGrid;
            _columnDefinitions = new NotificationList<RTColumnDefinition>(OnColumnDefinitionsCollectionChangedCallback);
            _rowDefinitions = new NotificationList<RTRowDefinition>(OnRowDefinitionsCollectionChangedCallback);
            _cells = new Dictionary<CellPostion, ICell>();
        }

        public void AddRow()
        {
            for (var i = 0; i < ColumnDefinitions.Count; i++)
            {
                var cellPostion = new CellPostion(RowDefinitions.Count, i, 1, 1);
                var cell = CreateCell();
                _cells.Add(cellPostion, cell);
            }

            _rowDefinitions.Add(new RTRowDefinition { Height = MinHeight });
        }

        /// <summary>
        /// Merge cell
        /// </summary>
        /// <param name="startRowIndex">The start row index.</param>
        /// <param name="endRowIndex">The end row index.</param>
        /// <param name="startColumnIndex">The start column index.</param>
        /// <param name="endColumnIndex">The end column index.</param>
        public void MergeCell(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex)
        {
            var rowCount = RowDefinitions.Count;
            var columnCount = ColumnDefinitions.Count;
            if (startRowIndex < 0 || startRowIndex > rowCount - 1 || endRowIndex < 0 || endRowIndex > rowCount - 1 ||
                startColumnIndex < 0 || startColumnIndex > columnCount - 1 || endColumnIndex < 0 ||
                endColumnIndex > columnCount - 1 || startRowIndex>endRowIndex || startColumnIndex>endColumnIndex)
            {
                return;
            }

            if (endRowIndex <= startRowIndex && endColumnIndex <= startColumnIndex)
            {
                return;
            }

            var newCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = new List<KeyValuePair<CellPostion, ICell>>();
            var existCells = _cells.Where(c =>
                c.Key.Row >= startRowIndex && c.Key.Row <= endRowIndex && c.Key.Column >= startColumnIndex &&
                c.Key.Column <= endColumnIndex).ToList();
            foreach (var cell in existCells)
            {
                _cells.Remove(cell.Key);
                removeCells.Add(cell);
            }

            var rowSpan = endRowIndex - startRowIndex + 1;
            var columnSpan = endColumnIndex - startColumnIndex + 1;

            var list = new List<RTColumnDefinition>();
            if (rowCount == 1 && startColumnIndex < endColumnIndex)
            {
                var divideCount = endColumnIndex - startColumnIndex;
                var addCells = _cells.Where(c =>
               c.Key.Row >= startRowIndex && c.Key.Row <= endRowIndex  &&
               c.Key.Column > endColumnIndex).ToList();
                foreach (var cell in addCells)
                {
                    _cells.Remove(cell.Key);
                    removeCells.Add(cell);
                }
                foreach (var cell in addCells)
                {
                    var newCellPosition = new CellPostion(cell.Key.Row, cell.Key.Column- divideCount,
                        cell.Key.RowSpan, cell.Key.ColumnSpan);
                    var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, cell.Value);
                    newCells.Add(newCell);
                    _cells.Add(newCell.Key, newCell.Value);
                }

                IsMergeCell = true;
                MergeDeleteStartColumnIndex = startColumnIndex + 1;
                MergeDeleteEndColumnIndex = endColumnIndex;
                double addWidth = 0;
                for (var i = MergeDeleteStartColumnIndex; i <= MergeDeleteEndColumnIndex; i++)
                {
                    if (_columnDefinitions.Count > MergeDeleteStartColumnIndex)
                    {
                        var col = _columnDefinitions[MergeDeleteStartColumnIndex];
                        addWidth += col.Width;
                        list.Add(col);
                        _columnDefinitions.RemoveAt(MergeDeleteStartColumnIndex);
                    }
                }
                _columnDefinitions[startColumnIndex].Width += addWidth;
                IsMergeCell = false;
                ColumnDefinitionCollectionChanged?.Invoke(this, new CollectionChangedArgs<RTColumnDefinition>(list, new List<RTColumnDefinition>()));

                columnSpan = 1;
            }

            if (rowCount > 1 && startColumnIndex < endColumnIndex)
            {
                var colSpan = endColumnIndex - startColumnIndex + 1;
                var isNeedDeleteColumn = true;
                for(var i = 0; i < rowCount; i++)
                {
                    if (i >= startRowIndex && i <= endRowIndex)
                        continue;
                    var existSameCell = _cells.Any(a => a.Key.Row == i && a.Key.Column == startColumnIndex && a.Key.ColumnSpan == colSpan);
                    if (!existSameCell)                   
                    {
                        isNeedDeleteColumn = false;
                        break;
                    }
                }

                if (isNeedDeleteColumn)
                {
                    var divideCount = endColumnIndex - startColumnIndex;
                    var addCells = _cells.Where(c =>c.Key.Column > endColumnIndex).ToList();
                    foreach (var cell in addCells)
                    {
                        _cells.Remove(cell.Key);
                        removeCells.Add(cell);
                    }
                    foreach (var cell in addCells)
                    {
                        var newCellPosition = new CellPostion(cell.Key.Row, cell.Key.Column - divideCount,
                            cell.Key.RowSpan, cell.Key.ColumnSpan);
                        var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, cell.Value);
                        newCells.Add(newCell);
                        _cells.Add(newCell.Key, newCell.Value);
                    }


                    var currentColumnCells = _cells.Where(c => (c.Key.Row<startRowIndex||c.Key.Row>endRowIndex)&&c.Key.Column == startColumnIndex).ToList();
                    foreach (var cell in currentColumnCells)
                    {
                        _cells.Remove(cell.Key);
                        removeCells.Add(cell);
                    }
                    foreach (var cell in currentColumnCells)
                    {
                        var newCellPosition = new CellPostion(cell.Key.Row, cell.Key.Column,
                            cell.Key.RowSpan, cell.Key.ColumnSpan - divideCount);
                        var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, cell.Value);
                        newCells.Add(newCell);
                        _cells.Add(newCell.Key, newCell.Value);
                    }

                    IsMergeCell = true;
                    MergeDeleteStartColumnIndex = startColumnIndex + 1;
                    MergeDeleteEndColumnIndex = endColumnIndex;
                    double addWidth = 0;
                    for (var i = MergeDeleteStartColumnIndex; i <= MergeDeleteEndColumnIndex; i++)
                    {
                        if (_columnDefinitions.Count > MergeDeleteStartColumnIndex)
                        {
                            var col = _columnDefinitions[MergeDeleteStartColumnIndex];
                            addWidth += col.Width;
                            list.Add(col);
                            _columnDefinitions.RemoveAt(MergeDeleteStartColumnIndex);
                        }
                    }
                    _columnDefinitions[startColumnIndex].Width += addWidth;
                    IsMergeCell = false;
                    ColumnDefinitionCollectionChanged?.Invoke(this, new CollectionChangedArgs<RTColumnDefinition>(list, new List<RTColumnDefinition>()));

                    columnSpan = 1;
                }
            }

            var newCellPostion = new CellPostion(startRowIndex, startColumnIndex, rowSpan, columnSpan);
            var newcell = CreateCell();
            _cells.Add(newCellPostion, newcell);
            newCells.Add(new KeyValuePair<CellPostion, ICell>(newCellPostion, newcell));
            SetCellBordersAfterCellMerged(new KeyValuePair<CellPostion, ICell>(newCellPostion, newcell), existCells);
            RaiseCellsChanged(newCells, removeCells);
            IsMergeCell = null;
        }

        public void SetCells(int row, int column)
        {
            var newCellPostions = new List<KeyValuePair<CellPostion, ICell>>();
            for (var i = 0; i < row; i++)
            {
                for (var j = 0; j < column; j++)
                {
                    if (i < RowDefinitions.Count && j < ColumnDefinitions.Count)
                        continue;

                    var cell = CreateCell();
                    var cellPosition = new CellPostion(i, j, 1, 1);
                    newCellPostions.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    _cells.Add(cellPosition, cell);
                }
            }

            for (var i = RowDefinitions.Count; i < row; i++)
            {
                _rowDefinitions.Add(new RTRowDefinition { Height = MinHeight});
            }
            
            for (var i = ColumnDefinitions.Count; i < column; i++)
            {
                _columnDefinitions.Add(new RTColumnDefinition());
            }

            SetCellBordersAfterSplitCell(newCellPostions);
            RaiseCellsChanged(newCellPostions, new List<KeyValuePair<CellPostion, ICell>>());
        }

        public void InsertRow(CellPostion postion, bool isRowAbove)
        {
            var rowIndex = postion.Row;
            var addRowCount = (isRowAbove ? 0 : 1 + postion.RowSpan - 1);
            var newRowIndex = rowIndex + addRowCount;
            _rowDefinitions.Insert(newRowIndex, new RTRowDefinition() { Height = MinHeight});

            var newCells = new List<KeyValuePair<CellPostion, ICell>>();

            var copyCells = new List<KeyValuePair<CellPostion, ICell>>();
            var mutliRowCells = new List<KeyValuePair<CellPostion, ICell>>();
            var newAddedCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = _cells.Where(c => c.Key.Row + c.Key.RowSpan - 1 >= newRowIndex).ToList();

            var cancelCells = new List<KeyValuePair<CellPostion, ICell>>();
            if (isRowAbove)
            {
                //copyCells = _cells.Where(a => (a.Key.Row == rowIndex - 1 && a.Key.RowSpan == 1) || (a.Key.Row + a.Key.RowSpan == rowIndex)).ToList();
                copyCells = _cells.Where(a => a.Key.Row == rowIndex).ToList();
                mutliRowCells = _cells.Where(c => (newRowIndex <= (c.Key.Row + c.Key.RowSpan - 1)) && newRowIndex >= c.Key.Row && c.Key.RowSpan > 1).ToList();
            }
            else
            {
                copyCells = _cells.Where(a => (a.Key.Row == newRowIndex - 1 || (a.Key.Row + a.Key.RowSpan - 1) == newRowIndex - 1) && (a.Key.Row + a.Key.RowSpan - 1) < newRowIndex).ToList();

                //copyCells = _cells.Where(a => (a.Key.Row == rowIndex || (a.Key.Row + a.Key.RowSpan - 1) == rowIndex) && (a.Key.Row + a.Key.RowSpan - 1) < newRowIndex).ToList();
                foreach (var cell in copyCells)
                {
                    if (_cells.Any(a => a.Key.Row < newRowIndex && (a.Key.Column <= cell.Key.Column && (a.Key.Column + a.Key.ColumnSpan - 1) >= cell.Key.Column) && (a.Key.RowSpan - 1 + a.Key.Row) >= newRowIndex))
                    {
                        cancelCells.Add(cell);
                    }
                }
                mutliRowCells = _cells.Where(c => (newRowIndex <= (c.Key.Row + c.Key.RowSpan - 1)) && newRowIndex >= c.Key.Row && c.Key.RowSpan > 1).ToList();
            }

            ////往下移
            foreach (var removeCell in removeCells)
            {
                _cells.Remove(removeCell.Key);
            }
            foreach (var removeCell in mutliRowCells)
            {
                _cells.Remove(removeCell.Key);
            }
            foreach (var removeCell in removeCells)
            {
                if (mutliRowCells.Any(a => a.Key == removeCell.Key))
                    continue;
                var newCellPosition = new CellPostion(removeCell.Key.Row + 1, removeCell.Key.Column,
                    removeCell.Key.RowSpan, removeCell.Key.ColumnSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }
            foreach (var removeCell in mutliRowCells)
            {
                var addRows = 0;
                var addRowSpan = 1;
                if (!isRowAbove)
                {
                    addRows = removeCell.Key.Row == newRowIndex ? 1 : 0;
                    if (rowIndex + postion.RowSpan > removeCell.Key.Row && rowIndex + postion.RowSpan <= removeCell.Key.Row + removeCell.Key.RowSpan - 1 && removeCell.Key.Column != postion.Column)
                    {
                        addRowSpan = 1;
                    }
                    else
                    {
                        addRowSpan = 0;
                    }
                }
                else
                {
                    addRows = removeCell.Key.Row == newRowIndex ? 1 : 0;
                    if (rowIndex > removeCell.Key.Row && rowIndex <= removeCell.Key.Row + removeCell.Key.RowSpan && removeCell.Key.Column != postion.Column)
                    {
                        addRowSpan = 1;
                    }
                    else
                    {
                        addRowSpan = 0;
                    }
                }
                var newCellPosition = new CellPostion(removeCell.Key.Row + addRows, removeCell.Key.Column,
                removeCell.Key.RowSpan + addRowSpan, removeCell.Key.ColumnSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }

            if (mutliRowCells.Count > 0)
            {
                removeCells.AddRange(mutliRowCells);
            }
            foreach (var copyCell in copyCells)
            {
                if (cancelCells.Any(a => a.Key == copyCell.Key))
                    continue;
                var cellPostion = new CellPostion(newRowIndex, copyCell.Key.Column, 1, copyCell.Key.ColumnSpan);
                var cell = CreateCell();
                newCells.Add(new KeyValuePair<CellPostion, ICell>(cellPostion, cell));
                _cells.Add(cellPostion, cell);
                newAddedCells.Add(new KeyValuePair<CellPostion, ICell>(cellPostion, cell));
            }

            SetCellBordersAfterInsertRow(newAddedCells, isRowAbove);
            RaiseCellsChanged(newCells, removeCells);

            var activieRowIndex = isRowAbove ? postion.Row : postion.Row + postion.RowSpan;
            var newActiveCell = Cells.FirstOrDefault(a => a.Key.Row == activieRowIndex && a.Key.Column == postion.Column).Value;
            if (newActiveCell != null)
            {
                var document = GetReportTemplateDocument();
                document?.RaiseCurrentElementChanged(newActiveCell);
            }
        }

        public double GetMaxWidth()
        {
            var report = GetReportTemplateDocument();
            var maxWidth = 0.0;
            if (report != null)
            {
                maxWidth = report.PageSize.Width - report.PagePadding.Left - report.PagePadding.Right - 4.5;
            }

            return maxWidth;
        }

        public double GetTotalWidth()
        {
            return ColumnDefinitions.Sum(a => a.Width);
        }

        public void SetWidth(CellPostion currentPosition, double actualWidth)
        {
            var list = new List<RTColumnDefinition>();
            for (var i = currentPosition.Column; i <= currentPosition.Column + currentPosition.ColumnSpan - 1; i++)
            {
                _columnDefinitions[i].Width = actualWidth;
                list.Add(_columnDefinitions[i]);
            }
            ColumnWidthChanged?.Invoke(this, new CollectionChangedArgs<RTColumnDefinition>(new List<RTColumnDefinition>(), list));
        }

        public void SetHeight(CellPostion currentPosition, double height, WidthTypeEnum heightType)
        {
            var list = new List<RTRowDefinition>();
            for (var i = currentPosition.Row; i <= currentPosition.Row + currentPosition.RowSpan - 1; i++)
            {
                if (heightType == WidthTypeEnum.FixedValue)
                {
                    _rowDefinitions[i].Height = height;
                    _rowDefinitions[i].MinHeight = MinHeight;
                }
                else
                {
                    if (_rowDefinitions[i].Height < height)
                    {
                        _rowDefinitions[i].Height = height;
                    }

                    _rowDefinitions[i].MinHeight = height;
                }
                _rowDefinitions[i].Type = heightType;
                list.Add(_rowDefinitions[i]);
            }
            RowHeightChanged?.Invoke(this, new CollectionChangedArgs<RTRowDefinition>(new List<RTRowDefinition>(), list));
        }

        private void DivideColWidth(List<RTColumnDefinition> existCols)
        {
            var count = existCols.Count;
            DivideWidth = MinWidth / count;
            if (existCols.Any(a => a.Width < DivideWidth + MinWidth))
            {
                var newExistCols = existCols.Where(a => a.Width > DivideWidth + MinWidth).ToList();
                DivideColWidth(newExistCols);
            }
            else
            {
                foreach (var col in existCols)
                {
                    col.Width -= DivideWidth;
                }
            }
        }


        public void InsertColumn(CellPostion postion, bool isLeft)
        {
            var maxWidth = GetMaxWidth();
            var tableWidth = GetTotalWidth();
            IsInsertColumn = true;

            if (tableWidth + MinWidth > maxWidth)
            {
                var existCols = _columnDefinitions.Where(a => a.Width > MinWidth + Tolerance).ToList();
                DivideColWidth(existCols);
            }

            var colIndex = postion.Column;
            var addCol = (isLeft ? 0 : 1 + postion.ColumnSpan - 1);
            var newColIndex = colIndex + addCol;
            _columnDefinitions.Insert(newColIndex, new RTColumnDefinition() { Width = MinWidth });

            var newCells = new List<KeyValuePair<CellPostion, ICell>>();
            var mutliColCells = new List<KeyValuePair<CellPostion, ICell>>();
            var newAddedCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = _cells.Where(c => c.Key.Column >= newColIndex).ToList();

            var copyCells = new List<KeyValuePair<CellPostion, ICell>>();
            var cancelCells = new List<KeyValuePair<CellPostion, ICell>>();
            if (isLeft)
            {
                copyCells = _cells.Where(a => a.Key.Column == colIndex).ToList();
                //copyCells = _cells.Where(a => (a.Key.Column == colIndex - 1 && a.Key.ColumnSpan == 1) || (a.Key.Column + a.Key.ColumnSpan == colIndex)).ToList();
                mutliColCells = _cells.Where(c => (newColIndex <= (c.Key.Column + c.Key.ColumnSpan - 1)) && newColIndex >= c.Key.Column && c.Key.ColumnSpan > 1).ToList();
            }
            else
            {
                copyCells = _cells.Where(a => (a.Key.Column == newColIndex - 1 || (a.Key.Column + a.Key.ColumnSpan - 1) == newColIndex - 1) && (a.Key.Column + a.Key.ColumnSpan - 1) < newColIndex).ToList();

                //copyCells = _cells.Where(a => (a.Key.Column == colIndex || (a.Key.Column + a.Key.ColumnSpan - 1) == colIndex) && (a.Key.Column + a.Key.ColumnSpan - 1) < newColIndex).ToList();
                foreach (var cell in copyCells)
                {
                    if (_cells.Any(a => a.Key.Column < newColIndex && (a.Key.Row <= cell.Key.Row && (a.Key.Row + a.Key.RowSpan - 1) >= cell.Key.Row) && (a.Key.ColumnSpan - 1 + a.Key.Column) >= newColIndex))
                    {
                        cancelCells.Add(cell);
                    }
                }
                mutliColCells = _cells.Where(c => (newColIndex <= (c.Key.Column + c.Key.ColumnSpan - 1)) && newColIndex >= c.Key.Column && c.Key.ColumnSpan > 1).ToList();
            }

            ////往下移
            foreach (var removeCell in removeCells)
            {
                _cells.Remove(removeCell.Key);
            }
            foreach (var removeCell in mutliColCells)
            {
                _cells.Remove(removeCell.Key);
            }
            foreach (var removeCell in removeCells)
            {
                if (mutliColCells.Any(a => a.Key == removeCell.Key))
                    continue;
                var newCellPosition = new CellPostion(removeCell.Key.Row, removeCell.Key.Column + 1,
                    removeCell.Key.RowSpan, removeCell.Key.ColumnSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }
            foreach (var removeCell in mutliColCells)
            {
                var addCols = 0;
                var addColSpan = 1;
                if (!isLeft)
                {
                    addCols = removeCell.Key.Column == newColIndex ? 1 : 0;
                    if (colIndex + postion.ColumnSpan > removeCell.Key.Column && colIndex + postion.ColumnSpan <= removeCell.Key.Column + removeCell.Key.ColumnSpan - 1 && removeCell.Key.Row != postion.Row)
                    {
                        addColSpan = 1;
                    }
                    else
                    {
                        addColSpan = 0;
                    }
                }
                else
                {
                    addCols = removeCell.Key.Column == newColIndex ? 1 : 0;
                    if (colIndex > removeCell.Key.Column && colIndex < removeCell.Key.Column + removeCell.Key.ColumnSpan && removeCell.Key.Row != postion.Row)
                    {
                        addColSpan = 1;
                    }
                    else
                    {
                        addColSpan = 0;
                    }
                }
                var newCellPosition = new CellPostion(removeCell.Key.Row, removeCell.Key.Column + addCols,
                    removeCell.Key.RowSpan, removeCell.Key.ColumnSpan + addColSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }
            if (mutliColCells.Count > 0)
            {
                removeCells.AddRange(mutliColCells);
            }

            foreach (var copyCell in copyCells)
            {
                if (cancelCells.Any(a => a.Key == copyCell.Key))
                    continue;
                var cellPostion = new CellPostion(copyCell.Key.Row, newColIndex, copyCell.Key.RowSpan, 1);
                var cell = CreateCell();
                newCells.Add(new KeyValuePair<CellPostion, ICell>(cellPostion, cell));
                _cells.Add(cellPostion, cell);
                newAddedCells.Add(new KeyValuePair<CellPostion, ICell>(cellPostion, cell));
            }

            SetCellBordersAfterInsertColumn(newAddedCells, isLeft);
            RaiseCellsChanged(newCells, removeCells);
            IsInsertColumn = false;

            var activeColIndex = isLeft ? postion.Column : postion.Column + postion.ColumnSpan;
            var newActiveCell = Cells.FirstOrDefault(a => a.Key.Row == postion.Row && a.Key.Column == activeColIndex).Value;
            if (newActiveCell != null)
            {
                var document = GetReportTemplateDocument();
                document?.RaiseCurrentElementChanged(newActiveCell);
            }
        }

        private void SetCellBordersAfterInsertColumn(List<KeyValuePair<CellPostion, ICell>> newAddedCells,bool isLeft)
        {
            foreach (var item in newAddedCells)
            {
                var cellPosition = item.Key;
                var cell = item.Value;
                var leftCell = GetCell(cellPosition.Row, cellPosition.Column - 1);
                var rightCell = GetCell(cellPosition.Row, cellPosition.Column + cellPosition.ColumnSpan);

                if (isLeft)
                {
                    cell.Borders.Left.BorderStyle = leftCell != null ? RTBorderStyle.None : rightCell.Borders.Left.BorderStyle;
                    cell.Borders.Top.BorderStyle = rightCell.Borders.Top.BorderStyle;
                    cell.Borders.Bottom.BorderStyle = rightCell.Borders.Bottom.BorderStyle;

                    cell.Borders.Right.BorderStyle = rightCell.Borders.Right.BorderStyle;
                    if (leftCell == null)
                    {
                        rightCell.Borders.Left.BorderStyle = RTBorderStyle.None;
                    }

                    cell.Borders.Left.Color = rightCell.Borders.Left.Color;
                    cell.Borders.Top.Color = rightCell.Borders.Top.Color;
                    cell.Borders.Bottom.Color = rightCell.Borders.Bottom.Color;
                    cell.Borders.Right.Color = rightCell.Borders.Right.Color;

                    cell.Borders.Left.Thickness = rightCell.Borders.Left.Thickness;
                    cell.Borders.Top.Thickness = rightCell.Borders.Top.Thickness;
                    cell.Borders.Bottom.Thickness = rightCell.Borders.Bottom.Thickness;
                    cell.Borders.Right.Thickness = rightCell.Borders.Right.Thickness;
                }
                else
                {
                    cell.Borders.Left.BorderStyle = RTBorderStyle.None;
                    cell.Borders.Top.BorderStyle = leftCell.Borders.Top.BorderStyle;
                    cell.Borders.Bottom.BorderStyle = leftCell.Borders.Bottom.BorderStyle;
                    cell.Borders.Right.BorderStyle = leftCell.Borders.Right.BorderStyle;

                    cell.Borders.Left.Color = leftCell.Borders.Left.Color;
                    cell.Borders.Top.Color = leftCell.Borders.Top.Color;
                    cell.Borders.Bottom.Color = leftCell.Borders.Bottom.Color;
                    cell.Borders.Right.Color = leftCell.Borders.Right.Color;

                    cell.Borders.Left.Thickness = leftCell.Borders.Left.Thickness;
                    cell.Borders.Top.Thickness = leftCell.Borders.Top.Thickness;
                    cell.Borders.Bottom.Thickness = leftCell.Borders.Bottom.Thickness;
                    cell.Borders.Right.Thickness = leftCell.Borders.Right.Thickness;
                }
            }
        }
        private void SetCellBordersAfterInsertRow(List<KeyValuePair<CellPostion, ICell>> newAddedCells, bool isAbove)
        {
            foreach (var item in newAddedCells)
            {
                var cellPosition = item.Key;
                var cell = item.Value;
                var topCell = GetCell(cellPosition.Row - 1, cellPosition.Column);
                var bottomCell = GetCell(cellPosition.Row + cellPosition.RowSpan, cellPosition.Column);
                var nextBottomCell = GetCell(cellPosition.Row + cellPosition.RowSpan + 1, cellPosition.Column);
                if (isAbove)
                {
                    cell.Borders.Left.BorderStyle = bottomCell.Borders.Left.BorderStyle;
                    cell.Borders.Top.BorderStyle = bottomCell.Borders.Top.BorderStyle;
                    cell.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                    cell.Borders.Right.BorderStyle = bottomCell.Borders.Right.BorderStyle;

                    cell.Borders.Left.Color = bottomCell.Borders.Left.Color;
                    cell.Borders.Top.Color = bottomCell.Borders.Top.Color;
                    cell.Borders.Bottom.Color = bottomCell.Borders.Bottom.Color;
                    cell.Borders.Right.Color = bottomCell.Borders.Right.Color;


                    cell.Borders.Left.Thickness = bottomCell.Borders.Left.Thickness;
                    cell.Borders.Top.Thickness = bottomCell.Borders.Top.Thickness;
                    cell.Borders.Bottom.Thickness = bottomCell.Borders.Bottom.Thickness;
                    cell.Borders.Right.Thickness = bottomCell.Borders.Right.Thickness;
                }
                else
                {
                    cell.Borders.Bottom.BorderStyle = bottomCell != null ? RTBorderStyle.None : topCell.Borders.Bottom.BorderStyle;
                    cell.Borders.Left.BorderStyle = topCell.Borders.Left.BorderStyle;
                    cell.Borders.Top.BorderStyle = topCell.Borders.Top.BorderStyle;
                    cell.Borders.Right.BorderStyle = topCell.Borders.Right.BorderStyle;

                    if (bottomCell == null)
                    {
                        topCell.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                    }

                    cell.Borders.Left.Color = topCell.Borders.Left.Color;
                    cell.Borders.Top.Color = topCell.Borders.Top.Color;
                    cell.Borders.Bottom.Color = topCell.Borders.Bottom.Color;
                    cell.Borders.Right.Color = topCell.Borders.Right.Color;

                    cell.Borders.Left.Thickness = topCell.Borders.Left.Thickness;
                    cell.Borders.Top.Thickness = topCell.Borders.Top.Thickness;
                    cell.Borders.Bottom.Thickness = topCell.Borders.Bottom.Thickness;
                    cell.Borders.Right.Thickness = topCell.Borders.Right.Thickness;
                }
            }
        }

        public void DeleteSelectedRow(int rowIndex)
        {
            IsDelete = true;
            DeleteRowIndex = rowIndex;
            _rowDefinitions.RemoveAt(rowIndex);
            var newCells = new List<KeyValuePair<CellPostion, ICell>>();
            var newAddedCells = new List<KeyValuePair<CellPostion, ICell>>();
            var mutliRowCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = _cells.Where(c => c.Key.Row >= rowIndex).ToList();
            var belowCells = _cells.Where(c => c.Key.Row > rowIndex).ToList();

            var currentRowCells = _cells.Where(c => c.Key.Row == rowIndex || (c.Key.Row <= rowIndex && rowIndex <= c.Key.Row + c.Key.RowSpan - 1)).ToList();

            mutliRowCells = currentRowCells.Where(a => a.Key.RowSpan > 1).ToList();


            foreach (var removeCell in removeCells)
            {
                _cells.Remove(removeCell.Key);
            }
            foreach (var removeCell in mutliRowCells)
            {
                _cells.Remove(removeCell.Key);
                removeCells.Add(removeCell);
            }
            foreach (var removeCell in mutliRowCells)
            {
                var newCellPosition = new CellPostion(removeCell.Key.Row, removeCell.Key.Column,
                      removeCell.Key.RowSpan - 1, removeCell.Key.ColumnSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }

            foreach (var removeCell in belowCells)
            {
                var newCellPosition = new CellPostion(removeCell.Key.Row - 1, removeCell.Key.Column,
                    removeCell.Key.RowSpan, removeCell.Key.ColumnSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }
            SetCellBordersAfterDeleteRow();
            RaiseCellsChanged(newCells, removeCells);
            IsDelete = false;

            var newActiveCell = Cells.FirstOrDefault(a => a.Key.Row == rowIndex && a.Key.Column == 0).Value;
            var document = GetReportTemplateDocument();
            if (newActiveCell != null)
            {
                document?.RaiseCurrentElementChanged(newActiveCell);
            }
            else
            {
                document?.RaiseCurrentElementChanged(this);
            }
        }

        private void SetCellBordersAfterDeleteRow()
        {
            var lastRowCells = _cells.Where(a => a.Key.Row == _rowDefinitions.Count - 1).ToList();
            for (var i = 0; i < ColumnDefinitions.Count; i++)
            {
                var cell = lastRowCells.Where(a => a.Key.Column == i).ToList();
                if (cell.Count == 0)
                {
                    var actualCellList = _cells.Where(a => a.Key.Column == i && a.Key.RowSpan > 1).OrderByDescending(a => a.Key.Row).ToList();
                    if (actualCellList.Count == 0)
                        continue;
                    var actualCell = actualCellList[0].Value;
                    if (actualCell != null && actualCell.Borders.Bottom != null && actualCell.Borders.Bottom.BorderStyle == RTBorderStyle.None)
                    {
                        actualCell.Borders.Bottom.BorderStyle = actualCell.Borders.Top.BorderStyle;
                    }
                }
                else
                {
                    var actualCell = cell[0].Value;
                    if (actualCell.Borders.Bottom != null && actualCell.Borders.Bottom.BorderStyle == RTBorderStyle.None)
                    {
                        actualCell.Borders.Bottom.BorderStyle = actualCell.Borders.Top.BorderStyle;
                    }
                }

            }
        }

        public void DeleteSelectedColumn(int colIndex)
        {
            IsDelete = true;
            DeleteColumnIndex = colIndex;
            _columnDefinitions.RemoveAt(colIndex);

            var newCells = new List<KeyValuePair<CellPostion, ICell>>();
            var mutliColCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = _cells.Where(c => c.Key.Column >= colIndex).ToList();
            var belowCells = _cells.Where(c => c.Key.Column > colIndex).ToList();

            var currentColCells = _cells.Where(c => c.Key.Column == colIndex || (c.Key.Column <= colIndex && colIndex <= c.Key.Column + c.Key.ColumnSpan - 1)).ToList();

            mutliColCells = currentColCells.Where(a => a.Key.ColumnSpan > 1).ToList();


            foreach (var removeCell in removeCells)
            {
                _cells.Remove(removeCell.Key);
            }
            foreach (var removeCell in mutliColCells)
            {
                _cells.Remove(removeCell.Key);
                removeCells.Add(removeCell);
            }

            foreach (var removeCell in belowCells)
            {
                var newCellPosition = new CellPostion(removeCell.Key.Row, removeCell.Key.Column - 1,
                    removeCell.Key.RowSpan, removeCell.Key.ColumnSpan);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }
            foreach (var removeCell in mutliColCells)
            {
                var newCellPosition = new CellPostion(removeCell.Key.Row, removeCell.Key.Column,
                        removeCell.Key.RowSpan, removeCell.Key.ColumnSpan - 1);
                var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                newCells.Add(newCell);
                _cells.Add(newCell.Key, newCell.Value);
            }

            foreach (var item in newCells)
            {
                var cellPosition = item.Key;
                var cell = item.Value;
                var leftCell = GetCell(cellPosition.Row, cellPosition.Column - 1);

                if (leftCell == null && cell.Borders.Left.BorderStyle == RTBorderStyle.None)
                {
                    cell.Borders.Left.BorderStyle = cell.Borders.Top.BorderStyle;
                }
            }

            RaiseCellsChanged(newCells, removeCells);
            IsDelete = false;

            var newActiveCell = Cells.FirstOrDefault(a => a.Key.Row == 0 && a.Key.Column == colIndex).Value;
            var document = GetReportTemplateDocument();
            if (newActiveCell != null)
            {
                document?.RaiseCurrentElementChanged(newActiveCell);
            }
            else
            {
                document?.RaiseCurrentElementChanged(this);
            }
        }

        /// <summary>
        /// Split Cell
        /// </summary>
        /// <param name="position">current cell</param>
        /// <param name="rowCount">split row count</param>
        /// <param name="columnCount">split columnt count</param>
        public void SplitCell(CellPostion position, int rowCount,int columnCount)
        {
            if (rowCount > 0)
            {
                SplitRow(position, rowCount);
            }

            if (columnCount > 0)
            {
                SplitColumn(position, columnCount);
            }
        }

        /// <summary>
        /// Gets cell position, includes row index, cell index, row span and column span.
        /// </summary>
        /// <param name="cell">The searched cell.</param>
        /// <returns>The <see cref="CellPostion"/></returns>
        public CellPostion GetCellPosition(ICell cell)
        {
            var matchedCell = _cells.FirstOrDefault(c => c.Value == cell);
            return matchedCell.Key;
        }

        /// <summary>
        /// Get cell by cell position.
        /// </summary>
        /// <param name="rowIndex">The row index.</param>
        /// <param name="columnIndex">The column index.</param>
        /// <returns>The <see cref="ICell"/></returns>
        public ICell GetCell(int rowIndex, int columnIndex)
        {
            return _cells.FirstOrDefault(c =>
                c.Key.Row <= rowIndex && c.Key.RowSpan + c.Key.Row - 1 >= rowIndex && c.Key.Column <= columnIndex &&
                c.Key.Column + c.Key.ColumnSpan - 1 >= columnIndex).Value;
        }

        /// <summary>
        /// Sets row defnitions.
        /// </summary>
        /// <param name="rowDefinitions">The list of <see cref="RTRowDefinition"/></param>
        public void SetRowDefinitions(IEnumerable<RTRowDefinition> rowDefinitions)
        {
            _rowDefinitions.AddRange(rowDefinitions);
        }

        /// <summary>
        /// Sets column definitions.
        /// </summary>
        /// <param name="columnDefinitions">The list of <see cref="RTColumnDefinition"/></param>
        public void SetColumnDefinitions(IEnumerable<RTColumnDefinition> columnDefinitions)
        {
            _columnDefinitions.AddRange(columnDefinitions);
        }

        private void SplitRow(CellPostion position, int rowCount)
        {
            var rowPosition = position.Row;
            var currentRowSpan = position.RowSpan;
            var newCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = new List<KeyValuePair<CellPostion, ICell>>();
            var newAddedCells = new List<KeyValuePair<CellPostion, ICell>>();
            if (currentRowSpan == 1)
            {
                removeCells = _cells.Where(c => c.Key.Row > position.Row).ToList();
                ////往下移
                foreach (var removeCell in removeCells)
                {
                    _cells.Remove(removeCell.Key);
                }
                foreach (var removeCell in removeCells)
                {
                    var newCellPosition = new CellPostion(removeCell.Key.Row + rowCount, removeCell.Key.Column,
                        removeCell.Key.RowSpan, removeCell.Key.ColumnSpan);
                    var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                    newCells.Add(newCell);
                    _cells.Add(newCell.Key, newCell.Value);
                }

                ////Add row defintion
                var currentRowHeight = _rowDefinitions[position.Row].Height;
                var currentRowMinHeight = _rowDefinitions[position.Row].MinHeight;
                var type = _rowDefinitions[position.Row].Type;
                for (var i = 1; i <= rowCount; i++)
                {
                    _rowDefinitions.Insert(position.Row + i, new RTRowDefinition { Height = currentRowHeight,MinHeight= currentRowMinHeight, Type = type });
                }

                ////Create new cell
                for (var i = 0; i < rowCount; i++)
                {
                    var rowIndex = position.Row + i + 1;
                    var cell = CreateCell();
                    var cellPosition = new CellPostion(rowIndex, position.Column, 1, position.ColumnSpan);
                    newCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    newAddedCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    _cells.Add(cellPosition, cell);
                }

                ////同一行左
                if (position.Column > 0)
                {
                    var oldLeftCells = _cells.Where(c => c.Key.Column < position.Column && c.Key.Row <= position.Row &&
                                       c.Key.Row + c.Key.RowSpan - 1 >= position.Row).ToList();
                    foreach (var oldCell in oldLeftCells)
                    {
                        _cells.Remove(oldCell.Key);
                        removeCells.Add(oldCell);
                        var newCellPosition = new CellPostion(oldCell.Key.Row, oldCell.Key.Column,
                            oldCell.Key.RowSpan + rowCount, oldCell.Key.ColumnSpan);
                        var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, oldCell.Value);
                        newCells.Add(newCell);
                        _cells.Add(newCell.Key, newCell.Value);
                    }
                }
                ////同一行右
                if (position.Column < _columnDefinitions.Count)
                {
                    var oldRightCells = _cells.Where(c => c.Key.Column > position.Column && c.Key.Row <= position.Row &&
                                                          c.Key.Row + c.Key.RowSpan - 1 >= position.Row).ToList();
                    foreach (var oldCell in oldRightCells)
                    {
                        _cells.Remove(oldCell.Key);
                        removeCells.Add(oldCell);
                        var newCellPosition = new CellPostion(oldCell.Key.Row, oldCell.Key.Column,
                            oldCell.Key.RowSpan + rowCount, oldCell.Key.ColumnSpan);
                        var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, oldCell.Value);
                        newCells.Add(newCell);
                        _cells.Add(newCell.Key, newCell.Value);
                    }
                }
            }
            else
            {
                var oldCell = GetCell(position.Row, position.Column);
                removeCells.Add(new KeyValuePair<CellPostion, ICell>(position, oldCell));
                _cells.Remove(position);
                var rowCountForEachCellView = position.RowSpan / (rowCount + 1);
                for (var i = 0; i < rowCount + 1; i++)
                {
                    var rowIndex = rowPosition + i * rowCountForEachCellView;
                    var cell = CreateCell();
                    if (i == 0)
                    {
                        foreach (var block in oldCell.Blocks)
                        {
                            block.Parent = cell;
                            cell.AddBlock(block);
                        }
                    }
                    var cellPosition = new CellPostion(rowIndex, position.Column, rowCountForEachCellView, position.ColumnSpan);
                    newCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    newAddedCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    _cells.Add(cellPosition, cell);
                }
            }

            SetCellBordersAfterSplitCell(newAddedCells);
            RaiseCellsChanged(newCells, removeCells);
            if (newAddedCells.Count > 0)
            {
                var newActiveCell = newAddedCells[0].Value;
                if (newActiveCell != null)
                {
                    var document = GetReportTemplateDocument();
                    document?.RaiseCurrentElementChanged(newActiveCell);
                }
            }
        }

        private void SplitColumn(CellPostion position, int columnCount)
        {
            var colPosition = position.Column;
            var currentColSpan = position.ColumnSpan;
            var newCells = new List<KeyValuePair<CellPostion, ICell>>();
            var removeCells = new List<KeyValuePair<CellPostion, ICell>>();
            var newAddedCells = new List<KeyValuePair<CellPostion, ICell>>();
            if (currentColSpan == 1)
            {
                removeCells = _cells.Where(c => c.Key.Column > colPosition).ToList();
                foreach (var removeCell in removeCells)
                {
                    _cells.Remove(removeCell.Key);
                }

                foreach (var removeCell in removeCells)
                {
                    var newCellPosition = new CellPostion(removeCell.Key.Row, removeCell.Key.Column + columnCount,
                        removeCell.Key.RowSpan, removeCell.Key.ColumnSpan);
                    var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, removeCell.Value);
                    newCells.Add(newCell);
                    _cells.Add(newCell.Key, newCell.Value);
                }

                var currentColumnWidth = _columnDefinitions[position.Column].Width;
                var newColumnWidth = currentColumnWidth / (columnCount + 1);
                _columnDefinitions[position.Column].Width = newColumnWidth;
                for (var i = 1; i <= columnCount; i++)
                {
                    _columnDefinitions.Insert(position.Column + i, new RTColumnDefinition { Width = newColumnWidth});
                }

                for (var i = 0; i < columnCount; i++)
                {
                    var colIndex = position.Column + i + 1;
                    var cell = CreateCell();
                    var cellPosition = new CellPostion(position.Row, colIndex, position.RowSpan, 1);
                    newCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    newAddedCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    _cells.Add(cellPosition, cell);
                }

                if (position.Row > 0)
                {
                    var oldTopCells = _cells.Where(c => c.Key.Row < position.Row && c.Key.Column <= position.Column &&
                                                        c.Key.Column + c.Key.ColumnSpan - 1 >= position.Column).ToList();
                    foreach (var oldCell in oldTopCells)
                    {
                        _cells.Remove(oldCell.Key);
                        removeCells.Add(oldCell);
                        var newCellPosition = new CellPostion(oldCell.Key.Row, oldCell.Key.Column,
                            oldCell.Key.RowSpan, oldCell.Key.ColumnSpan + columnCount);
                        var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, oldCell.Value);
                        newCells.Add(newCell);
                        _cells.Add(newCell.Key, newCell.Value);
                    }
                }

                if (position.Row < _rowDefinitions.Count)
                {
                    var oldBottomCells = _cells.Where(c => c.Key.Row > position.Row && c.Key.Column <= position.Column &&
                                                           c.Key.Column + c.Key.ColumnSpan - 1 >= position.Column).ToList();
                    foreach (var oldCell in oldBottomCells)
                    {
                        _cells.Remove(oldCell.Key);
                        removeCells.Add(oldCell);
                        var newCellPosition = new CellPostion(oldCell.Key.Row, oldCell.Key.Column,
                            oldCell.Key.RowSpan, oldCell.Key.ColumnSpan + columnCount);
                        var newCell = new KeyValuePair<CellPostion, ICell>(newCellPosition, oldCell.Value);
                        newCells.Add(newCell);
                        _cells.Add(newCell.Key, newCell.Value);
                    }
                }
            }
            else
            {
                var oldCell = GetCell(position.Row, position.Column);
                removeCells.Add(new KeyValuePair<CellPostion, ICell>(position, oldCell));
                _cells.Remove(position);
                var colCountForEachCellView = position.ColumnSpan / (columnCount + 1);
                for (var i = 0; i < columnCount + 1; i++)
                {
                    var colIndex = position.Column + i * colCountForEachCellView;
                    var cell = CreateCell();
                    if (i == 0)
                    {
                        foreach (var block in oldCell.Blocks)
                        {
                            block.Parent = cell;
                            cell.AddBlock(block);
                        }
                    }
                    var cellPosition = new CellPostion(position.Row, colIndex, position.RowSpan, colCountForEachCellView);
                    newCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    newAddedCells.Add(new KeyValuePair<CellPostion, ICell>(cellPosition, cell));
                    _cells.Add(cellPosition, cell);
                }
            }

            SetCellBordersAfterSplitCell(newAddedCells);
            RaiseCellsChanged(newCells, removeCells);

            if (newAddedCells.Count > 0)
            {
                var newActiveCell = newAddedCells[0].Value;
                if (newActiveCell != null)
                {
                    var document = GetReportTemplateDocument();
                    document?.RaiseCurrentElementChanged(newActiveCell);
                }
            }
        }

        protected virtual RTCell CreateCell()
        {
            var cell = new RTCell(this);
            cell.Borders.Bottom.BorderStyle = RTBorderStyle.None;
            cell.Borders.Top.BorderStyle = RTBorderStyle.None;
            cell.Borders.Left.BorderStyle = RTBorderStyle.None;
            cell.Borders.Right.BorderStyle = RTBorderStyle.None;
            return cell;
        }

        private void OnRowDefinitionsCollectionChangedCallback(object sender, CollectionChangedArgs<RTRowDefinition> e)
        {
            RowDefinitionCollectionChanged?.Invoke(this, new CollectionChangedArgs<RTRowDefinition>(e.RemovedItems, e.AddedItems));
        }

        private void OnColumnDefinitionsCollectionChangedCallback(object sender, CollectionChangedArgs<RTColumnDefinition> e)
        {
            ColumnDefinitionCollectionChanged?.Invoke(this, new CollectionChangedArgs<RTColumnDefinition>(e.RemovedItems, e.AddedItems));
        }

        private void RaiseCellsChanged(IEnumerable<KeyValuePair<CellPostion, ICell>> addedCells, IEnumerable<KeyValuePair<CellPostion, ICell>> removedCells)
        {
            var args = new CellsChangedArgs(addedCells, removedCells);
            CellsChanged?.Invoke(this, args);
        }

        private void SetCellBordersAfterSplitCell(List<KeyValuePair<CellPostion, ICell>> addedCells)
        {
            foreach (var item in addedCells)
            {
                var cellPosition = item.Key;
                var cell = item.Value;
                var topCell = GetCell(cellPosition.Row - 1, cellPosition.Column);
                var leftCell = GetCell(cellPosition.Row, cellPosition.Column - 1);
                var rightCell = GetCell(cellPosition.Row, cellPosition.Column +cellPosition.ColumnSpan);
                var bottomCell = GetCell(cellPosition.Row+cellPosition.RowSpan, cellPosition.Column);
                if (topCell != null)
                {
                    topCell.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                }
                if (leftCell != null)
                {
                    cell.Borders.Left.BorderStyle = RTBorderStyle.None;
                }

                if (rightCell != null)
                {
                    rightCell.Borders.Left.BorderStyle = RTBorderStyle.None;
                }

                if (bottomCell != null)
                {
                    cell.Borders.Bottom.BorderStyle = RTBorderStyle.None;
                    bottomCell.Borders.Top.BorderStyle = RTBorderStyle.Solid;
                }
            }
        }

        private void SetCellBordersAfterCellMerged(KeyValuePair<CellPostion, ICell> newCell, List<KeyValuePair<CellPostion, ICell>> removedCells)
        {
            var cellPosition = newCell.Key;
            var cell = newCell.Value;
            var topCell = GetCellFromRemovedCells(cellPosition.Row, cellPosition.Column, removedCells);
            if (topCell != null)
            {
                CopyBorderStyle(topCell.Borders.Top, cell.Borders.Top);
                CopyBorderStyle(topCell.Borders.Left, cell.Borders.Left);
                cell.Background = topCell.Background;
            }
            var rightCell = GetCellFromRemovedCells(cellPosition.Row,
                cellPosition.Column + cellPosition.ColumnSpan - 1, removedCells);
            if (rightCell != null)
            {
                CopyBorderStyle(rightCell.Borders.Right, cell.Borders.Right);
            }
            var bottomCell = GetCellFromRemovedCells(cellPosition.Row + cellPosition.RowSpan - 1,
                cellPosition.Column,removedCells);
            if (bottomCell != null)
            {
                CopyBorderStyle(bottomCell.Borders.Bottom, cell.Borders.Bottom);
            }
        }

        private void CopyBorderStyle(RTBorder sourceBorder, RTBorder targetBorder)
        {
            targetBorder.Color = sourceBorder.Color;
            targetBorder.BorderStyle = sourceBorder.BorderStyle;
            targetBorder.Thickness = sourceBorder.Thickness;
        }

        private ICell GetCellFromRemovedCells(int rowIndex, int columnIndex, IEnumerable<KeyValuePair<CellPostion, ICell>> removedCells)
        {
            return removedCells.FirstOrDefault(c =>
                c.Key.Row <= rowIndex && c.Key.RowSpan + c.Key.Row - 1 >= rowIndex && c.Key.Column <= columnIndex &&
                c.Key.Column + c.Key.ColumnSpan - 1 >= columnIndex).Value;
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        /// <returns></returns>
        public override IElement Clone()
        {
            var grid = new RTGrid(null);
            InitializeCloneElement(grid);
            var cellDictionary= new Dictionary<CellPostion, ICell>();
            foreach (var cell in Cells)
            {
                var cloneCell = cell.Value.Clone() as RTCell;
                if (cloneCell != null)
                {
                    cloneCell.Parent = grid;
                    cellDictionary.Add(cell.Key.Clone(), cloneCell);
                }
            }
            grid.Cells = cellDictionary;
            grid.SetRowDefinitions(RowDefinitions.Select(x => new RTRowDefinition() {Height = x.Height,MinHeight=x.MinHeight,Type=x.Type}).ToList());
            grid.SetColumnDefinitions(ColumnDefinitions.Select(x => new RTColumnDefinition() { Width = x.Width }).ToList());
            return grid;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is RTGrid grid)
            {
                _rowDefinitions.Clear();
                SetRowDefinitions(grid.RowDefinitions.Select(x => new RTRowDefinition() { Height = x.Height, MinHeight = x.MinHeight, Type = x.Type }).ToList());
                _columnDefinitions.Clear();
                SetColumnDefinitions(grid.ColumnDefinitions.Select(x => new RTColumnDefinition() { Width = x.Width }).ToList());
                CellsClear?.Invoke(this, null);

                //var oldCellList = new Dictionary<CellPostion, ICell>();
                //foreach (var cell in Cells)
                //{
                //    var cloneCell = cell.Value.Clone() as RTCell;
                //    if (cloneCell != null)
                //    {
                //        cloneCell.Parent = grid;
                //        oldCellList.Add(cell.Key.Clone(), cloneCell);
                //    }
                //}

                var newCellList = new Dictionary<CellPostion, ICell>();
                foreach (var cell in grid.Cells)
                {
                    var cloneCell = cell.Value.Clone() as RTCell;
                    if (cloneCell != null)
                    {
                        cloneCell.Parent = grid;
                        newCellList.Add(cell.Key.Clone(), cloneCell);
                    }
                }

                Cells = grid.Cells;
                foreach(var cell in Cells)
                {
                    cell.Value.Parent = this;
                }
                RaiseCellsChanged(newCellList, new List<KeyValuePair<CellPostion, ICell>>());
                base.UpdateElementProperty(grid);
            }
        }
        /// <summary>
        /// Set table Average ColumnWidth
        /// </summary>
        public void SetAverageColumnWidth()
        {
            var list = new List<RTColumnDefinition>();
            var totalWidth = GetTotalWidth();
            var avergeWidth = totalWidth / ColumnDefinitions.Count;
            foreach(var col in _columnDefinitions)
            {
                col.Width = avergeWidth;
                list.Add(col);
            }
            ColumnWidthChanged?.Invoke(this, new CollectionChangedArgs<RTColumnDefinition>(new List<RTColumnDefinition>(), list));
        }

        /// <summary>
        /// Set table Cell Label Group Name
        /// </summary>
        public void SetCellLabelGroupName(bool val)
        {
            var newCellList = new List<KeyValuePair<CellPostion, ICell>>();
            var orderCells = _cells.OrderBy(c => c.Key.Row).ThenBy(c => c.Key.Column);
            int row = -1;
            int col = -1;
            var defaultTableName = "";
            foreach (var cellItem in orderCells)
            {
                var res = true;
                if (cellItem.Value.Blocks?.Count > 0)
                {
                    if (val)
                    {
                        foreach (var blockElement in cellItem.Value.Blocks)
                        {
                            if (blockElement is Paragraph paragraph)
                            {
                                foreach (var element in paragraph.Elements)
                                {
                                    if (element is StaticText staticText)
                                    {
                                        if (string.IsNullOrEmpty(defaultTableName) && cellItem.Key.ColumnSpan == _columnDefinitions.Count)
                                        {
                                            defaultTableName = ((StaticText)element).Text.Trim();
                                        }
                                        res = res & true;
                                    }
                                    else
                                    {
                                        res = res & false;
                                    }
                                }
                            }
                        }
                        if (res)
                        {
                            ((RTCell)cellItem.Value).GroupField = defaultTableName + "_" + "group_" + cellItem.Key.Row + "_" + cellItem.Key.Column;
                            row = cellItem.Key.Row;
                            col = cellItem.Key.Column;
                        }
                        else
                        {
                            if (row > -1 && col > -1)
                            {
                                ((RTCell)cellItem.Value).GroupField = defaultTableName + "_" + "group_" + row + "_" + col;
                            }
                        }
                    }
                    else
                    {
                        ((RTCell)cellItem.Value).GroupField = "";
                    }
                }
            }
            if (newCellList?.Count > 0)
            {
                RaiseCellsChanged(newCellList, new List<KeyValuePair<CellPostion, ICell>>());
            }
        }
    }
}