﻿using System;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Text;
using System.Xml;
using Vinno.IUS.Common.Log;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    public static class ReportDataContractSerializer
    {
        private static readonly DataContractSerializer Serializer;
        private static readonly XmlWriterSettings WriterSettings;

        static ReportDataContractSerializer()
        {
            var knownTypes = Assembly.GetExecutingAssembly().GetExportedTypes()
                .Where(x => !x.IsInterface && !x.IsGenericTypeDefinition)
                .Where(x => typeof(IReportElement).IsAssignableFrom(x) || typeof(IElementValue).IsAssignableFrom(x));
            Serializer = new DataContractSerializer(typeof(ReportInfo), "ReportInfo", "",
                knownTypes);
            WriterSettings = new XmlWriterSettings
            {
                Indent = true,
                NewLineHandling = NewLineHandling.Entitize,
                OmitXmlDeclaration = true
            };
        }

        /// <summary>
        ///     Deserialize report template document.
        /// </summary>
        /// <param name="jsonStr">The json string.</param>
        /// <returns>The <see cref="ReportInfo" /></returns>
        public static ReportInfo Deserialize(string jsonStr)
        {
            using (var stream = new MemoryStream(Encoding.UTF8.GetBytes(jsonStr)))
            {
                return DeSerialize(stream);
            }
        }

        /// <summary>
        ///     Deserialize report template document.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>The <see cref="ReportInfo" /></returns>
        public static ReportInfo DeSerialize(string filePath)
        {
            using (var stream = File.OpenRead(filePath))
            {
                return DeSerialize(stream);
            }
        }

        public static ReportInfo DeSerialize(Stream stream)
        {
            try
            {
                return (ReportInfo) Serializer.ReadObject(stream);
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize report info error {exception}");
                return null;
            }
        }

        public static string Serialize(ReportInfo reportInfo, bool needLine = true)
        {
            using (var stream = new MemoryStream())
            {
                Serialize(stream, reportInfo);
                return Encoding.UTF8.GetString(stream.ToArray());
            }
        }

        /// <summary>
        ///     Serialize report template document.
        /// </summary>
        /// <param name="filePath">The report template file path.</param>
        /// <param name="reportInfo">The report info to be serialized.</param>
        public static void Serialize(string filePath, ReportInfo reportInfo)
        {
            using (var stream = File.Create(filePath))
            {
                Serialize(stream, reportInfo);
            }
        }

        public static void Serialize(Stream stream, ReportInfo reportInfo)
        {
            using (var writer = XmlWriter.Create(stream, WriterSettings))
            {
                Serializer.WriteStartObject(writer, reportInfo);
                writer.WriteAttributeString("xmlns", "arrays", null,
                    "http://schemas.microsoft.com/2003/10/Serialization/Arrays");
                Serializer.WriteObjectContent(writer, reportInfo);
                Serializer.WriteEndObject(writer);
            }
        }
    }
}