﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common.Interfaces;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{

    /// <summary>
    /// Define the report structure.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    public class ReportInfo : IFileStorage
    {
        /// <summary>
        /// Gets or sets the report id.
        /// </summary>
        [DataMember]
        public string Id { get; set; }

        /// <summary>
        /// Gets or sets the report template.
        /// </summary>
        [DataMember]
        public IReportTemplateDocument Template { get; set; }

        /// <summary>
        /// Gets or sets the values.
        /// </summary>
        [DataMember]
        public Dictionary<ElementTag, IElementValue> ElementValues { get; set; }

        /// <summary>
        /// Gets or sets the values.
        /// </summary>
        [DataMember]
        public Dictionary<ElementTag, Dictionary<MeasureTag, TextElementValue>> MeasureElementValues { get; set; }

        /// <summary>
        /// Gets or sets the report preview image info.
        /// </summary>
        [DataMember]
        public List<ImageInfo> PreviewImages { get; set; }

        /// <summary>
        /// Gets or sets the tag list
        /// </summary>
        [DataMember]
        public List<string> CustomTagList { get; set; }

        /// <summary>
        ///  Gets or sets enum organs mirror
        /// </summary>
        [DataMember]
        public EnumOrgansMirror EnumOrgansMirror { get; set; }

        /// <summary>
        ///  Gets or sets enum organs mirror
        /// </summary>
        [DataMember]
        public DateTime UpdateTime { get; set; }


        /// <summary>
        /// 质控状态
        /// </summary>
        [DataMember]
        public int QualifiedState { get; set; }

        /// <summary>
        /// 质控类型
        /// </summary>
        [DataMember]
        public int QualityType { get; set; }

        /// <summary>
        /// Initialize a <see cref="ReportInfo"/>
        /// </summary>
        public ReportInfo()
        {
            Id = Guid.NewGuid().ToString();
            CustomTagList = new List<string>();
            ElementValues = new Dictionary<ElementTag, IElementValue>();
            MeasureElementValues = ElementTag.FetusTags
                .ToDictionary(x => x, x => new Dictionary<MeasureTag, TextElementValue>());
            PreviewImages = new List<ImageInfo>();
        }
        /// <summary>
        ///Get all fileImageValues
        /// </summary>
        public IEnumerable<FileImageValue> GetAllFileImageValues()
        {
            var fileImageValueList = ElementValues.Values.OfType<FileImageValue>().ToList();
            var imageListValues = ElementValues.Values.OfType<ImageListElementValue>();
            var fileImageListValues = imageListValues.SelectMany(v => v.Value).OfType<FileImageValue>();
            fileImageValueList.AddRange(fileImageListValues);
            return fileImageValueList;
        }

        /// <summary>
        /// ReportInfo Clone
        /// </summary>
        public ReportInfo Clone()
        {
            var reportInfo = new ReportInfo();
            reportInfo.Id = Id;
            if (Template != null)
            {
                reportInfo.Template = ((ReportTemplateDocument)Template).Clone();
            }

            foreach (var elementKeyValuePair in ElementValues)
            {
                reportInfo.ElementValues.Add(elementKeyValuePair.Key, elementKeyValuePair.Value.Clone());
            }
            reportInfo.CustomTagList = CustomTagList;
            reportInfo.EnumOrgansMirror = EnumOrgansMirror;
            reportInfo.MeasureElementValues = MeasureElementValues.ToDictionary(x => x.Key, x => Clone(x, x.Value));
            return reportInfo;
        }

        private static Dictionary<MeasureTag, TextElementValue> Clone(KeyValuePair<ElementTag, Dictionary<MeasureTag, TextElementValue>> pair, Dictionary<MeasureTag, TextElementValue> value)
        {
            return value.ToDictionary(x => x.Key, y => (TextElementValue)y.Value.Clone());
        }

        public void SimuMeasureTagValues(Func<string, string> factoryIdTranslator)
        {
            var elements = Template.GetTemplateMeasureElements();
            foreach (var element in elements)
            {
                var measureTag = element.MeasureTag;
                var descrption = measureTag.GetDescrption(factoryIdTranslator);
                var value = new TextElementValue(descrption);
                ElementTag.FetusTags.ForEach(x => MeasureElementValues[x][measureTag] = value);
                element.FontColor = new RTColor(0xff, 0x80, 0x80, 0x80);
                element.FontStyles.Clear();
                element.FontStyles.Add(RTFontStyle.Italic);
            }
        }


    }

    public class ConsultationReportInfo : ReportInfo
    {
        public ConsultationReportInfo() { }

        public ConsultationReportInfo(ReportInfo reportInfo)
        {
            Id = reportInfo.Id;
            Template = reportInfo.Template;
            CustomTagList = reportInfo.CustomTagList;
            QualifiedState = reportInfo.QualifiedState;
            QualityType = reportInfo.QualityType;
            foreach (var elementKeyValuePair in reportInfo.ElementValues)
            {
                ElementValues.Add(elementKeyValuePair.Key, elementKeyValuePair.Value);
            }
        }
    }
    public enum EnumOrgansMirror
    {
        Null = 0,            // 空
        NotSpecified = 1,    // 未指定
        Breast = 2,          // 乳腺
        Abdomen = 3,           // 腹部
        Liver = 4,        // 胆囊
        Cholecyst = 5        // 胆囊
    }
    public enum CustomTagEnumeration
    {
        Group,
        Lable
    }

    public class CustomTagTree
    {
        /// <summary>
        /// Name
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// Description
        /// </summary>
        public string Description { get; set; }

        /// <summary>
        /// Label Type
        /// </summary>
        public CustomTagEnumeration LabelType { get; set; }

        /// <summary>
        /// Children
        /// </summary>
        public List<CustomTagTree> Children { get; set; }

        /// <summary>
        /// CreateTime
        /// </summary>
        public DateTime CreateTime { get; set; }

        /// <summary>
        /// Is show
        /// </summary>
        public bool IsShow { get; set; }

    }
}