﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common;
using Vinno.IUS.Common.Args;
using Vinno.IUS.Common.Interfaces;
using Vinno.IUS.Common.Log;
using Vinno.IUS.Common.Utilities;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The report template document class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class ReportTemplateDocument : IReportTemplateDocument, IFileStorage
    {
        // private static readonly bool _isUbuntu = File.Exists("/etc/os-release") && File.ReadAllText("/etc/os-release").Contains("Ubuntu");
        private NotificationList<IBlockElement> _blocks;
        private NotificationList<IBlockElement> _headers;
        private NotificationList<IBlockElement> _footers;
        private Dictionary<string, Dictionary<string, string>> _customizeTranslation;
        private HashSet<string> _customizeComments;
        private HashSet<string> _customizeReportComments;
        private static string _fontName;
        private HashSet<CustomizeCalculation> _customizeCalculations;

        /// <summary>
        /// The event when new element added.
        /// </summary>
        public event EventHandler<IElement> CurrentElementChanged;

        /// <summary>
        /// The event occurs when page size changed.
        /// </summary>
        public event EventHandler PageSizeChanged;

        /// <summary>
        /// The event occurs when page headerheight or footerheight changed.
        /// </summary>
        public event EventHandler HeightChanged;
        public event EventHandler InvertColorChanged;

        /// <summary>
        /// The event occurs when blocks collection changes.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<IBlockElement>> BlocksCollectionChanged;

        /// <summary>
        /// The event occurs when headers collection changes.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<IBlockElement>> HeadersCollectionChanged;

        /// <summary>
        /// The event occurs when footers collection changes.
        /// </summary>
        public event EventHandler<CollectionChangedArgs<IBlockElement>> FootersCollectionChanged;

        /// <summary>
        /// Gets the id.
        /// </summary>
        [DataMember]
        public string Id { get; set; }

        /// <summary>
        /// Gets or sets the update time.
        /// </summary>
        [DataMember]
        public DateTime UpdateTime { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the report template is created by user.
        /// </summary>
        [DataMember]
        public bool IsCustom { get; set; }

        /// <summary>
        /// Gets the author of the report template.
        /// </summary>
        [DataMember]
        public string Author { get; set; }

        /// <summary>
        /// Gets the report template name.
        /// </summary>
        [DataMember]
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the page padding. Default is 20mm.
        /// </summary>
        [DataMember]
        public RTThickness PagePadding { get; set; }

        /// <summary>
        /// Gets or sets the page size.
        /// </summary>
        [DataMember]
        public RTPageSize PageSize { get; set; }
        /// <summary>
        /// Gets or sets the height of header.
        /// </summary>
        [DataMember]
        public double? HeaderHeight { get; set; }
        /// <summary>
        /// Gets or sets the height of footer.
        /// </summary>
        [DataMember]
        public double? FooterHeight { get; set; }

        /// <summary>
        /// get or set the distance between the header and the top of the page
        /// </summary>
        [DataMember]
        public double HeaderDistance { get; set; }

        /// <summary>
        /// get or set the distance between the footer and the bottom of the page
        /// </summary>
        [DataMember]
        public double FooterDistance { get; set; }

        /// <summary>
        /// Gets the custom input element tags of the report template.
        /// </summary>
        [DataMember]
        public Dictionary<string, ElementTag> Tags { get; private set; }

        /// <summary>
        /// key1=language, key2=keyword
        /// </summary>
        [DataMember]
        public Dictionary<string, Dictionary<string, string>> CustomizeTranslation
        {
            get
            {
                return _customizeTranslation ?? (_customizeTranslation = new[]
                {
                    "Chinese",
                    "English",
                    "German",
                    "Greek",
                    "Malay",
                    "Portuguese",
                    "Romanian",
                    "Spanish",
                    "Swedish",
                    "Norwegian",
                    "Danish",
                    "Finnish",
                    "French",
                    "Polish",
                    "Russian",
                    "Uighur",
                    "Italian",
                    "Czech",
                    "Khmer",
                    "Hungarian"
                }.ToDictionary(x => x, x => new Dictionary<string, string>()));
            }
            set => _customizeTranslation = value;
        }

        [DataMember]
        public HashSet<string> CustomizeComments
        {
            get => _customizeComments ?? (_customizeComments = new HashSet<string>());
            set => _customizeComments = value;
        }

        [DataMember]
        public HashSet<string> CustomizeReportComments
        {
            get => _customizeReportComments ?? (_customizeReportComments = new HashSet<string>());
            set => _customizeReportComments = value;
        }

        [DataMember]
        public HashSet<CustomizeCalculation> CustomizeCalculations
        {
            get => _customizeCalculations ?? (_customizeCalculations = new HashSet<CustomizeCalculation>());
            set => _customizeCalculations = value;
        }

        /// <summary>
        /// Gets all the report template blocks.
        /// </summary>
        public IReadOnlyList<IBlockElement> Blocks => _blocks.ToList();

        [JsonIgnore]
        [DataMember]
        private IEnumerable<IBlockElement> DataContractBlocks
        {
            get => _blocks;
            set => _blocks = new NotificationList<IBlockElement>(value, OnBlocksCollectionChanged);
        }

        /// <summary>
        /// Gets the report template header.
        /// </summary>
        public IReadOnlyList<IBlockElement> Header => _headers.ToList();

        [JsonIgnore]
        [DataMember]
        private IEnumerable<IBlockElement> DataContractHeader
        {
            get => _headers;
            set => _headers = new NotificationList<IBlockElement>(value, OnHeadersCollectionChanged);
        }

        /// <summary>
        /// Gets the report template footer.
        /// </summary>
        public IReadOnlyList<IBlockElement> Footer => _footers.ToList();

        [JsonIgnore]
        [DataMember]
        private IEnumerable<IBlockElement> DataContractFooter
        {
            get => _footers;
            set => _footers = new NotificationList<IBlockElement>(value, OnFootersCollectionChanged);
        }

        [DataMember]
        public double BaseFontSize { get; set; }

        [DataMember]
        public bool InvertColor { get; set; }

        [DataMember]
        public string Version { get; set; }

        private static string GetFontName()
        {
            return "Noto Sans SC";
        }

        /// <summary>
        /// Initialize a <see cref="ReportTemplateDocument"/>
        /// </summary>
        public ReportTemplateDocument()
        {
            _fontName = TranslateHelper.CurrentLanguage == TranslateHelper.ChineseLanguageKey ? GetFontName() : "Noto Sans SC";
            Id = Guid.NewGuid().ToString();
            PageSize = RTPageSize.A4;
            PagePadding = new RTThickness(56.83);
            HeaderHeight = FooterHeight = null;
            Tags = new Dictionary<string, ElementTag>();
            _blocks = new NotificationList<IBlockElement>(OnBlocksCollectionChanged);
            _headers = new NotificationList<IBlockElement>(OnHeadersCollectionChanged);
            _footers = new NotificationList<IBlockElement>(OnFootersCollectionChanged);
            BaseFontSize = 9;
            HeaderDistance = FooterDistance = 34;
        }

        /// <summary>
        /// Get all element tags
        /// </summary>
        public IEnumerable<ElementTag> GetAllTags()
        {
            var elements = GetAllElements();
            return elements.Select(e => e.Tag).Where(t => t != null);
        }
        /// <summary>
        /// Get all elements
        /// </summary>
        public IEnumerable<IElement> GetAllElements()
        {
            var elements = GetElementsFromBlocks(Header).ToList();
            elements.AddRange(GetElementsFromBlocks(Blocks));
            elements.AddRange(GetElementsFromBlocks(Footer));
            return elements;
        }

        /// <summary>
        /// Load report template document by stream.
        /// </summary>
        /// <param name="stream">The stream.</param>
        /// <param name="reportTemplateSerializer">The serializer.</param>
        /// <returns>The <see cref="IReportTemplateDocument"/></returns>
        public static IReportTemplateDocument Load(Stream stream, IReportTemplateSerializer reportTemplateSerializer)
        {
            return null;
        }

        /// <summary>
        /// Load report template document by file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <param name="reportTemplateSerializer">The serializer.</param>
        /// <returns>The <see cref="IReportTemplateDocument"/></returns>
        public static IReportTemplateDocument Load(byte[] jsonData)
        {
            using (var stream = new MemoryStream(jsonData))
                return RepportDeserializeHelper.DeserializeTemplate(stream);
        }

        /// <summary>
        /// Load report template document by file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <param name="reportTemplateSerializer">The serializer.</param>
        /// <returns>The <see cref="IReportTemplateDocument"/></returns>
        public static string Load(IReportTemplateDocument reportTemplateDocument)
        {
                return RepportDeserializeHelper.SerializeTemplate(reportTemplateDocument);
        }


        /// <summary>
        /// Load report template document by file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <param name="reportTemplateSerializer">The serializer.</param>
        /// <returns>The <see cref="IReportTemplateDocument"/></returns>
        public static IReportTemplateDocument Load(string filePath)
        {
            using (var stream = File.OpenRead(filePath))
                return RepportDeserializeHelper.DeserializeTemplate(stream);
        }

        /// <summary>
        /// Save report template document to a file.
        /// </summary>
        /// <param name="filePath">The target saved file path.</param>
        /// <param name="reportTemplateSerializer">The serializer.</param>
        public void Save(string filePath, IReportTemplateSerializer reportTemplateSerializer)
        {
            reportTemplateSerializer.Serialize(filePath, this);
        }

        /// <summary>
        /// Save report template document to stream.
        /// </summary>
        /// <param name="stream">The target stream.</param>
        /// <param name="reportTemplateSerializer">The serializer.</param>
        public void Save(Stream stream, IReportTemplateSerializer reportTemplateSerializer)
        {

        }

        /// <summary>
        /// Add a block.
        /// </summary>
        /// <param name="block">The new added block.</param>
        public void AddBlock(IBlockElement block)
        {
            var index = _blocks.Count;
            block.Index = index;
            _blocks.Add(block);
        }

        /// <summary>
        /// Remove a block.
        /// </summary>
        /// <param name="block">The removed block.</param>
        public void RemoveBlock(IBlockElement block)
        {
            if (_blocks.Contains(block))
            {
                _blocks.Remove(block);
            }

            var updatedBlocks = _blocks.Where(b => b.Index >= block.Index);
            foreach (var updatedBlock in updatedBlocks)
            {
                updatedBlock.Index = updatedBlock.Index - 1;
            }
        }

        /// <summary>
        /// Insert block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        public void InsertBlock(IBlockElement block, int index)
        {
            try
            {
                var updatedBlocks = _blocks.Where(b => b.Index >= index);
                foreach (var updatedBlock in updatedBlocks)
                {
                    updatedBlock.Index = updatedBlock.Index + 1;
                }

                block.Index = index;
                _blocks.Insert(index, block);
            }
            catch (Exception e)
            {
                Logger.WriteLineError($"Insert block error {e}");
            }
        }

        /// <summary>
        /// Add a block for footer.
        /// </summary>
        /// <param name="block">The new added block.</param>
        public void AddFooterBlock(IBlockElement block)
        {
            var index = _footers.Count;
            block.Index = index;
            _footers.Add(block);
        }

        /// <summary>
        /// Remove a block from footer.
        /// </summary>
        /// <param name="block">The removed block.</param>
        public void RemoveFooterBlock(IBlockElement block)
        {
            if (_footers.Contains(block))
            {
                _footers.Remove(block);
            }

            var updatedBlocks = _footers.Where(b => b.Index >= block.Index);
            foreach (var updatedBlock in updatedBlocks)
            {
                updatedBlock.Index = updatedBlock.Index - 1;
            }
        }

        /// <summary>
        /// Insert footer block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        public void InsertFooterBlock(IBlockElement block, int index)
        {
            try
            {
                var updatedBlocks = _footers.Where(b => b.Index >= index);
                foreach (var updatedBlock in updatedBlocks)
                {
                    updatedBlock.Index = updatedBlock.Index + 1;
                }

                block.Index = index;
                _footers.Insert(index, block);
            }
            catch (Exception e)
            {
                Logger.WriteLineError($"Insert footer block error {e}");
            }
        }

        /// <summary>
        /// Get page width
        /// </summary>
        public double GetPageClientWidth()
        {
            return PageSize.Width - PagePadding.Left - PagePadding.Right;
        }

        /// <summary>
        /// Get page client height assuming <see cref="Header"/>/<see cref="Footer"/> both having visible content/>
        /// </summary>
        /// <returns></returns>
        public double GetPageClientHeight()
        {
            var headerHeight = HeaderHeight ?? 0;
            var topDistance = Math.Max(PagePadding.Top, headerHeight + HeaderDistance);
            var footerHeight = FooterHeight ?? 0;
            var bottomDistance = Math.Max(PagePadding.Bottom, footerHeight + FooterDistance);
            return PageSize.Height - topDistance - bottomDistance;
        }

        /// <summary>
        /// Add a block for header.
        /// </summary>
        /// <param name="block">The new added block.</param>
        public void AddHeaderBlock(IBlockElement block)
        {
            var index = _headers.Count;
            block.Index = index;
            _headers.Add(block);
        }

        /// <summary>
        /// Remove a block from header.
        /// </summary>
        /// <param name="block">The removed block.</param>
        public void RemoveHeaderBlock(IBlockElement block)
        {
            if (_headers.Contains(block))
            {
                _headers.Remove(block);
            }

            var updatedBlocks = _headers.Where(b => b.Index >= block.Index);
            foreach (var updatedBlock in updatedBlocks)
            {
                updatedBlock.Index = updatedBlock.Index - 1;
            }
        }

        /// <summary>
        /// Insert header block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        public void InsertHeaderBlock(IBlockElement block, int index)
        {
            try
            {
                var updatedBlocks = _headers.Where(b => b.Index >= index);
                foreach (var updatedBlock in updatedBlocks)
                {
                    updatedBlock.Index = updatedBlock.Index + 1;
                }

                block.Index = index;
                _headers.Insert(index, block);
            }
            catch (Exception e)
            {
                Logger.WriteLineError($"Insert header block error {e}");
            }
        }

        /// <summary>
        /// Add a input tag.
        /// </summary>
        /// <param name="tag">The new added input tag.</param>
        public void AddTag(ElementTag tag)
        {
            if (Tags.ContainsKey(tag.Id))
            {
                return;
            }

            Tags.Add(tag.Id, tag);
        }

        /// <summary>
        /// Remove a input tag.
        /// </summary>
        /// <param name="tag">The removed input tag.</param>
        public void RemoveTag(ElementTag tag)
        {
            if (Tags.ContainsKey(tag.Id))
            {
                Tags.Remove(tag.Id);
            }
        }

        /// <summary>
        /// Create report template element.
        /// </summary>
        /// <typeparam name="T">The report template element type.</typeparam>
        /// <param name="parent">The elment parent.</param>
        /// <returns>The report template element.</returns>
        public static T CreateElement<T>(IReportElement parent) where T : Element
        {
            if (typeof(T) == typeof(Space))
            {
                var space = new Space((IElement)parent);
                return space as T;
            }

            if (typeof(T) == typeof(Line))
            {
                var line = new Line((IElement)parent);
                return line as T;
            }

            if (typeof(T) == typeof(StaticImage))
            {
                var staticImage = new StaticImage((IElement)parent);
                return staticImage as T;
            }

            if (typeof(T) == typeof(InputImage))
            {
                var inputImage = new InputImage((IElement)parent);
                return inputImage as T;
            }

            if (typeof(T) == typeof(InputImageList))
            {
                var inputImageList = new InputImageList(parent);
                return inputImageList as T;
            }
            if (typeof(T) == typeof(AtlasElement))
            {
                var alasElement = new AtlasElement(parent);
                return alasElement as T;
            }
            if (typeof(T) == typeof(StaticText))
            {
                var staticText = new StaticText((IElement)parent);
                staticText.FontName = _fontName;
                return staticText as T;
            }

            if (typeof(T) == typeof(InputText))
            {
                var inputText = new InputText((IElement)parent);
                inputText.FontName = _fontName;
                return inputText as T;
            }

            if (typeof(T) == typeof(SingleSelected))
            {
                var singleSelected = new SingleSelected((IElement)parent);
                singleSelected.FontName = _fontName;
                return singleSelected as T;
            }
            if (typeof(T) == typeof(SingleSelectedVertical))
            {
                var singleSelected = new SingleSelectedVertical((IElement)parent);
                singleSelected.FontName = _fontName;
                return singleSelected as T;
            }
            if (typeof(T) == typeof(MultiSelected))
            {
                var multiSelected = new MultiSelected((IElement)parent);
                multiSelected.FontName = _fontName;
                return multiSelected as T;
            }
            if (typeof(T) == typeof(MultiSelectedVertical))
            {
                var multiSelected = new MultiSelectedVertical((IElement)parent);
                multiSelected.FontName = _fontName;
                return multiSelected as T;
            }

            if (typeof(T) == typeof(Paragraph))
            {
                var paragraph = new Paragraph(parent);
                return paragraph as T;
            }

            if (typeof(T) == typeof(RTGrid))
            {
                var rtGrid = new RTGrid(parent);
                return rtGrid as T;
            }

            if (typeof(T) == typeof(RTCell))
            {
                var rtCell = new RTCell((IGrid)parent);
                return rtCell as T;
            }

            if (typeof(T) == typeof(RTTable))
            {
                var rtTable = new RTTable(parent);
                return rtTable as T;
            }

            if (typeof(T) == typeof(DateTimeElement))
            {
                var dateTimeElement = new DateTimeElement((IElement)parent);
                dateTimeElement.FontName = _fontName;
                return dateTimeElement as T;
            }

            if (typeof(T) == typeof(PageNumberElement))
            {
                var pageNumberElement = new PageNumberElement((IElement)parent);
                pageNumberElement.FontName = _fontName;
                return pageNumberElement as T;
            }

            return default(T);
        }

        /// <summary>
        /// Raise current element changed event.
        /// </summary>
        public void RaiseCurrentElementChanged(IElement element)
        {
            CurrentElementChanged?.Invoke(this, element);
        }

        /// <summary>
        /// Raise page size changed event.
        /// </summary>
        public void RaisePageSizeChanged()
        {
            PageSizeChanged?.Invoke(this, EventArgs.Empty);
        }

        /// <summary>
        ///  Raise height changed event.
        /// </summary>
        public void RaiseHeightChanged()
        {
            HeightChanged?.Invoke(this, EventArgs.Empty);
        }
        /// <summary>
        ///  Raise InvertColor changed event.
        /// </summary>
        public void RaiseInvertColorChanged()
        {
            InvertColorChanged?.Invoke(this, EventArgs.Empty);
        }

        /// <summary>
        ///  ReportTemplateDocument Clone
        /// </summary>
        public ReportTemplateDocument Clone()
        {
            var reportTemplateDocument = new ReportTemplateDocument();
            reportTemplateDocument.Id = Id;
            reportTemplateDocument.Author = Author;
            reportTemplateDocument.Name = Name;
            reportTemplateDocument.IsCustom = IsCustom;
            reportTemplateDocument.UpdateTime = UpdateTime;
            reportTemplateDocument.PagePadding = new RTThickness(PagePadding.Left, PagePadding.Top, PagePadding.Right,
                PagePadding.Bottom);
            reportTemplateDocument.PageSize = PageSize;
            reportTemplateDocument.BaseFontSize = BaseFontSize;
            reportTemplateDocument.HeaderHeight = HeaderHeight;
            reportTemplateDocument.FooterHeight = FooterHeight;
            reportTemplateDocument.InvertColor = InvertColor;
            reportTemplateDocument.HeaderDistance = HeaderDistance;
            reportTemplateDocument.FooterDistance = FooterDistance;
            foreach (var tag in Tags)
            {
                reportTemplateDocument.Tags.Add(tag.Key, tag.Value);
            }

            foreach (var pair1 in CustomizeTranslation)
            {
                var dictionary =
                    reportTemplateDocument.CustomizeTranslation[pair1.Key] = new Dictionary<string, string>();
                foreach (var pair2 in pair1.Value)
                    dictionary[pair2.Key] = pair2.Value;
            }

            reportTemplateDocument.CustomizeComments = new HashSet<string>(CustomizeComments);
            reportTemplateDocument.CustomizeReportComments = new HashSet<string>(CustomizeReportComments);
            var collection = CustomizeCalculations.Select(x => new CustomizeCalculation
            {
                Formula = x.Formula,
                Id = x.Id,
                Unit = x.Unit
            });
            reportTemplateDocument.CustomizeCalculations = new HashSet<CustomizeCalculation>(collection);
            foreach (var block in Blocks)
            {
                var cloneBlock = block.Clone() as IBlockElement;
                if (cloneBlock != null)
                {
                    cloneBlock.Parent = reportTemplateDocument;
                    reportTemplateDocument.AddBlock(cloneBlock);
                }
            }

            foreach (var header in Header)
            {
                var cloneBlock = header.Clone() as IBlockElement;
                if (cloneBlock != null)
                {
                    cloneBlock.Parent = reportTemplateDocument;
                    reportTemplateDocument.AddHeaderBlock(cloneBlock);
                }
            }

            foreach (var footer in Footer)
            {
                var cloneBlock = footer.Clone() as IBlockElement;
                if (cloneBlock != null)
                {
                    cloneBlock.Parent = reportTemplateDocument;
                    reportTemplateDocument.AddFooterBlock(cloneBlock);
                }
            }

            return reportTemplateDocument;
        }

        /// <summary>
        ///  ReportTemplateDocument Clone for UndoRedo
        /// </summary>
        public ReportTemplateDocument CloneForUndoRedo()
        {
            var reportTemplateDocument = new ReportTemplateDocument();
            reportTemplateDocument.PagePadding = new RTThickness(PagePadding.Left, PagePadding.Top, PagePadding.Right,
                PagePadding.Bottom);
            reportTemplateDocument.PageSize = PageSize;
            reportTemplateDocument.BaseFontSize = BaseFontSize;
            reportTemplateDocument.HeaderHeight = HeaderHeight;
            reportTemplateDocument.FooterHeight = FooterHeight;
            reportTemplateDocument.InvertColor = InvertColor;
            return reportTemplateDocument;
        }

        private void OnFootersCollectionChanged(object sender, CollectionChangedArgs<IBlockElement> e)
        {
            FootersCollectionChanged?.Invoke(this, new CollectionChangedArgs<IBlockElement>(e.RemovedItems, e.AddedItems));
        }

        private void OnHeadersCollectionChanged(object sender, CollectionChangedArgs<IBlockElement> e)
        {
            HeadersCollectionChanged?.Invoke(this, new CollectionChangedArgs<IBlockElement>(e.RemovedItems, e.AddedItems));
        }

        private void OnBlocksCollectionChanged(object sender, CollectionChangedArgs<IBlockElement> e)
        {
            BlocksCollectionChanged?.Invoke(this, new CollectionChangedArgs<IBlockElement>(e.RemovedItems, e.AddedItems));
        }

        private IEnumerable<IElement> GetElementsFromBlocks(IEnumerable<IBlockElement> blockElements)
        {
            var elements = new List<IElement>();
            foreach (var block in blockElements)
            {
                if (block is Paragraph paragraph)
                {
                    elements.AddRange(paragraph.Elements);
                }

                if (block is RTGrid rtGrid)
                {
                    elements.AddRange(rtGrid.Elements);
                }

                if (block is InputImageList || block is AtlasElement)
                {
                    elements.Add(block);
                }
            }

            return elements;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public void UpdateElementProperty(IReportTemplateDocument reportTemplateDocument)
        {
            PagePadding.Left = reportTemplateDocument.PagePadding.Left;
            PagePadding.Right = reportTemplateDocument.PagePadding.Right;
            PagePadding.Bottom = reportTemplateDocument.PagePadding.Bottom;
            PagePadding.Top = reportTemplateDocument.PagePadding.Top;
            PageSize = reportTemplateDocument.PageSize;
            BaseFontSize = reportTemplateDocument.BaseFontSize;
            HeaderHeight = reportTemplateDocument.HeaderHeight;
            FooterHeight = reportTemplateDocument.FooterHeight;
            InvertColor = reportTemplateDocument.InvertColor;

            RaisePageSizeChanged();
            RaiseHeightChanged();
            RaiseInvertColorChanged();
        }
    }
}