﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Vinno.IUS.Common.Utilities;
using Vinno.IUS.Common.Log;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.JsonConverters;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The report template serializer.
    /// </summary>
    public class ReportTemplateJsonSerializer : IReportTemplateSerializer
    {
        /// <summary>
        /// Serialize report template document.
        /// </summary>
        /// <param name="filePath">The report template file path.</param>
        /// <param name="reportTemplateDocument">The report template document to be serialized.</param>
        public void Serialize(string filePath, IReportTemplateDocument reportTemplateDocument)
        {
            reportTemplateDocument.Version = ReportTemplateVersion.GetReportTemplateVersion();
            var jsonString = GetSerializeJson(reportTemplateDocument);
            var data = new UTF8Encoding().GetBytes(jsonString);
            FileHelper.SaveFile(filePath, data);
        }

        /// <summary>
        ///Get json that  serialize report template document.
        /// </summary>
        /// <param name="reportTemplateDocument"></param>
        /// <returns></returns>
        public string GetSerializeJson(IReportTemplateDocument reportTemplateDocument)
        {
            reportTemplateDocument.Version = ReportTemplateVersion.GetReportTemplateVersion();
            var settings = new JsonSerializerSettings
            {
                ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                Converters =
                    new List<JsonConverter>
                    {
                        new StringEnumConverter(),
                        new ReadOnlyCellDictionaryJsonConverter()
                    }
            };
            var jsonString = JsonConvert.SerializeObject(reportTemplateDocument, Formatting.Indented, settings);
            return jsonString;
        }

        /// <summary>
        /// Deserialize report template document.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>The <see cref="IReportTemplateDocument"/></returns>
        public IReportTemplateDocument DeSerialize(string filePath)
        {
            using (var stream = File.OpenRead(filePath))
            {
                return DeSerialize(stream);
            }
        }

        /// <summary>
        /// Deserialize report template document.
        /// </summary>
        /// <param name="stream">The file stream.</param>
        /// <returns>The <see cref="IReportTemplateDocument"/></returns>
        public IReportTemplateDocument DeSerialize(Stream stream)
        {
            try
            {
                var settings = new JsonSerializerSettings
                {
                    ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                    Converters =
                        new List<JsonConverter> { new ReportTemplateDocumentJsonConverter() }
                };
                return (IReportTemplateDocument)JsonSerializer.Create(settings)
                    .Deserialize(new JsonTextReader(new StreamReader(stream)), typeof(ReportTemplateDocument));
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize report template document error {exception}");
                return null;
            }
        }

        public IReportTemplateDocument DeSerializeJson(string json)
        {
            try
            {
                if (IsValidJson(json))
                {
                    var settings = new JsonSerializerSettings
                    {
                        ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                        Converters =
                            new List<JsonConverter> { new ReportTemplateDocumentJsonConverter() }
                    };
                    return JsonConvert.DeserializeObject<ReportTemplateDocument>(json, settings);
                }

                return null;
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize report template document error {exception}");
                return null;
            }
        }

        private bool IsValidJson(string value)
        {
            return value.StartsWith("{") && value.EndsWith("}") ||
                   value.StartsWith("[") && value.EndsWith("]");
        }
    }
}