﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The text element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public abstract class TextElement:InlineElement, ITextElement
    {
        private RTColor _fontColor;
        private List<RTFontStyle> _fontStyles;

        public event EventHandler OnFontColorChanged;
        public event EventHandler OnFontStylesChanged;

        /// <summary>
        /// Initialize a <see cref="TextElement"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        protected TextElement(IElement parent) : base(parent)
        {
            LineLength = 5;
            FontSize = 10.5;
            FontName = "宋体";
            FontStyles = new List<RTFontStyle>();
            _fontColor = RTColor.Black;
            Background = RTColor.White;
            VerticalAlignment = VerticalLayout.Center;
        }

        /// <summary>
        /// Gets or sets the font name.
        /// </summary>
        [DataMember]
        public string FontName { get; set; }

        /// <summary>
        /// Gets or sets the font size.
        /// </summary>
        [DataMember]
        public double FontSize { get; set; }

        /// <summary>
        /// Gets or sets the font styles.
        /// </summary>
        [DataMember]
        public List<RTFontStyle> FontStyles
        {
            get => _fontStyles;
            set
            {
                if (_fontStyles != value)
                {
                    _fontStyles = value;
                    OnFontStylesChanged?.Invoke(_fontStyles, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets or sets the font color.
        /// </summary>
        [DataMember]
        public RTColor FontColor
        {
            get => _fontColor;
            set
            {
                if (!_fontColor.Equals(value))
                {
                    _fontColor = value;
                    OnFontColorChanged?.Invoke(_fontColor, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets or sets the background.
        /// </summary>
        [DataMember]
        public RTColor Background { get; set; }

        /// <summary>
        /// Gets or sets the lenght of the characters
        /// </summary>
        [DataMember]
        public int LineLength { get; set; }

        [DataMember]
        public double LineWidth { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the text can wrap.
        /// </summary>
        [DataMember]
        public bool TextWrap { get; set; }

        protected void InitializeCloneTextElement(ITextElement element)
        {
            InitializeCloneElement(element);
            element.FontName = FontName;
            element.FontSize = FontSize;
            element.FontStyles = FontStyles.ToList();
            element.FontColor = FontColor;
            element.Background = Background;
            element.LineLength = LineLength;
            element.LineWidth = LineWidth;
            element.TextWrap = TextWrap;
        }

        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is TextElement textElement)
            {
                FontName = textElement.FontName;
                FontSize = textElement.FontSize;
                FontStyles = textElement.FontStyles;
                FontColor = textElement.FontColor;
                Background = textElement.Background;
                LineLength = textElement.LineLength;
                LineWidth = textElement.LineWidth;
                TextWrap = textElement.TextWrap;
                base.UpdateElementProperty(textElement);
            }
        }
    }
}